#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "rodent.h"
#include "timer.h"

void ReadLine(char *str, int n) {
  char *ptr;

  if (fgets(str, n, stdin) == NULL)
    exit(0);
  if ((ptr = strchr(str, '\n')) != NULL)
    *ptr = '\0';
}

char *ParseToken(char *string, char *token) {

  while (*string == ' ')
    string++;
  while (*string != ' ' && *string != '\0')
    *token++ = *string++;
  *token = '\0';
  return string;
}

void UciLoop(void) {

  char command[4096], token[80], *ptr;
  POS p[1];

  setbuf(stdin, NULL);
  setbuf(stdout, NULL);
  SetPosition(p, START_POS);
  AllocTrans(16);
  for (;;) {
    ReadLine(command, sizeof(command));
    ptr = ParseToken(command, token);
    if (strcmp(token, "uci") == 0) {
      printf("id name Fox 1.3\n");
      printf("id author Lucas Monge (based in MiniRodent by Pawel Koziol)\n");
      printf("option name Hash type spin default 16 min 1 max 4096\n");
      printf("option name Clear Hash type button\n");
      printf("option name UCI_LimitStrength type check default false\n");
      printf("option name UCI_Elo type spin default 1500 min 1000 max 2700\n");
      // printf("option name Cub Weight type spin default 0 min 0 max 10000\n");
      // printf("option name MiniRodent Weight type spin default 0 min 0 max 10000\n");
      printf("uciok\n");
    } else if (strcmp(token, "isready") == 0) {
      printf("readyok\n");
    } else if (strcmp(token, "setoption") == 0) {
      ParseSetoption(ptr);
    } else if (strcmp(token, "position") == 0) {
      ParsePosition(p, ptr);
    } else if (strcmp(token, "perft") == 0) {
      ptr = ParseToken(ptr, token);
      int depth = atoi(token);
      if (depth == 0) depth = 5;
      Timer.SetStartTime();
      nodes = Perft(p, 0, depth);
      printf (" perft %d : %d nodes in %d miliseconds\n", depth, nodes, Timer.GetElapsedTime() );
    } else if (strcmp(token, "print") == 0) {
      PrintBoard(p);
    } else if (strcmp(token, "step") == 0) {
      ParseMoves(p, ptr);
    } else if (strcmp(token, "go") == 0) {
      ParseGo(p, ptr);
    } else if (strcmp(token, "bench") == 0) {
      ptr = ParseToken(ptr, token);
      Bench(atoi(token));
    } else if (strcmp(token, "quit") == 0) {
      exit(0);
    }
  }
}

void ParseSetoption(char *ptr) {

  char token[80], name[80], value[80] = "";
  int weight;

  ptr = ParseToken(ptr, token);
  name[0] = '\0';
  for (;;) {
    ptr = ParseToken(ptr, token);
    if (*token == '\0' || strcmp(token, "value") == 0)
      break;
    strcat(name, token);
    strcat(name, " ");
  }
  name[strlen(name) - 1] = '\0';
  if (strcmp(token, "value") == 0) {
    value[0] = '\0';

    for (;;) {
      ptr = ParseToken(ptr, token);
      if (*token == '\0')
        break;
      strcat(value, token);
      strcat(value, " ");
    }
    value[strlen(value) - 1] = '\0';
  }

  if (strcmp(name, "Hash") == 0) {
    AllocTrans(atoi(value));
  } else if (strcmp(name, "Clear Hash") == 0) {
    ResetEngine();
  } else if (strcmp(name, "Attack") == 0) {
    weights[F_ATT] = atoi(value);
    ResetEngine();
  } else if (strcmp(name, "Mobility") == 0) {
    weights[F_MOB] = atoi(value);
    ResetEngine();
  } else if (strcmp(name, "PassedPawns") == 0) {
    weights[F_PASSERS] = atoi(value);
    ResetEngine();
  } else if (strcmp(name, "PawnStructure") == 0) {
    weights[F_PAWNS] = atoi(value);
    ResetEngine();
  } else if (strcmp(name, "Cub Weight") == 0) {
    weight = atoi(value);
    if(weight >= 0 && weight <= 10000) {
      cub_weight = weight;
      if (cub_weight + mr_weight > 10000) mr_weight = 10000 - cub_weight;
      fox_weight = 10000 - cub_weight - mr_weight;
      ResetEngine();
      show_weights();
    }
  } else if (strcmp(name, "MiniRodent Weight") == 0) {
    weight = atoi(value);
    if(weight >= 0 && weight <= 10000) {
      mr_weight = weight;
      if (cub_weight + mr_weight > 10000) cub_weight = 10000 - mr_weight;
      fox_weight = 10000 - cub_weight - mr_weight;
      ResetEngine();
      show_weights();
    }
  } else if (strcmp(name, "UCI_LimitStrength") == 0) {
    uci_limitstrength = value == "true";
    calc_limit_elo();
  } else if (strcmp(name, "UCI_Elo") == 0) {
    uci_elo = atoi(value);
    calc_limit_elo();
  }
}

int ParseMoves(POS *p, char *ptr) {

  char token[80];
  UNDO u[1];
  int num_moves = 0;
  int move;

  for (;;) {

    // Get next move to parse

    ptr = ParseToken(ptr, token);

    // No more moves!

    if (*token == '\0') break;

    move = StrToMove(p, token);
    p->DoMove(move, u);
    last_sq = Tsq(move);

    num_moves++;

    // We won't be taking back moves beyond this point:

    if (p->rev_moves == 0) p->head = 0;
  }
  return num_moves;
}

void ParsePosition(POS *p, char *ptr) {

  char token[80], fen[80];
  bool start_position = false;
  int num_moves = 0;

  ptr = ParseToken(ptr, token);
  if (strcmp(token, "fen") == 0) {
    fen[0] = '\0';
    for (;;) {
      ptr = ParseToken(ptr, token);

      if (*token == '\0' || strcmp(token, "moves") == 0)
        break;

      strcat(fen, token);
      strcat(fen, " ");
    }
    SetPosition(p, fen);
  } else {
    ptr = ParseToken(ptr, token);
    SetPosition(p, START_POS);
    last_sq = 28;   // e4
    start_position = true;
  }

  if (strcmp(token, "moves") == 0)
    num_moves = ParseMoves(p, ptr);

  if( start_position && num_moves < 3) use_book = true;

}

void ParseGo(POS *p, char *ptr) {

  char token[80], bestmove_str[6], ponder_str[6];
  int pv[MAX_PLY];

  Timer.Clear();
  pondering = 0;

  for (;;) {
    ptr = ParseToken(ptr, token);
    if (*token == '\0')
      break;
    if (strcmp(token, "ponder") == 0) {
      pondering = 1;
    } else if (strcmp(token, "wtime") == 0) {
      ptr = ParseToken(ptr, token);
      Timer.SetData(W_TIME, atoi(token));
    } else if (strcmp(token, "btime") == 0) {
      ptr = ParseToken(ptr, token);
      Timer.SetData(B_TIME, atoi(token));
    } else if (strcmp(token, "winc") == 0) {
      ptr = ParseToken(ptr, token);
      Timer.SetData(W_INC, atoi(token));
    } else if (strcmp(token, "binc") == 0) {
      ptr = ParseToken(ptr, token);
      Timer.SetData(B_INC, atoi(token));
    } else if (strcmp(token, "movestogo") == 0) {
      ptr = ParseToken(ptr, token);
      Timer.SetData(MOVES_TO_GO, atoi(token));
    } else if (strcmp(token, "movetime") == 0) {
      ptr = ParseToken(ptr, token);
      Timer.SetData(MOVE_TIME, atoi(token));
    } else if (strcmp(token, "depth") == 0) {
      ptr = ParseToken(ptr, token);
      Timer.SetData(FLAG_INFINITE, 1);
      Timer.SetData(MAX_DEPTH, atoi(token));
    } else if (strcmp(token, "infinite") == 0) {
      Timer.SetData(FLAG_INFINITE, 1);
    } else if (strcmp(token, "nodes") == 0) {
      ptr = ParseToken(ptr, token);
      Timer.SetData(MAX_NODES, atoi(token));
    }
  }

  Timer.SetSideData(p->side);
  Timer.SetMoveTiming();
  Think(p, pv);
  MoveToStr(pv[0], bestmove_str);
  if (pv[1]) {
    MoveToStr(pv[1], ponder_str);
    printf("bestmove %s ponder %s\n", bestmove_str, ponder_str);
  } else
    printf("bestmove %s\n", bestmove_str);
}

void ResetEngine(void) {

  ClearHist();
  ClearTrans();
  ClearEvalHash();
}
