/*
    Texel - A UCI chess engine.
    Copyright (C) 2012,2014  Peter Österlund, peterosterlund2@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * kpkTable.cpp
 *
 *  Created on: Feb 26, 2012
 *      Author: petero
 */

#include "endGameEval.hpp"

const U8 EndGameEval::kpkTable[2*32*64*48/8] = {
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xc2, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x85, 0x80, 0xff, 0xff, 0xff, 0xff,
  0x0a, 0x00, 0xff, 0xff, 0xff, 0xff, 0x16, 0x02, 0xff, 0xff, 0xff, 0xff,
  0x2e, 0x06, 0xff, 0xff, 0xff, 0xff, 0x5e, 0x0e, 0xff, 0xff, 0xff, 0xff,
  0xaf, 0x0f, 0xff, 0xff, 0xff, 0xff, 0x5f, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0xc0, 0xc2, 0xe0, 0xff, 0xff, 0xff, 0x80, 0x85, 0xc0, 0xff, 0xff, 0xff,
  0x00, 0x0a, 0x80, 0xff, 0xff, 0xff, 0x02, 0x14, 0x00, 0xff, 0xff, 0xff,
  0x06, 0x2a, 0x00, 0xff, 0xff, 0xff, 0x0e, 0x56, 0x03, 0xff, 0xff, 0xff,
  0x0f, 0xaf, 0x07, 0xff, 0xff, 0xff, 0x1f, 0x5f, 0x0f, 0xff, 0xff, 0xff,
  0xc0, 0xc0, 0xe2, 0xf0, 0xff, 0xff, 0x80, 0x80, 0xc5, 0xe0, 0xff, 0xff,
  0x00, 0x00, 0x8a, 0xc0, 0xff, 0xff, 0x02, 0x00, 0x14, 0x80, 0xff, 0xff,
  0x06, 0x02, 0x28, 0x01, 0xff, 0xff, 0x0e, 0x06, 0x53, 0x03, 0xff, 0xff,
  0x0f, 0x0f, 0xa7, 0x07, 0xff, 0xff, 0x1f, 0x1f, 0x4f, 0x0f, 0xff, 0xff,
  0xc0, 0xc0, 0xe0, 0xf2, 0xf8, 0xff, 0x80, 0x80, 0xc0, 0xe5, 0xf0, 0xff,
  0x00, 0x00, 0x80, 0xca, 0xe0, 0xff, 0x02, 0x00, 0x00, 0x94, 0xc1, 0xff,
  0x06, 0x02, 0x00, 0x29, 0x83, 0xff, 0x0e, 0x06, 0x03, 0x53, 0x07, 0xff,
  0x0f, 0x0f, 0x07, 0xa7, 0x0f, 0xff, 0x1f, 0x1f, 0x0f, 0x4f, 0x1f, 0xff,
  0xc0, 0xc0, 0xe0, 0xf0, 0xfa, 0xfc, 0x80, 0x80, 0xc0, 0xe0, 0xf5, 0xf8,
  0x00, 0x00, 0x80, 0xc0, 0xea, 0xf1, 0x02, 0x00, 0x00, 0x80, 0xd5, 0xe3,
  0x06, 0x02, 0x00, 0x01, 0xab, 0xc7, 0x0e, 0x06, 0x03, 0x03, 0x57, 0x8f,
  0x0f, 0x0f, 0x07, 0x07, 0xaf, 0x1f, 0x1f, 0x1f, 0x0f, 0x0f, 0x5f, 0x3f,
  0xfe, 0xc0, 0xe0, 0xf0, 0xf8, 0xfe, 0xfe, 0x80, 0xc0, 0xe0, 0xf0, 0xfd,
  0x1e, 0x00, 0x80, 0xc0, 0xe0, 0xfb, 0x1e, 0x00, 0x00, 0x80, 0xc1, 0xf7,
  0x1e, 0x02, 0x00, 0x01, 0x83, 0xef, 0x1e, 0x06, 0x03, 0x03, 0x07, 0xdf,
  0x1f, 0x0f, 0x07, 0x07, 0x0f, 0xbf, 0x1f, 0x1f, 0x0f, 0x0f, 0x1f, 0x7f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfa, 0xf4, 0xff, 0xff, 0xff, 0xff, 0x85, 0x80, 0xff, 0xff, 0xff, 0xff,
  0x0a, 0x00, 0xff, 0xff, 0xff, 0xff, 0x16, 0x02, 0xff, 0xff, 0xff, 0xff,
  0x2e, 0x06, 0xff, 0xff, 0xff, 0xff, 0x5e, 0x0e, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0x1f, 0xff, 0xff, 0xff, 0xff, 0x5f, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf2, 0xe0, 0xff, 0xff, 0xff, 0x80, 0x85, 0xc0, 0xff, 0xff, 0xff,
  0x00, 0x0a, 0x80, 0xff, 0xff, 0xff, 0x02, 0x14, 0x00, 0xff, 0xff, 0xff,
  0x06, 0x2a, 0x00, 0xff, 0xff, 0xff, 0x0e, 0x56, 0x03, 0xff, 0xff, 0xff,
  0x1f, 0xaf, 0x07, 0xff, 0xff, 0xff, 0x1f, 0x5f, 0x0f, 0xff, 0xff, 0xff,
  0xf8, 0xf0, 0xe2, 0xf0, 0xff, 0xff, 0x80, 0x80, 0xc5, 0xe0, 0xff, 0xff,
  0x00, 0x00, 0x8a, 0xc0, 0xff, 0xff, 0x02, 0x00, 0x14, 0x80, 0xff, 0xff,
  0x06, 0x02, 0x28, 0x01, 0xff, 0xff, 0x0e, 0x06, 0x53, 0x03, 0xff, 0xff,
  0x1f, 0x0f, 0xa7, 0x07, 0xff, 0xff, 0x1f, 0x1f, 0x4f, 0x0f, 0xff, 0xff,
  0xf8, 0xf0, 0xe0, 0xf2, 0xf8, 0xff, 0x80, 0x80, 0xc0, 0xe5, 0xf0, 0xff,
  0x00, 0x00, 0x80, 0xca, 0xe0, 0xff, 0x02, 0x00, 0x00, 0x94, 0xc1, 0xff,
  0x06, 0x02, 0x00, 0x29, 0x83, 0xff, 0x0e, 0x06, 0x03, 0x53, 0x07, 0xff,
  0x1f, 0x0f, 0x07, 0xa7, 0x0f, 0xff, 0x1f, 0x1f, 0x0f, 0x4f, 0x1f, 0xff,
  0xf8, 0xf0, 0xe0, 0xf0, 0xfa, 0xfc, 0x80, 0x80, 0xc0, 0xe0, 0xf5, 0xf8,
  0x00, 0x00, 0x80, 0xc0, 0xea, 0xf1, 0x02, 0x00, 0x00, 0x80, 0xd5, 0xe3,
  0x06, 0x02, 0x00, 0x01, 0xab, 0xc7, 0x0e, 0x06, 0x03, 0x03, 0x57, 0x8f,
  0x1f, 0x0f, 0x07, 0x07, 0xaf, 0x1f, 0x1f, 0x1f, 0x0f, 0x0f, 0x5f, 0x3f,
  0xfe, 0xf0, 0xe0, 0xf0, 0xf8, 0xfe, 0xfe, 0x80, 0xc0, 0xe0, 0xf0, 0xfd,
  0x7e, 0x00, 0x80, 0xc0, 0xe0, 0xfb, 0x3e, 0x00, 0x00, 0x80, 0xc1, 0xf7,
  0x3e, 0x02, 0x00, 0x01, 0x83, 0xef, 0x3e, 0x06, 0x03, 0x03, 0x07, 0xdf,
  0x3f, 0x0f, 0x07, 0x07, 0x0f, 0xbf, 0x3f, 0x1f, 0x0f, 0x0f, 0x1f, 0x7f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xe8, 0xff, 0xff, 0xff, 0xff,
  0x0a, 0x00, 0xff, 0xff, 0xff, 0xff, 0x14, 0x02, 0xff, 0xff, 0xff, 0xff,
  0x2e, 0x06, 0xff, 0xff, 0xff, 0xff, 0x5e, 0x0e, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0x1f, 0xff, 0xff, 0xff, 0xff, 0x7f, 0x3f, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfa, 0xf0, 0xff, 0xff, 0xff, 0xf0, 0xe5, 0xc0, 0xff, 0xff, 0xff,
  0x00, 0x0a, 0x80, 0xff, 0xff, 0xff, 0x00, 0x14, 0x00, 0xff, 0xff, 0xff,
  0x06, 0x2a, 0x00, 0xff, 0xff, 0xff, 0x0e, 0x56, 0x03, 0xff, 0xff, 0xff,
  0x1f, 0xaf, 0x07, 0xff, 0xff, 0xff, 0x3f, 0x5f, 0x0f, 0xff, 0xff, 0xff,
  0xfc, 0xf8, 0xf2, 0xf0, 0xff, 0xff, 0xf0, 0xe0, 0xc5, 0xe0, 0xff, 0xff,
  0x00, 0x00, 0x8a, 0xc0, 0xff, 0xff, 0x00, 0x00, 0x14, 0x80, 0xff, 0xff,
  0x06, 0x02, 0x28, 0x01, 0xff, 0xff, 0x0e, 0x06, 0x53, 0x03, 0xff, 0xff,
  0x1f, 0x0f, 0xa7, 0x07, 0xff, 0xff, 0x3f, 0x1f, 0x4f, 0x0f, 0xff, 0xff,
  0xfc, 0xf8, 0xf0, 0xf2, 0xf8, 0xff, 0xf0, 0xe0, 0xc0, 0xe5, 0xf0, 0xff,
  0x00, 0x00, 0x80, 0xca, 0xe0, 0xff, 0x00, 0x00, 0x00, 0x94, 0xc1, 0xff,
  0x06, 0x02, 0x00, 0x29, 0x83, 0xff, 0x0e, 0x06, 0x03, 0x53, 0x07, 0xff,
  0x1f, 0x0f, 0x07, 0xa7, 0x0f, 0xff, 0x3f, 0x1f, 0x0f, 0x4f, 0x1f, 0xff,
  0xfc, 0xf8, 0xf0, 0xf0, 0xfa, 0xfc, 0xf0, 0xe0, 0xc0, 0xe0, 0xf5, 0xf8,
  0x00, 0x00, 0x80, 0xc0, 0xea, 0xf1, 0x00, 0x00, 0x00, 0x80, 0xd5, 0xe3,
  0x06, 0x02, 0x00, 0x01, 0xab, 0xc7, 0x0e, 0x06, 0x03, 0x03, 0x57, 0x8f,
  0x1f, 0x0f, 0x07, 0x07, 0xaf, 0x1f, 0x3f, 0x1f, 0x0f, 0x0f, 0x5f, 0x3f,
  0xfe, 0xf8, 0xf0, 0xf0, 0xf8, 0xfe, 0xfe, 0xe0, 0xc0, 0xe0, 0xf0, 0xfd,
  0x7e, 0x00, 0x80, 0xc0, 0xe0, 0xfb, 0x7e, 0x00, 0x00, 0x80, 0xc1, 0xf7,
  0x7e, 0x02, 0x00, 0x01, 0x83, 0xef, 0x7e, 0x06, 0x03, 0x03, 0x07, 0xdf,
  0x7f, 0x0f, 0x07, 0x07, 0x0f, 0xbf, 0x7f, 0x1f, 0x0f, 0x0f, 0x1f, 0x7f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0x6a, 0x50, 0xff, 0xff, 0xff, 0xff, 0x14, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x2a, 0x04, 0xff, 0xff, 0xff, 0xff, 0x5e, 0x0e, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0x1f, 0xff, 0xff, 0xff, 0xff, 0x7f, 0x3f, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfa, 0xf0, 0xff, 0xff, 0xff, 0xf8, 0xf5, 0xe0, 0xff, 0xff, 0xff,
  0x60, 0x4a, 0x80, 0xff, 0xff, 0xff, 0x00, 0x14, 0x00, 0xff, 0xff, 0xff,
  0x02, 0x28, 0x00, 0xff, 0xff, 0xff, 0x0e, 0x56, 0x03, 0xff, 0xff, 0xff,
  0x1f, 0xaf, 0x07, 0xff, 0xff, 0xff, 0x3f, 0x5f, 0x0f, 0xff, 0xff, 0xff,
  0xfc, 0xf8, 0xf2, 0xf0, 0xff, 0xff, 0xf8, 0xf0, 0xe5, 0xe0, 0xff, 0xff,
  0x60, 0x40, 0x8a, 0xc0, 0xff, 0xff, 0x00, 0x00, 0x14, 0x80, 0xff, 0xff,
  0x02, 0x00, 0x28, 0x01, 0xff, 0xff, 0x0e, 0x06, 0x53, 0x03, 0xff, 0xff,
  0x1f, 0x0f, 0xa7, 0x07, 0xff, 0xff, 0x3f, 0x1f, 0x4f, 0x0f, 0xff, 0xff,
  0xfc, 0xf8, 0xf0, 0xf2, 0xf8, 0xff, 0xf8, 0xf0, 0xe0, 0xe5, 0xf0, 0xff,
  0x60, 0x40, 0x80, 0xca, 0xe0, 0xff, 0x00, 0x00, 0x00, 0x94, 0xc1, 0xff,
  0x02, 0x00, 0x00, 0x29, 0x83, 0xff, 0x0e, 0x06, 0x03, 0x53, 0x07, 0xff,
  0x1f, 0x0f, 0x07, 0xa7, 0x0f, 0xff, 0x3f, 0x1f, 0x0f, 0x4f, 0x1f, 0xff,
  0xfc, 0xf8, 0xf0, 0xf0, 0xfa, 0xfc, 0xf8, 0xf0, 0xe0, 0xe0, 0xf5, 0xf8,
  0x60, 0x40, 0x80, 0xc0, 0xea, 0xf1, 0x00, 0x00, 0x00, 0x80, 0xd5, 0xe3,
  0x02, 0x00, 0x00, 0x01, 0xab, 0xc7, 0x0e, 0x06, 0x03, 0x03, 0x57, 0x8f,
  0x1f, 0x0f, 0x07, 0x07, 0xaf, 0x1f, 0x3f, 0x1f, 0x0f, 0x0f, 0x5f, 0x3f,
  0xfe, 0xf8, 0xf0, 0xf0, 0xf8, 0xfe, 0xfe, 0xf0, 0xe0, 0xe0, 0xf0, 0xfd,
  0x7e, 0x40, 0x80, 0xc0, 0xe0, 0xfb, 0x7e, 0x00, 0x00, 0x80, 0xc1, 0xf7,
  0x7e, 0x00, 0x00, 0x01, 0x83, 0xef, 0x7e, 0x06, 0x03, 0x03, 0x07, 0xdf,
  0x7f, 0x0f, 0x07, 0x07, 0x0f, 0xbf, 0x7f, 0x1f, 0x0f, 0x0f, 0x1f, 0x7f,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xee, 0xff, 0xff, 0xff, 0xff, 0xff, 0xde, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xbe, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7e, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0x0a, 0x02, 0xff, 0xff, 0xff, 0xff, 0x16, 0x07, 0xff, 0xff, 0xff, 0xff,
  0x2e, 0x07, 0xff, 0xff, 0xff, 0xff, 0x5e, 0x0f, 0xff, 0xff, 0xff, 0xff,
  0xae, 0x0f, 0xff, 0xff, 0xff, 0xff, 0x5e, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0xc0, 0xc2, 0xe0, 0xff, 0xff, 0xff, 0x80, 0x85, 0xc0, 0xff, 0xff, 0xff,
  0x02, 0x0a, 0x80, 0xff, 0xff, 0xff, 0x06, 0x16, 0x02, 0xff, 0xff, 0xff,
  0x06, 0x2e, 0x06, 0xff, 0xff, 0xff, 0x0e, 0x5e, 0x0f, 0xff, 0xff, 0xff,
  0x0e, 0xaf, 0x0f, 0xff, 0xff, 0xff, 0x1e, 0x5f, 0x1f, 0xff, 0xff, 0xff,
  0xfe, 0xc0, 0xe2, 0xf0, 0xff, 0xff, 0xfe, 0x80, 0xc5, 0xe0, 0xff, 0xff,
  0x02, 0x00, 0x8a, 0xc0, 0xff, 0xff, 0x06, 0x02, 0x14, 0x80, 0xff, 0xff,
  0x06, 0x06, 0x2a, 0x01, 0xff, 0xff, 0x0e, 0x0e, 0x57, 0x03, 0xff, 0xff,
  0x0e, 0x0f, 0xaf, 0x07, 0xff, 0xff, 0x1e, 0x1f, 0x5f, 0x0f, 0xff, 0xff,
  0xc0, 0xc0, 0xe0, 0xf2, 0xf8, 0xff, 0x80, 0x80, 0xc0, 0xe5, 0xf0, 0xff,
  0x02, 0x00, 0x80, 0xca, 0xe0, 0xff, 0x06, 0x02, 0x00, 0x94, 0xc1, 0xff,
  0x06, 0x06, 0x02, 0x29, 0x83, 0xff, 0x0e, 0x0e, 0x07, 0x53, 0x07, 0xff,
  0x0e, 0x0f, 0x0f, 0xa7, 0x0f, 0xff, 0x1e, 0x1f, 0x1f, 0x4f, 0x1f, 0xff,
  0xfe, 0xc0, 0xe0, 0xf0, 0xfa, 0xfc, 0xfe, 0x80, 0xc0, 0xe0, 0xf5, 0xf8,
  0x0e, 0x00, 0x80, 0xc0, 0xea, 0xf1, 0x0e, 0x02, 0x00, 0x80, 0xd5, 0xe3,
  0x0e, 0x06, 0x02, 0x01, 0xab, 0xc7, 0x0e, 0x0e, 0x07, 0x03, 0x57, 0x8f,
  0x0e, 0x0f, 0x0f, 0x07, 0xaf, 0x1f, 0x1e, 0x1f, 0x1f, 0x0f, 0x5f, 0x3f,
  0xfe, 0xc0, 0xe0, 0xf0, 0xf8, 0xfe, 0xfe, 0x80, 0xc0, 0xe0, 0xf0, 0xfd,
  0x1e, 0x00, 0x80, 0xc0, 0xe0, 0xfb, 0x1e, 0x02, 0x00, 0x80, 0xc1, 0xf7,
  0x1e, 0x06, 0x02, 0x01, 0x83, 0xef, 0x1e, 0x0e, 0x07, 0x03, 0x07, 0xdf,
  0x1e, 0x0f, 0x0f, 0x07, 0x0f, 0xbf, 0x1e, 0x1f, 0x1f, 0x0f, 0x1f, 0x7f,
  0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xed, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xbd, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7d, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xfb, 0xff, 0xff, 0xff, 0xff, 0x15, 0x07, 0xff, 0xff, 0xff, 0xff,
  0x2d, 0x0f, 0xff, 0xff, 0xff, 0xff, 0x5d, 0x0f, 0xff, 0xff, 0xff, 0xff,
  0xbd, 0x1f, 0xff, 0xff, 0xff, 0xff, 0x5d, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfa, 0xf4, 0xff, 0xff, 0xff, 0x80, 0x85, 0xc0, 0xff, 0xff, 0xff,
  0x00, 0x0a, 0x80, 0xff, 0xff, 0xff, 0x04, 0x16, 0x02, 0xff, 0xff, 0xff,
  0x0c, 0x2e, 0x06, 0xff, 0xff, 0xff, 0x0c, 0x5e, 0x0f, 0xff, 0xff, 0xff,
  0x1d, 0xbf, 0x1f, 0xff, 0xff, 0xff, 0x1d, 0x5f, 0x1f, 0xff, 0xff, 0xff,
  0xfc, 0xf8, 0xf2, 0xf0, 0xff, 0xff, 0xfc, 0x80, 0xc5, 0xe0, 0xff, 0xff,
  0x7c, 0x00, 0x8a, 0xc0, 0xff, 0xff, 0x04, 0x02, 0x14, 0x80, 0xff, 0xff,
  0x0c, 0x06, 0x2a, 0x01, 0xff, 0xff, 0x0c, 0x0e, 0x57, 0x03, 0xff, 0xff,
  0x1d, 0x1f, 0xaf, 0x07, 0xff, 0xff, 0x1d, 0x1f, 0x5f, 0x0f, 0xff, 0xff,
  0xfc, 0xf8, 0xf0, 0xf2, 0xf8, 0xff, 0x80, 0x80, 0xc0, 0xe5, 0xf0, 0xff,
  0x00, 0x00, 0x80, 0xca, 0xe0, 0xff, 0x04, 0x02, 0x00, 0x94, 0xc1, 0xff,
  0x0c, 0x06, 0x02, 0x29, 0x83, 0xff, 0x0c, 0x0e, 0x07, 0x53, 0x07, 0xff,
  0x1d, 0x1f, 0x0f, 0xa7, 0x0f, 0xff, 0x1d, 0x1f, 0x1f, 0x4f, 0x1f, 0xff,
  0xfc, 0xf8, 0xf0, 0xf0, 0xfa, 0xfc, 0xfc, 0x80, 0xc0, 0xe0, 0xf5, 0xf8,
  0x7c, 0x00, 0x80, 0xc0, 0xea, 0xf1, 0x1c, 0x02, 0x00, 0x80, 0xd5, 0xe3,
  0x1c, 0x06, 0x02, 0x01, 0xab, 0xc7, 0x1c, 0x0e, 0x07, 0x03, 0x57, 0x8f,
  0x1d, 0x1f, 0x0f, 0x07, 0xaf, 0x1f, 0x1d, 0x1f, 0x1f, 0x0f, 0x5f, 0x3f,
  0xfc, 0xf8, 0xf0, 0xf0, 0xf8, 0xfe, 0xfc, 0x80, 0xc0, 0xe0, 0xf0, 0xfd,
  0x7c, 0x00, 0x80, 0xc0, 0xe0, 0xfb, 0x3c, 0x02, 0x00, 0x80, 0xc1, 0xf7,
  0x3c, 0x06, 0x02, 0x01, 0x83, 0xef, 0x3c, 0x0e, 0x07, 0x03, 0x07, 0xdf,
  0x3d, 0x1f, 0x0f, 0x07, 0x0f, 0xbf, 0x3d, 0x1f, 0x1f, 0x0f, 0x1f, 0x7f,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xeb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdb, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xbb, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7b, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfa, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0x2b, 0x0f, 0xff, 0xff, 0xff, 0xff, 0x5b, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0xbb, 0x1f, 0xff, 0xff, 0xff, 0xff, 0x7b, 0x3f, 0xff, 0xff, 0xff, 0xff,
  0xfa, 0xfe, 0xfc, 0xff, 0xff, 0xff, 0xf8, 0xf5, 0xe8, 0xff, 0xff, 0xff,
  0x00, 0x0a, 0x80, 0xff, 0xff, 0xff, 0x02, 0x14, 0x02, 0xff, 0xff, 0xff,
  0x0a, 0x2e, 0x06, 0xff, 0xff, 0xff, 0x1a, 0x5e, 0x0f, 0xff, 0xff, 0xff,
  0x1b, 0xbf, 0x1f, 0xff, 0xff, 0xff, 0x3b, 0x7f, 0x3f, 0xff, 0xff, 0xff,
  0xfa, 0xfc, 0xfa, 0xf0, 0xff, 0xff, 0xfa, 0xf0, 0xe5, 0xe0, 0xff, 0xff,
  0x7a, 0x00, 0x8a, 0xc0, 0xff, 0xff, 0x7a, 0x00, 0x14, 0x80, 0xff, 0xff,
  0x0a, 0x06, 0x2a, 0x01, 0xff, 0xff, 0x1a, 0x0e, 0x57, 0x03, 0xff, 0xff,
  0x1b, 0x1f, 0xaf, 0x07, 0xff, 0xff, 0x3b, 0x3f, 0x5f, 0x0f, 0xff, 0xff,
  0xfa, 0xfc, 0xf8, 0xf2, 0xf8, 0xff, 0xf8, 0xf0, 0xe0, 0xe5, 0xf0, 0xff,
  0x00, 0x00, 0x80, 0xca, 0xe0, 0xff, 0x02, 0x00, 0x00, 0x94, 0xc1, 0xff,
  0x0a, 0x06, 0x02, 0x29, 0x83, 0xff, 0x1a, 0x0e, 0x07, 0x53, 0x07, 0xff,
  0x1b, 0x1f, 0x0f, 0xa7, 0x0f, 0xff, 0x3b, 0x3f, 0x1f, 0x4f, 0x1f, 0xff,
  0xfa, 0xfc, 0xf8, 0xf0, 0xfa, 0xfc, 0xfa, 0xf0, 0xe0, 0xe0, 0xf5, 0xf8,
  0x7a, 0x00, 0x80, 0xc0, 0xea, 0xf1, 0x7a, 0x00, 0x00, 0x80, 0xd5, 0xe3,
  0x3a, 0x06, 0x02, 0x01, 0xab, 0xc7, 0x3a, 0x0e, 0x07, 0x03, 0x57, 0x8f,
  0x3b, 0x1f, 0x0f, 0x07, 0xaf, 0x1f, 0x3b, 0x3f, 0x1f, 0x0f, 0x5f, 0x3f,
  0xfa, 0xfc, 0xf8, 0xf0, 0xf8, 0xfe, 0xfa, 0xf0, 0xe0, 0xe0, 0xf0, 0xfd,
  0x7a, 0x00, 0x80, 0xc0, 0xe0, 0xfb, 0x7a, 0x00, 0x00, 0x80, 0xc1, 0xf7,
  0x7a, 0x06, 0x02, 0x01, 0x83, 0xef, 0x7a, 0x0e, 0x07, 0x03, 0x07, 0xdf,
  0x7b, 0x1f, 0x0f, 0x07, 0x0f, 0xbf, 0x7b, 0x3f, 0x1f, 0x0f, 0x1f, 0x7f,
  0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf3, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xe7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xb7, 0xff, 0xff, 0xff, 0xff, 0xff, 0x77, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf6, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xf7, 0xef, 0xff, 0xff, 0xff, 0xff, 0x57, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0xb7, 0x3f, 0xff, 0xff, 0xff, 0xff, 0x77, 0x3f, 0xff, 0xff, 0xff, 0xff,
  0xf6, 0xfe, 0xfc, 0xff, 0xff, 0xff, 0xf4, 0xfd, 0xf8, 0xff, 0xff, 0xff,
  0x70, 0x6a, 0xd0, 0xff, 0xff, 0xff, 0x00, 0x14, 0x00, 0xff, 0xff, 0xff,
  0x06, 0x2a, 0x04, 0xff, 0xff, 0xff, 0x16, 0x5e, 0x0f, 0xff, 0xff, 0xff,
  0x37, 0xbf, 0x1f, 0xff, 0xff, 0xff, 0x37, 0x7f, 0x3f, 0xff, 0xff, 0xff,
  0xf6, 0xfc, 0xfa, 0xf0, 0xff, 0xff, 0xf4, 0xf8, 0xf5, 0xe0, 0xff, 0xff,
  0x76, 0x60, 0xca, 0xc0, 0xff, 0xff, 0x76, 0x00, 0x14, 0x80, 0xff, 0xff,
  0x76, 0x02, 0x28, 0x01, 0xff, 0xff, 0x16, 0x0e, 0x57, 0x03, 0xff, 0xff,
  0x37, 0x1f, 0xaf, 0x07, 0xff, 0xff, 0x37, 0x3f, 0x5f, 0x0f, 0xff, 0xff,
  0xf6, 0xfc, 0xf8, 0xf2, 0xf8, 0xff, 0xf4, 0xf8, 0xf0, 0xe5, 0xf0, 0xff,
  0x70, 0x60, 0xc0, 0xca, 0xe0, 0xff, 0x00, 0x00, 0x00, 0x94, 0xc1, 0xff,
  0x06, 0x02, 0x00, 0x29, 0x83, 0xff, 0x16, 0x0e, 0x07, 0x53, 0x07, 0xff,
  0x37, 0x1f, 0x0f, 0xa7, 0x0f, 0xff, 0x37, 0x3f, 0x1f, 0x4f, 0x1f, 0xff,
  0xf6, 0xfc, 0xf8, 0xf0, 0xfa, 0xfc, 0xf6, 0xf8, 0xf0, 0xe0, 0xf5, 0xf8,
  0x76, 0x60, 0xc0, 0xc0, 0xea, 0xf1, 0x76, 0x00, 0x00, 0x80, 0xd5, 0xe3,
  0x76, 0x02, 0x00, 0x01, 0xab, 0xc7, 0x76, 0x0e, 0x07, 0x03, 0x57, 0x8f,
  0x77, 0x1f, 0x0f, 0x07, 0xaf, 0x1f, 0x77, 0x3f, 0x1f, 0x0f, 0x5f, 0x3f,
  0xf6, 0xfc, 0xf8, 0xf0, 0xf8, 0xfe, 0xf6, 0xf8, 0xf0, 0xe0, 0xf0, 0xfd,
  0x76, 0x60, 0xc0, 0xc0, 0xe0, 0xfb, 0x76, 0x00, 0x00, 0x80, 0xc1, 0xf7,
  0x76, 0x02, 0x00, 0x01, 0x83, 0xef, 0x76, 0x0e, 0x07, 0x03, 0x07, 0xdf,
  0x77, 0x1f, 0x0f, 0x07, 0x0f, 0xbf, 0x77, 0x3f, 0x1f, 0x0f, 0x1f, 0x7f,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xfe, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0x0a, 0x02, 0xff, 0xff, 0xff, 0xff, 0x17, 0x06, 0xff, 0xff, 0xff, 0xff,
  0x2f, 0x06, 0xff, 0xff, 0xff, 0xff, 0x5f, 0x0e, 0xff, 0xff, 0xff, 0xff,
  0xaf, 0x0e, 0xff, 0xff, 0xff, 0xff, 0x5f, 0x1e, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff,
  0x02, 0x0a, 0x82, 0xff, 0xff, 0xff, 0x07, 0x16, 0x07, 0xff, 0xff, 0xff,
  0x07, 0x2e, 0x07, 0xff, 0xff, 0xff, 0x0f, 0x5e, 0x0f, 0xff, 0xff, 0xff,
  0x0f, 0xae, 0x0f, 0xff, 0xff, 0xff, 0x1f, 0x5e, 0x1f, 0xff, 0xff, 0xff,
  0xc0, 0xc0, 0xe2, 0xf0, 0xff, 0xff, 0xfe, 0x80, 0xc5, 0xe0, 0xff, 0xff,
  0x02, 0x02, 0x8a, 0xc0, 0xff, 0xff, 0x06, 0x06, 0x16, 0x82, 0xff, 0xff,
  0x06, 0x06, 0x2e, 0x07, 0xff, 0xff, 0x0e, 0x0e, 0x5f, 0x0f, 0xff, 0xff,
  0x0f, 0x0e, 0xaf, 0x0f, 0xff, 0xff, 0x1f, 0x1e, 0x5f, 0x1f, 0xff, 0xff,
  0xfe, 0xfe, 0xe0, 0xf2, 0xf8, 0xff, 0xfe, 0xfe, 0xc0, 0xe5, 0xf0, 0xff,
  0x06, 0x02, 0x80, 0xca, 0xe0, 0xff, 0x06, 0x06, 0x02, 0x94, 0xc1, 0xff,
  0x06, 0x06, 0x06, 0x2b, 0x83, 0xff, 0x0e, 0x0e, 0x0f, 0x57, 0x07, 0xff,
  0x0f, 0x0e, 0x0f, 0xaf, 0x0f, 0xff, 0x1f, 0x1e, 0x1f, 0x5f, 0x1f, 0xff,
  0xfe, 0xc0, 0xe0, 0xf0, 0xfa, 0xfc, 0xfe, 0x80, 0xc0, 0xe0, 0xf5, 0xf8,
  0x0e, 0x02, 0x80, 0xc0, 0xea, 0xf1, 0x0e, 0x06, 0x02, 0x80, 0xd5, 0xe3,
  0x0e, 0x06, 0x06, 0x03, 0xab, 0xc7, 0x0e, 0x0e, 0x0f, 0x07, 0x57, 0x8f,
  0x0f, 0x0e, 0x0f, 0x0f, 0xaf, 0x1f, 0x1f, 0x1e, 0x1f, 0x1f, 0x5f, 0x3f,
  0xfe, 0xfe, 0xe0, 0xf0, 0xf8, 0xfe, 0xfe, 0xfe, 0xc0, 0xe0, 0xf0, 0xfd,
  0x1e, 0x0e, 0x80, 0xc0, 0xe0, 0xfb, 0x1e, 0x0e, 0x02, 0x80, 0xc1, 0xf7,
  0x1e, 0x0e, 0x06, 0x03, 0x83, 0xef, 0x1e, 0x0e, 0x0f, 0x07, 0x07, 0xdf,
  0x1f, 0x0e, 0x0f, 0x0f, 0x0f, 0xbf, 0x1f, 0x1e, 0x1f, 0x1f, 0x1f, 0x7f,
  0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xfd, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xf9, 0xff, 0xff, 0xff, 0xff, 0x17, 0x05, 0xff, 0xff, 0xff, 0xff,
  0x2f, 0x0d, 0xff, 0xff, 0xff, 0xff, 0x5f, 0x0d, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0x1d, 0xff, 0xff, 0xff, 0xff, 0x5f, 0x1d, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xfd, 0xfb, 0xff, 0xff, 0xff, 0x07, 0x15, 0x07, 0xff, 0xff, 0xff,
  0x0f, 0x2d, 0x0f, 0xff, 0xff, 0xff, 0x0f, 0x5d, 0x0f, 0xff, 0xff, 0xff,
  0x1f, 0xbd, 0x1f, 0xff, 0xff, 0xff, 0x1f, 0x5d, 0x1f, 0xff, 0xff, 0xff,
  0xfe, 0xfc, 0xfa, 0xf4, 0xff, 0xff, 0x80, 0x80, 0xc5, 0xe0, 0xff, 0xff,
  0x7e, 0x00, 0x8a, 0xc0, 0xff, 0xff, 0x06, 0x04, 0x16, 0x82, 0xff, 0xff,
  0x0e, 0x0c, 0x2e, 0x07, 0xff, 0xff, 0x0e, 0x0c, 0x5f, 0x0f, 0xff, 0xff,
  0x1f, 0x1d, 0xbf, 0x1f, 0xff, 0xff, 0x1f, 0x1d, 0x5f, 0x1f, 0xff, 0xff,
  0xfe, 0xfc, 0xf8, 0xf2, 0xf8, 0xff, 0xfe, 0xfc, 0xc0, 0xe5, 0xf0, 0xff,
  0x7e, 0x7c, 0x80, 0xca, 0xe0, 0xff, 0x0e, 0x04, 0x02, 0x94, 0xc1, 0xff,
  0x0e, 0x0c, 0x06, 0x2b, 0x83, 0xff, 0x0e, 0x0c, 0x0f, 0x57, 0x07, 0xff,
  0x1f, 0x1d, 0x1f, 0xaf, 0x0f, 0xff, 0x1f, 0x1d, 0x1f, 0x5f, 0x1f, 0xff,
  0xfe, 0xfc, 0xf8, 0xf0, 0xfa, 0xfc, 0xfe, 0x80, 0xc0, 0xe0, 0xf5, 0xf8,
  0x7e, 0x00, 0x80, 0xc0, 0xea, 0xf1, 0x1e, 0x04, 0x02, 0x80, 0xd5, 0xe3,
  0x1e, 0x0c, 0x06, 0x03, 0xab, 0xc7, 0x1e, 0x0c, 0x0f, 0x07, 0x57, 0x8f,
  0x1f, 0x1d, 0x1f, 0x0f, 0xaf, 0x1f, 0x1f, 0x1d, 0x1f, 0x1f, 0x5f, 0x3f,
  0xfe, 0xfc, 0xf8, 0xf0, 0xf8, 0xfe, 0xfe, 0xfc, 0xc0, 0xe0, 0xf0, 0xfd,
  0x7e, 0x7c, 0x80, 0xc0, 0xe0, 0xfb, 0x3e, 0x1c, 0x02, 0x80, 0xc1, 0xf7,
  0x3e, 0x1c, 0x06, 0x03, 0x83, 0xef, 0x3e, 0x1c, 0x0f, 0x07, 0x07, 0xdf,
  0x3f, 0x1d, 0x1f, 0x0f, 0x0f, 0xbf, 0x3f, 0x1d, 0x1f, 0x1f, 0x1f, 0x7f,
  0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xfb, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf9, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xf3, 0xff, 0xff, 0xff, 0xff,
  0x2f, 0x0b, 0xff, 0xff, 0xff, 0xff, 0x5f, 0x1b, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0x1b, 0xff, 0xff, 0xff, 0xff, 0x7f, 0x3b, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfa, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff,
  0x0f, 0x2b, 0x0f, 0xff, 0xff, 0xff, 0x1f, 0x5b, 0x1f, 0xff, 0xff, 0xff,
  0x1f, 0xbb, 0x1f, 0xff, 0xff, 0xff, 0x3f, 0x7b, 0x3f, 0xff, 0xff, 0xff,
  0xfe, 0xfa, 0xfe, 0xfc, 0xff, 0xff, 0xfe, 0xf8, 0xf5, 0xe8, 0xff, 0xff,
  0x00, 0x00, 0x8a, 0xc0, 0xff, 0xff, 0x7e, 0x02, 0x14, 0x82, 0xff, 0xff,
  0x0e, 0x0a, 0x2e, 0x07, 0xff, 0xff, 0x1e, 0x1a, 0x5f, 0x0f, 0xff, 0xff,
  0x1f, 0x1b, 0xbf, 0x1f, 0xff, 0xff, 0x3f, 0x3b, 0x7f, 0x3f, 0xff, 0xff,
  0xfe, 0xfa, 0xfc, 0xfa, 0xf8, 0xff, 0xfe, 0xfa, 0xf0, 0xe5, 0xf0, 0xff,
  0x7e, 0x7a, 0x80, 0xca, 0xe0, 0xff, 0x7e, 0x7a, 0x00, 0x94, 0xc1, 0xff,
  0x1e, 0x0a, 0x06, 0x2b, 0x83, 0xff, 0x1e, 0x1a, 0x0f, 0x57, 0x07, 0xff,
  0x1f, 0x1b, 0x1f, 0xaf, 0x0f, 0xff, 0x3f, 0x3b, 0x3f, 0x5f, 0x1f, 0xff,
  0xfe, 0xfa, 0xfc, 0xf8, 0xfa, 0xfc, 0xfe, 0xf8, 0xf0, 0xe0, 0xf5, 0xf8,
  0x7e, 0x00, 0x80, 0xc0, 0xea, 0xf1, 0x7e, 0x02, 0x00, 0x80, 0xd5, 0xe3,
  0x3e, 0x0a, 0x06, 0x03, 0xab, 0xc7, 0x3e, 0x1a, 0x0f, 0x07, 0x57, 0x8f,
  0x3f, 0x1b, 0x1f, 0x0f, 0xaf, 0x1f, 0x3f, 0x3b, 0x3f, 0x1f, 0x5f, 0x3f,
  0xfe, 0xfa, 0xfc, 0xf8, 0xf8, 0xfe, 0xfe, 0xfa, 0xf0, 0xe0, 0xf0, 0xfd,
  0x7e, 0x7a, 0x80, 0xc0, 0xe0, 0xfb, 0x7e, 0x7a, 0x00, 0x80, 0xc1, 0xf7,
  0x7e, 0x3a, 0x06, 0x03, 0x83, 0xef, 0x7e, 0x3a, 0x0f, 0x07, 0x07, 0xdf,
  0x7f, 0x3b, 0x1f, 0x0f, 0x0f, 0xbf, 0x7f, 0x3b, 0x3f, 0x1f, 0x1f, 0x7f,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xf7, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xf4, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf3, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xe7, 0xff, 0xff, 0xff, 0xff, 0x5f, 0x17, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0x37, 0xff, 0xff, 0xff, 0xff, 0x7f, 0x37, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xf6, 0xfe, 0xff, 0xff, 0xff, 0xfc, 0xf5, 0xfc, 0xff, 0xff, 0xff,
  0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff, 0xff,
  0xff, 0xf7, 0xef, 0xff, 0xff, 0xff, 0x1f, 0x57, 0x1f, 0xff, 0xff, 0xff,
  0x3f, 0xb7, 0x3f, 0xff, 0xff, 0xff, 0x3f, 0x77, 0x3f, 0xff, 0xff, 0xff,
  0xfe, 0xf6, 0xfe, 0xfc, 0xff, 0xff, 0xfc, 0xf4, 0xfd, 0xf8, 0xff, 0xff,
  0x7e, 0x70, 0xea, 0xd0, 0xff, 0xff, 0x00, 0x00, 0x14, 0x80, 0xff, 0xff,
  0x7e, 0x06, 0x2a, 0x05, 0xff, 0xff, 0x1e, 0x16, 0x5f, 0x0f, 0xff, 0xff,
  0x3f, 0x37, 0xbf, 0x1f, 0xff, 0xff, 0x3f, 0x37, 0x7f, 0x3f, 0xff, 0xff,
  0xfe, 0xf6, 0xfc, 0xfa, 0xf8, 0xff, 0xfe, 0xf4, 0xf8, 0xf5, 0xf0, 0xff,
  0x7e, 0x76, 0xe0, 0xca, 0xe0, 0xff, 0x7e, 0x76, 0x00, 0x94, 0xc1, 0xff,
  0x7e, 0x76, 0x02, 0x29, 0x83, 0xff, 0x3e, 0x16, 0x0f, 0x57, 0x07, 0xff,
  0x3f, 0x37, 0x1f, 0xaf, 0x0f, 0xff, 0x3f, 0x37, 0x3f, 0x5f, 0x1f, 0xff,
  0xfe, 0xf6, 0xfc, 0xf8, 0xfa, 0xfc, 0xfe, 0xf4, 0xf8, 0xf0, 0xf5, 0xf8,
  0x7e, 0x70, 0xe0, 0xc0, 0xea, 0xf1, 0x7e, 0x00, 0x00, 0x80, 0xd5, 0xe3,
  0x7e, 0x06, 0x02, 0x01, 0xab, 0xc7, 0x7e, 0x16, 0x0f, 0x07, 0x57, 0x8f,
  0x7f, 0x37, 0x1f, 0x0f, 0xaf, 0x1f, 0x7f, 0x37, 0x3f, 0x1f, 0x5f, 0x3f,
  0xfe, 0xf6, 0xfc, 0xf8, 0xf8, 0xfe, 0xfe, 0xf6, 0xf8, 0xf0, 0xf0, 0xfd,
  0x7e, 0x76, 0xe0, 0xc0, 0xe0, 0xfb, 0x7e, 0x76, 0x00, 0x80, 0xc1, 0xf7,
  0x7e, 0x76, 0x02, 0x01, 0x83, 0xef, 0x7e, 0x76, 0x0f, 0x07, 0x07, 0xdf,
  0x7f, 0x77, 0x1f, 0x0f, 0x0f, 0xbf, 0x7f, 0x77, 0x3f, 0x1f, 0x1f, 0x7f,
  0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xfe, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff,
  0x0b, 0x03, 0xfe, 0xff, 0xff, 0xff, 0x17, 0x07, 0xfe, 0xff, 0xff, 0xff,
  0x2f, 0x07, 0xfe, 0xff, 0xff, 0xff, 0x5f, 0x0f, 0xfe, 0xff, 0xff, 0xff,
  0xaf, 0x0f, 0xfe, 0xff, 0xff, 0xff, 0x5f, 0x1f, 0xfe, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xfc, 0xff, 0xff, 0xff,
  0x02, 0x0a, 0x82, 0xff, 0xff, 0xff, 0x07, 0x17, 0x06, 0xff, 0xff, 0xff,
  0x07, 0x2f, 0x06, 0xff, 0xff, 0xff, 0x0f, 0x5f, 0x0e, 0xff, 0xff, 0xff,
  0x0f, 0xaf, 0x0e, 0xff, 0xff, 0xff, 0x1f, 0x5f, 0x1e, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff,
  0x02, 0x02, 0x8a, 0xc2, 0xff, 0xff, 0x07, 0x07, 0x16, 0x87, 0xff, 0xff,
  0x07, 0x07, 0x2e, 0x07, 0xff, 0xff, 0x0f, 0x0f, 0x5e, 0x0f, 0xff, 0xff,
  0x0f, 0x0f, 0xae, 0x0f, 0xff, 0xff, 0x1f, 0x1f, 0x5e, 0x1f, 0xff, 0xff,
  0xfe, 0xc0, 0xe0, 0xf2, 0xf8, 0xff, 0xfe, 0xfe, 0xc0, 0xe5, 0xf0, 0xff,
  0x06, 0x02, 0x82, 0xca, 0xe0, 0xff, 0x06, 0x06, 0x06, 0x96, 0xc3, 0xff,
  0x06, 0x06, 0x06, 0x2f, 0x87, 0xff, 0x0f, 0x0f, 0x0e, 0x5f, 0x0f, 0xff,
  0x0f, 0x0f, 0x0e, 0xaf, 0x0f, 0xff, 0x1f, 0x1f, 0x1e, 0x5f, 0x1f, 0xff,
  0xfe, 0xfe, 0xfe, 0xf8, 0xfe, 0xfc, 0xfe, 0xfe, 0xfe, 0xe0, 0xfd, 0xf8,
  0x0e, 0x06, 0x82, 0xc0, 0xea, 0xf1, 0x0e, 0x06, 0x06, 0x82, 0xd7, 0xe3,
  0x0e, 0x06, 0x06, 0x07, 0xaf, 0xc7, 0x0f, 0x0f, 0x0e, 0x0f, 0x5f, 0x8f,
  0x0f, 0x0f, 0x0e, 0x0f, 0xaf, 0x1f, 0x1f, 0x1f, 0x1e, 0x1f, 0x5f, 0x3f,
  0xfe, 0xfe, 0xe0, 0xf8, 0xfc, 0xfe, 0xfe, 0xfe, 0xc0, 0xe0, 0xf8, 0xfd,
  0x1e, 0x0e, 0x82, 0xc0, 0xe0, 0xfb, 0x1e, 0x0e, 0x06, 0x82, 0xc3, 0xf7,
  0x1e, 0x0e, 0x06, 0x07, 0x87, 0xef, 0x1f, 0x0f, 0x0e, 0x0f, 0x0f, 0xdf,
  0x1f, 0x0f, 0x0e, 0x0f, 0x0f, 0xbf, 0x1f, 0x1f, 0x1e, 0x1f, 0x1f, 0x7f,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xfd, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xfd, 0xff, 0xff, 0xff, 0x17, 0x07, 0xfd, 0xff, 0xff, 0xff,
  0x2f, 0x0f, 0xfd, 0xff, 0xff, 0xff, 0x5f, 0x0f, 0xfd, 0xff, 0xff, 0xff,
  0xbf, 0x1f, 0xfd, 0xff, 0xff, 0xff, 0x5f, 0x1f, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xf9, 0xff, 0xff, 0xff, 0x07, 0x17, 0x05, 0xff, 0xff, 0xff,
  0x0f, 0x2f, 0x0d, 0xff, 0xff, 0xff, 0x0f, 0x5f, 0x0d, 0xff, 0xff, 0xff,
  0x1f, 0xbf, 0x1d, 0xff, 0xff, 0xff, 0x1f, 0x5f, 0x1d, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xfd, 0xfb, 0xff, 0xff, 0x07, 0x07, 0x15, 0x87, 0xff, 0xff,
  0x0f, 0x0f, 0x2d, 0x0f, 0xff, 0xff, 0x0f, 0x0f, 0x5d, 0x0f, 0xff, 0xff,
  0x1f, 0x1f, 0xbd, 0x1f, 0xff, 0xff, 0x1f, 0x1f, 0x5d, 0x1f, 0xff, 0xff,
  0xfe, 0xfe, 0xfc, 0xfa, 0xfc, 0xff, 0xfe, 0x80, 0xc0, 0xe5, 0xf0, 0xff,
  0x7e, 0x7e, 0x80, 0xca, 0xe0, 0xff, 0x0e, 0x06, 0x04, 0x96, 0xc3, 0xff,
  0x0e, 0x0e, 0x0c, 0x2f, 0x87, 0xff, 0x0f, 0x0f, 0x0d, 0x5f, 0x0f, 0xff,
  0x1f, 0x1f, 0x1d, 0xbf, 0x1f, 0xff, 0x1f, 0x1f, 0x1d, 0x5f, 0x1f, 0xff,
  0xfe, 0xfe, 0xfc, 0xf8, 0xfe, 0xfc, 0xfe, 0xfe, 0xfc, 0xf0, 0xfd, 0xf8,
  0x7e, 0x7e, 0xfc, 0xc0, 0xfa, 0xf1, 0x1e, 0x0e, 0x04, 0x82, 0xd7, 0xe3,
  0x1e, 0x0e, 0x0c, 0x07, 0xaf, 0xc7, 0x1f, 0x0f, 0x0d, 0x0f, 0x5f, 0x8f,
  0x1f, 0x1f, 0x1d, 0x1f, 0xbf, 0x1f, 0x1f, 0x1f, 0x1d, 0x1f, 0x5f, 0x3f,
  0xfe, 0xfe, 0xfc, 0xf8, 0xfc, 0xfe, 0xfe, 0xfe, 0xc0, 0xf0, 0xf8, 0xfd,
  0x7e, 0x7e, 0x80, 0xc0, 0xf0, 0xfb, 0x3e, 0x1e, 0x04, 0x82, 0xc3, 0xf7,
  0x3e, 0x1e, 0x0c, 0x07, 0x87, 0xef, 0x3f, 0x1f, 0x0d, 0x0f, 0x0f, 0xdf,
  0x3f, 0x1f, 0x1d, 0x1f, 0x1f, 0xbf, 0x3f, 0x1f, 0x1d, 0x1f, 0x1f, 0x7f,
  0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xfb, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xfe, 0xfe, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff,
  0x2f, 0x0f, 0xfb, 0xff, 0xff, 0xff, 0x5f, 0x1f, 0xfb, 0xff, 0xff, 0xff,
  0xbf, 0x1f, 0xfb, 0xff, 0xff, 0xff, 0x7f, 0x3f, 0xfb, 0xff, 0xff, 0xff,
  0xfe, 0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf9, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xf3, 0xff, 0xff, 0xff,
  0x0f, 0x2f, 0x0b, 0xff, 0xff, 0xff, 0x1f, 0x5f, 0x1b, 0xff, 0xff, 0xff,
  0x1f, 0xbf, 0x1b, 0xff, 0xff, 0xff, 0x3f, 0x7f, 0x3b, 0xff, 0xff, 0xff,
  0xfe, 0xfe, 0xfa, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff,
  0x0f, 0x0f, 0x2b, 0x0f, 0xff, 0xff, 0x1f, 0x1f, 0x5b, 0x1f, 0xff, 0xff,
  0x1f, 0x1f, 0xbb, 0x1f, 0xff, 0xff, 0x3f, 0x3f, 0x7b, 0x3f, 0xff, 0xff,
  0xfe, 0xfe, 0xfa, 0xfe, 0xfc, 0xff, 0xfe, 0xfe, 0xf8, 0xf5, 0xf8, 0xff,
  0x7e, 0x00, 0x80, 0xca, 0xe0, 0xff, 0x7e, 0x7e, 0x02, 0x94, 0xc3, 0xff,
  0x1e, 0x0e, 0x0a, 0x2f, 0x87, 0xff, 0x1f, 0x1f, 0x1b, 0x5f, 0x0f, 0xff,
  0x1f, 0x1f, 0x1b, 0xbf, 0x1f, 0xff, 0x3f, 0x3f, 0x3b, 0x7f, 0x3f, 0xff,
  0xfe, 0xfe, 0xfa, 0xfc, 0xfe, 0xfc, 0xfe, 0xfe, 0xfa, 0xf0, 0xfd, 0xf8,
  0x7e, 0x7e, 0xfa, 0xe0, 0xfa, 0xf1, 0x7e, 0x7e, 0x7a, 0x80, 0xf7, 0xe3,
  0x3e, 0x1e, 0x0a, 0x07, 0xaf, 0xc7, 0x3f, 0x1f, 0x1b, 0x0f, 0x5f, 0x8f,
  0x3f, 0x1f, 0x1b, 0x1f, 0xbf, 0x1f, 0x3f, 0x3f, 0x3b, 0x3f, 0x7f, 0x3f,
  0xfe, 0xfe, 0xfa, 0xfc, 0xfc, 0xfe, 0xfe, 0xfe, 0xf8, 0xf0, 0xf8, 0xfd,
  0x7e, 0x7e, 0x80, 0xe0, 0xf0, 0xfb, 0x7e, 0x7e, 0x02, 0x80, 0xe3, 0xf7,
  0x7e, 0x3e, 0x0a, 0x07, 0x87, 0xef, 0x7f, 0x3f, 0x1b, 0x0f, 0x0f, 0xdf,
  0x7f, 0x3f, 0x1b, 0x1f, 0x1f, 0xbf, 0x7f, 0x3f, 0x3b, 0x3f, 0x3f, 0x7f,
  0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xf7, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0xfe, 0xfe, 0xf7, 0xff, 0xff, 0xff, 0xfd, 0xfc, 0xf7, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff, 0xff,
  0xff, 0xef, 0xf7, 0xff, 0xff, 0xff, 0x5f, 0x1f, 0xf7, 0xff, 0xff, 0xff,
  0xbf, 0x3f, 0xf7, 0xff, 0xff, 0xff, 0x7f, 0x3f, 0xf7, 0xff, 0xff, 0xff,
  0xfe, 0xfe, 0xf6, 0xff, 0xff, 0xff, 0xfc, 0xfd, 0xf4, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf3, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xe7, 0xff, 0xff, 0xff, 0x1f, 0x5f, 0x17, 0xff, 0xff, 0xff,
  0x3f, 0xbf, 0x37, 0xff, 0xff, 0xff, 0x3f, 0x7f, 0x37, 0xff, 0xff, 0xff,
  0xfe, 0xfe, 0xf6, 0xfe, 0xff, 0xff, 0xfc, 0xfc, 0xf5, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xf7, 0xef, 0xff, 0xff, 0x1f, 0x1f, 0x57, 0x1f, 0xff, 0xff,
  0x3f, 0x3f, 0xb7, 0x3f, 0xff, 0xff, 0x3f, 0x3f, 0x77, 0x3f, 0xff, 0xff,
  0xfe, 0xfe, 0xf6, 0xfe, 0xfc, 0xff, 0xfe, 0xfc, 0xf4, 0xfd, 0xf8, 0xff,
  0xfe, 0xfe, 0xf0, 0xea, 0xf0, 0xff, 0x7e, 0x00, 0x00, 0x94, 0xc1, 0xff,
  0x7e, 0x7e, 0x06, 0x2b, 0x87, 0xff, 0x3f, 0x1f, 0x17, 0x5f, 0x0f, 0xff,
  0x3f, 0x3f, 0x37, 0xbf, 0x1f, 0xff, 0x3f, 0x3f, 0x37, 0x7f, 0x3f, 0xff,
  0xfe, 0xfe, 0xf6, 0xfc, 0xfe, 0xfc, 0xfe, 0xfe, 0xf4, 0xf8, 0xfd, 0xf8,
  0xfe, 0xfe, 0xf6, 0xe0, 0xfa, 0xf1, 0x7e, 0x7e, 0x76, 0xc0, 0xf7, 0xe3,
  0x7e, 0x7e, 0x76, 0x03, 0xef, 0xc7, 0x7f, 0x3f, 0x17, 0x0f, 0x5f, 0x8f,
  0x7f, 0x3f, 0x37, 0x1f, 0xbf, 0x1f, 0x7f, 0x3f, 0x37, 0x3f, 0x7f, 0x3f,
  0xfe, 0xfe, 0xf6, 0xfc, 0xfc, 0xfe, 0xfe, 0xfe, 0xf4, 0xf8, 0xf8, 0xfd,
  0xfe, 0xfe, 0xf0, 0xe0, 0xf0, 0xfb, 0x7e, 0x7e, 0x00, 0xc0, 0xe3, 0xf7,
  0x7e, 0x7e, 0x06, 0x03, 0xc7, 0xef, 0x7f, 0x7f, 0x17, 0x0f, 0x0f, 0xdf,
  0x7f, 0x7f, 0x37, 0x1f, 0x1f, 0xbf, 0x7f, 0x7f, 0x37, 0x3f, 0x3f, 0x7f,
  0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xfe, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xfe, 0xfe, 0xff, 0xfe, 0xff, 0xff, 0xfd, 0x81, 0xff, 0xfe, 0xff, 0xff,
  0x0b, 0x03, 0xff, 0xfe, 0xff, 0xff, 0x17, 0x07, 0xff, 0xfe, 0xff, 0xff,
  0x2f, 0x07, 0xff, 0xfe, 0xff, 0xff, 0x5f, 0x0f, 0xff, 0xfe, 0xff, 0xff,
  0xaf, 0x0f, 0xff, 0xfe, 0xff, 0xff, 0x5f, 0x1f, 0xff, 0xfe, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff,
  0x02, 0x0a, 0x83, 0xfe, 0xff, 0xff, 0x07, 0x17, 0x07, 0xfe, 0xff, 0xff,
  0x07, 0x2f, 0x07, 0xfe, 0xff, 0xff, 0x0f, 0x5f, 0x0f, 0xfe, 0xff, 0xff,
  0x0f, 0xaf, 0x0f, 0xfe, 0xff, 0xff, 0x1f, 0x5f, 0x1f, 0xfe, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xfc, 0xff, 0xff,
  0x02, 0x02, 0x8a, 0xc2, 0xff, 0xff, 0x07, 0x07, 0x17, 0x86, 0xff, 0xff,
  0x07, 0x07, 0x2f, 0x06, 0xff, 0xff, 0x0f, 0x0f, 0x5f, 0x0e, 0xff, 0xff,
  0x0f, 0x0f, 0xaf, 0x0e, 0xff, 0xff, 0x1f, 0x1f, 0x5f, 0x1e, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff,
  0x06, 0x02, 0x82, 0xca, 0xe2, 0xff, 0x07, 0x07, 0x07, 0x96, 0xc7, 0xff,
  0x07, 0x07, 0x07, 0x2e, 0x87, 0xff, 0x0f, 0x0f, 0x0f, 0x5e, 0x0f, 0xff,
  0x0f, 0x0f, 0x0f, 0xae, 0x0f, 0xff, 0x1f, 0x1f, 0x1f, 0x5e, 0x1f, 0xff,
  0xfe, 0xfe, 0xe0, 0xfc, 0xfe, 0xfc, 0xfe, 0xfe, 0xfe, 0xe0, 0xfd, 0xf8,
  0x0e, 0x06, 0x82, 0xc2, 0xea, 0xf1, 0x0e, 0x06, 0x06, 0x86, 0xd7, 0xe3,
  0x0f, 0x07, 0x07, 0x06, 0xaf, 0xc7, 0x0f, 0x0f, 0x0f, 0x0e, 0x5f, 0x8f,
  0x0f, 0x0f, 0x0f, 0x0e, 0xaf, 0x1f, 0x1f, 0x1f, 0x1f, 0x1e, 0x5f, 0x3f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfc, 0xff,
  0x1e, 0x0e, 0x86, 0xc6, 0xe2, 0xff, 0x1e, 0x0e, 0x06, 0x86, 0xc7, 0xf7,
  0x1f, 0x0f, 0x07, 0x06, 0x87, 0xef, 0x1f, 0x0f, 0x0f, 0x0e, 0x0f, 0xdf,
  0x1f, 0x0f, 0x0f, 0x0e, 0x0f, 0xbf, 0x1f, 0x1f, 0x1f, 0x1e, 0x1f, 0x7f,
  0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xfd, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xfe, 0xfe, 0xff, 0xfd, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xfd, 0xff, 0xff,
  0xfb, 0x03, 0xff, 0xfd, 0xff, 0xff, 0x17, 0x07, 0xff, 0xfd, 0xff, 0xff,
  0x2f, 0x0f, 0xff, 0xfd, 0xff, 0xff, 0x5f, 0x0f, 0xff, 0xfd, 0xff, 0xff,
  0xbf, 0x1f, 0xff, 0xfd, 0xff, 0xff, 0x5f, 0x1f, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xfb, 0xfd, 0xff, 0xff, 0x07, 0x17, 0x07, 0xfd, 0xff, 0xff,
  0x0f, 0x2f, 0x0f, 0xfd, 0xff, 0xff, 0x0f, 0x5f, 0x0f, 0xfd, 0xff, 0xff,
  0x1f, 0xbf, 0x1f, 0xfd, 0xff, 0xff, 0x1f, 0x5f, 0x1f, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xf9, 0xff, 0xff, 0x07, 0x07, 0x17, 0x85, 0xff, 0xff,
  0x0f, 0x0f, 0x2f, 0x0d, 0xff, 0xff, 0x0f, 0x0f, 0x5f, 0x0d, 0xff, 0xff,
  0x1f, 0x1f, 0xbf, 0x1d, 0xff, 0xff, 0x1f, 0x1f, 0x5f, 0x1d, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xfd, 0xfb, 0xff, 0x0f, 0x07, 0x07, 0x95, 0xc7, 0xff,
  0x0f, 0x0f, 0x0f, 0x2d, 0x8f, 0xff, 0x0f, 0x0f, 0x0f, 0x5d, 0x0f, 0xff,
  0x1f, 0x1f, 0x1f, 0xbd, 0x1f, 0xff, 0x1f, 0x1f, 0x1f, 0x5d, 0x1f, 0xff,
  0xfe, 0xfe, 0xfe, 0xfc, 0xfe, 0xfc, 0xfe, 0xfe, 0xc0, 0xf8, 0xfd, 0xf8,
  0x7e, 0x7e, 0xfe, 0xc0, 0xfa, 0xf1, 0x1e, 0x0e, 0x06, 0x84, 0xd7, 0xe3,
  0x1f, 0x0f, 0x0f, 0x0d, 0xaf, 0xc7, 0x1f, 0x0f, 0x0f, 0x0d, 0x5f, 0x8f,
  0x1f, 0x1f, 0x1f, 0x1d, 0xbf, 0x1f, 0x1f, 0x1f, 0x1f, 0x1d, 0x5f, 0x3f,
  0xfe, 0xfe, 0xfe, 0xfc, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfc, 0xfc, 0xff,
  0x7e, 0x7e, 0xfe, 0xfc, 0xfa, 0xff, 0x3e, 0x1e, 0x0e, 0x8c, 0xc7, 0xff,
  0x3f, 0x1f, 0x0f, 0x0d, 0x8f, 0xef, 0x3f, 0x1f, 0x0f, 0x0d, 0x0f, 0xdf,
  0x3f, 0x1f, 0x1f, 0x1d, 0x1f, 0xbf, 0x3f, 0x1f, 0x1f, 0x1d, 0x1f, 0x7f,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xfb, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0xfe, 0xfe, 0xff, 0xfb, 0xff, 0xff, 0xfd, 0xfc, 0xff, 0xfb, 0xff, 0xff,
  0xfb, 0xfb, 0xff, 0xfb, 0xff, 0xff, 0xf7, 0x07, 0xff, 0xfb, 0xff, 0xff,
  0x2f, 0x0f, 0xff, 0xfb, 0xff, 0xff, 0x5f, 0x1f, 0xff, 0xfb, 0xff, 0xff,
  0xbf, 0x1f, 0xff, 0xfb, 0xff, 0xff, 0x7f, 0x3f, 0xff, 0xfb, 0xff, 0xff,
  0xfe, 0xfe, 0xfe, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb, 0xff, 0xff,
  0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff,
  0x0f, 0x2f, 0x0f, 0xfb, 0xff, 0xff, 0x1f, 0x5f, 0x1f, 0xfb, 0xff, 0xff,
  0x1f, 0xbf, 0x1f, 0xfb, 0xff, 0xff, 0x3f, 0x7f, 0x3f, 0xfb, 0xff, 0xff,
  0xfe, 0xfe, 0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf9, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xf3, 0xff, 0xff,
  0x0f, 0x0f, 0x2f, 0x0b, 0xff, 0xff, 0x1f, 0x1f, 0x5f, 0x1b, 0xff, 0xff,
  0x1f, 0x1f, 0xbf, 0x1b, 0xff, 0xff, 0x3f, 0x3f, 0x7f, 0x3b, 0xff, 0xff,
  0xfe, 0xfe, 0xfe, 0xfa, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff,
  0x1f, 0x0f, 0x0f, 0x2b, 0x8f, 0xff, 0x1f, 0x1f, 0x1f, 0x5b, 0x1f, 0xff,
  0x1f, 0x1f, 0x1f, 0xbb, 0x1f, 0xff, 0x3f, 0x3f, 0x3f, 0x7b, 0x3f, 0xff,
  0xfe, 0xfe, 0xfe, 0xfa, 0xfe, 0xfc, 0xfe, 0xfe, 0xfe, 0xf8, 0xfd, 0xf8,
  0x7e, 0x7e, 0x80, 0xf0, 0xfa, 0xf1, 0x7e, 0x7e, 0x7e, 0x82, 0xf7, 0xe3,
  0x3f, 0x1f, 0x0f, 0x0b, 0xaf, 0xc7, 0x3f, 0x1f, 0x1f, 0x1b, 0x5f, 0x8f,
  0x3f, 0x1f, 0x1f, 0x1b, 0xbf, 0x1f, 0x3f, 0x3f, 0x3f, 0x3b, 0x7f, 0x3f,
  0xfe, 0xfe, 0xfe, 0xfa, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfa, 0xfc, 0xff,
  0x7e, 0x7e, 0xfe, 0xfa, 0xfa, 0xff, 0x7e, 0x7e, 0x7e, 0xfa, 0xf7, 0xff,
  0x7f, 0x3f, 0x1f, 0x1b, 0x8f, 0xff, 0x7f, 0x3f, 0x1f, 0x1b, 0x1f, 0xdf,
  0x7f, 0x3f, 0x1f, 0x1b, 0x1f, 0xbf, 0x7f, 0x3f, 0x3f, 0x3b, 0x3f, 0x7f,
  0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xf7, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xfe, 0xfe, 0xff, 0xf7, 0xff, 0xff, 0xfd, 0xfc, 0xff, 0xf7, 0xff, 0xff,
  0xfb, 0xf8, 0xff, 0xf7, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xf7, 0xff, 0xff,
  0xef, 0x0f, 0xff, 0xf7, 0xff, 0xff, 0x5f, 0x1f, 0xff, 0xf7, 0xff, 0xff,
  0xbf, 0x3f, 0xff, 0xf7, 0xff, 0xff, 0x7f, 0x3f, 0xff, 0xf7, 0xff, 0xff,
  0xfe, 0xfe, 0xfe, 0xf7, 0xff, 0xff, 0xfc, 0xfd, 0xfc, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xef, 0xf7, 0xff, 0xff, 0x1f, 0x5f, 0x1f, 0xf7, 0xff, 0xff,
  0x3f, 0xbf, 0x3f, 0xf7, 0xff, 0xff, 0x3f, 0x7f, 0x3f, 0xf7, 0xff, 0xff,
  0xfe, 0xfe, 0xfe, 0xf6, 0xff, 0xff, 0xfc, 0xfc, 0xfd, 0xf4, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf3, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xe7, 0xff, 0xff, 0x1f, 0x1f, 0x5f, 0x17, 0xff, 0xff,
  0x3f, 0x3f, 0xbf, 0x37, 0xff, 0xff, 0x3f, 0x3f, 0x7f, 0x37, 0xff, 0xff,
  0xfe, 0xfe, 0xfe, 0xf6, 0xfe, 0xff, 0xfe, 0xfc, 0xfc, 0xf5, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff,
  0xff, 0xff, 0xff, 0xf7, 0xef, 0xff, 0x3f, 0x1f, 0x1f, 0x57, 0x1f, 0xff,
  0x3f, 0x3f, 0x3f, 0xb7, 0x3f, 0xff, 0x3f, 0x3f, 0x3f, 0x77, 0x3f, 0xff,
  0xfe, 0xfe, 0xfe, 0xf6, 0xfe, 0xfc, 0xfe, 0xfe, 0xfc, 0xf4, 0xfd, 0xf8,
  0xfe, 0xfe, 0xfe, 0xf0, 0xfa, 0xf1, 0x7e, 0x7e, 0x00, 0xe2, 0xf7, 0xe3,
  0x7f, 0x7f, 0x7f, 0x07, 0xef, 0xc7, 0x7f, 0x3f, 0x1f, 0x17, 0x5f, 0x8f,
  0x7f, 0x3f, 0x3f, 0x37, 0xbf, 0x1f, 0x7f, 0x3f, 0x3f, 0x37, 0x7f, 0x3f,
  0xfe, 0xfe, 0xfe, 0xf6, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xf6, 0xfc, 0xff,
  0xfe, 0xfe, 0xfe, 0xf6, 0xfa, 0xff, 0x7e, 0x7e, 0x7e, 0xf6, 0xf7, 0xff,
  0x7f, 0x7f, 0x7f, 0x77, 0xef, 0xff, 0x7f, 0x7f, 0x3f, 0x37, 0x1f, 0xff,
  0x7f, 0x7f, 0x3f, 0x37, 0x3f, 0xbf, 0x7f, 0x7f, 0x3f, 0x37, 0x3f, 0x7f,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xfe, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0xfe, 0xfc, 0xff, 0xff, 0xfe, 0xff, 0xfd, 0x80, 0xff, 0xff, 0xfe, 0xff,
  0x0b, 0x01, 0xff, 0xff, 0xfe, 0xff, 0x17, 0x03, 0xff, 0xff, 0xfe, 0xff,
  0x2f, 0x07, 0xff, 0xff, 0xfe, 0xff, 0x5f, 0x0f, 0xff, 0xff, 0xfe, 0xff,
  0xaf, 0x0f, 0xff, 0xff, 0xfe, 0xff, 0x5f, 0x1f, 0xff, 0xff, 0xfe, 0xff,
  0xc0, 0xc2, 0xfe, 0xff, 0xfe, 0xff, 0x80, 0x85, 0xc1, 0xff, 0xfe, 0xff,
  0x00, 0x0a, 0x83, 0xff, 0xfe, 0xff, 0x03, 0x17, 0x07, 0xff, 0xfe, 0xff,
  0x07, 0x2f, 0x07, 0xff, 0xfe, 0xff, 0x0f, 0x5f, 0x0f, 0xff, 0xfe, 0xff,
  0x0f, 0xaf, 0x0f, 0xff, 0xfe, 0xff, 0x1f, 0x5f, 0x1f, 0xff, 0xfe, 0xff,
  0xff, 0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff,
  0x02, 0x02, 0x8a, 0xc3, 0xfe, 0xff, 0x07, 0x07, 0x17, 0x87, 0xfe, 0xff,
  0x07, 0x07, 0x2f, 0x07, 0xfe, 0xff, 0x0f, 0x0f, 0x5f, 0x0f, 0xfe, 0xff,
  0x0f, 0x0f, 0xaf, 0x0f, 0xfe, 0xff, 0x1f, 0x1f, 0x5f, 0x1f, 0xfe, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xff,
  0x02, 0x02, 0x82, 0xca, 0xe2, 0xff, 0x07, 0x07, 0x07, 0x97, 0xc6, 0xff,
  0x07, 0x07, 0x07, 0x2f, 0x86, 0xff, 0x0f, 0x0f, 0x0f, 0x5f, 0x0e, 0xff,
  0x0f, 0x0f, 0x0f, 0xaf, 0x0e, 0xff, 0x1f, 0x1f, 0x1f, 0x5f, 0x1e, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd,
  0x0e, 0x06, 0x82, 0xc2, 0xea, 0xf3, 0x0f, 0x07, 0x07, 0x87, 0xd6, 0xe7,
  0x0f, 0x07, 0x07, 0x07, 0xae, 0xc7, 0x0f, 0x0f, 0x0f, 0x0f, 0x5e, 0x8f,
  0x0f, 0x0f, 0x0f, 0x0f, 0xae, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x5e, 0x3f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfc, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfd,
  0x1e, 0x0e, 0x86, 0xc6, 0xe2, 0xff, 0x1f, 0x0f, 0x07, 0x87, 0xc6, 0xf7,
  0x1f, 0x0f, 0x07, 0x07, 0x86, 0xef, 0x1f, 0x0f, 0x0f, 0x0f, 0x0e, 0xdf,
  0x1f, 0x0f, 0x0f, 0x0f, 0x0e, 0xbf, 0x1f, 0x1f, 0x1f, 0x1f, 0x1e, 0x7f,
  0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xfd, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xfe, 0xfc, 0xff, 0xff, 0xfd, 0xff, 0xfd, 0xf8, 0xff, 0xff, 0xfd, 0xff,
  0xfb, 0x01, 0xff, 0xff, 0xfd, 0xff, 0x17, 0x03, 0xff, 0xff, 0xfd, 0xff,
  0x2f, 0x07, 0xff, 0xff, 0xfd, 0xff, 0x5f, 0x0f, 0xff, 0xff, 0xfd, 0xff,
  0xbf, 0x1f, 0xff, 0xff, 0xfd, 0xff, 0x5f, 0x1f, 0xff, 0xff, 0xfd, 0xff,
  0xfc, 0xfe, 0xfe, 0xff, 0xfd, 0xff, 0x80, 0x85, 0xfd, 0xff, 0xfd, 0xff,
  0x01, 0x0b, 0x83, 0xff, 0xfd, 0xff, 0x03, 0x17, 0x07, 0xff, 0xfd, 0xff,
  0x07, 0x2f, 0x0f, 0xff, 0xfd, 0xff, 0x0f, 0x5f, 0x0f, 0xff, 0xfd, 0xff,
  0x1f, 0xbf, 0x1f, 0xff, 0xfd, 0xff, 0x1f, 0x5f, 0x1f, 0xff, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff, 0x07, 0x07, 0x17, 0x87, 0xfd, 0xff,
  0x0f, 0x0f, 0x2f, 0x0f, 0xfd, 0xff, 0x0f, 0x0f, 0x5f, 0x0f, 0xfd, 0xff,
  0x1f, 0x1f, 0xbf, 0x1f, 0xfd, 0xff, 0x1f, 0x1f, 0x5f, 0x1f, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xf9, 0xff, 0x07, 0x07, 0x07, 0x97, 0xc5, 0xff,
  0x0f, 0x0f, 0x0f, 0x2f, 0x8d, 0xff, 0x0f, 0x0f, 0x0f, 0x5f, 0x0d, 0xff,
  0x1f, 0x1f, 0x1f, 0xbf, 0x1d, 0xff, 0x1f, 0x1f, 0x1f, 0x5f, 0x1d, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb, 0x1f, 0x0f, 0x07, 0x87, 0xd5, 0xe7,
  0x1f, 0x0f, 0x0f, 0x0f, 0xad, 0xcf, 0x1f, 0x0f, 0x0f, 0x0f, 0x5d, 0x8f,
  0x1f, 0x1f, 0x1f, 0x1f, 0xbd, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x5d, 0x3f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfc, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xf8, 0xff,
  0x7e, 0x7e, 0xfe, 0xfe, 0xfd, 0xfb, 0x3f, 0x1f, 0x0f, 0x8f, 0xc5, 0xff,
  0x3f, 0x1f, 0x0f, 0x0f, 0x8d, 0xef, 0x3f, 0x1f, 0x0f, 0x0f, 0x0d, 0xdf,
  0x3f, 0x1f, 0x1f, 0x1f, 0x1d, 0xbf, 0x3f, 0x1f, 0x1f, 0x1f, 0x1d, 0x7f,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xfb, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xfe, 0xfc, 0xff, 0xff, 0xfb, 0xff, 0xfd, 0xf8, 0xff, 0xff, 0xfb, 0xff,
  0xfb, 0xf1, 0xff, 0xff, 0xfb, 0xff, 0xf7, 0x03, 0xff, 0xff, 0xfb, 0xff,
  0x2f, 0x07, 0xff, 0xff, 0xfb, 0xff, 0x5f, 0x0f, 0xff, 0xff, 0xfb, 0xff,
  0xbf, 0x1f, 0xff, 0xff, 0xfb, 0xff, 0x7f, 0x3f, 0xff, 0xff, 0xfb, 0xff,
  0xfc, 0xfe, 0xfe, 0xff, 0xfb, 0xff, 0xf8, 0xfd, 0xfc, 0xff, 0xfb, 0xff,
  0x00, 0x0a, 0xfb, 0xff, 0xfb, 0xff, 0x03, 0x17, 0x07, 0xff, 0xfb, 0xff,
  0x07, 0x2f, 0x0f, 0xff, 0xfb, 0xff, 0x0f, 0x5f, 0x1f, 0xff, 0xfb, 0xff,
  0x1f, 0xbf, 0x1f, 0xff, 0xfb, 0xff, 0x3f, 0x7f, 0x3f, 0xff, 0xfb, 0xff,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff,
  0x0f, 0x0f, 0x2f, 0x0f, 0xfb, 0xff, 0x1f, 0x1f, 0x5f, 0x1f, 0xfb, 0xff,
  0x1f, 0x1f, 0xbf, 0x1f, 0xfb, 0xff, 0x3f, 0x3f, 0x7f, 0x3f, 0xfb, 0xff,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf9, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xf3, 0xff,
  0x0f, 0x0f, 0x0f, 0x2f, 0x8b, 0xff, 0x1f, 0x1f, 0x1f, 0x5f, 0x1b, 0xff,
  0x1f, 0x1f, 0x1f, 0xbf, 0x1b, 0xff, 0x3f, 0x3f, 0x3f, 0x7f, 0x3b, 0xff,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfa, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7,
  0x3f, 0x1f, 0x0f, 0x0f, 0xab, 0xcf, 0x3f, 0x1f, 0x1f, 0x1f, 0x5b, 0x9f,
  0x3f, 0x1f, 0x1f, 0x1f, 0xbb, 0x1f, 0x3f, 0x3f, 0x3f, 0x3f, 0x7b, 0x3f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xf8, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfa, 0xfd,
  0x7e, 0x7e, 0xfe, 0xfe, 0xf1, 0xff, 0x7f, 0x7f, 0x7f, 0xff, 0xfb, 0xf7,
  0x7f, 0x3f, 0x1f, 0x1f, 0x8b, 0xff, 0x7f, 0x3f, 0x1f, 0x1f, 0x1b, 0xdf,
  0x7f, 0x3f, 0x1f, 0x1f, 0x1b, 0xbf, 0x7f, 0x3f, 0x3f, 0x3f, 0x3b, 0x7f,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xf7, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0xfe, 0xfc, 0xff, 0xff, 0xf7, 0xff, 0xfd, 0xf8, 0xff, 0xff, 0xf7, 0xff,
  0xfb, 0xf0, 0xff, 0xff, 0xf7, 0xff, 0xf7, 0xe3, 0xff, 0xff, 0xf7, 0xff,
  0xef, 0x07, 0xff, 0xff, 0xf7, 0xff, 0x5f, 0x0f, 0xff, 0xff, 0xf7, 0xff,
  0xbf, 0x1f, 0xff, 0xff, 0xf7, 0xff, 0x7f, 0x3f, 0xff, 0xff, 0xf7, 0xff,
  0xfc, 0xfe, 0xfe, 0xff, 0xf7, 0xff, 0xf8, 0xfd, 0xfc, 0xff, 0xf7, 0xff,
  0xf0, 0xfa, 0xf8, 0xff, 0xf7, 0xff, 0x00, 0x14, 0xf7, 0xff, 0xf7, 0xff,
  0x07, 0x2f, 0x0f, 0xff, 0xf7, 0xff, 0x0f, 0x5f, 0x1f, 0xff, 0xf7, 0xff,
  0x1f, 0xbf, 0x3f, 0xff, 0xf7, 0xff, 0x3f, 0x7f, 0x3f, 0xff, 0xf7, 0xff,
  0xfe, 0xfe, 0xfe, 0xfe, 0xf7, 0xff, 0xfc, 0xfc, 0xfd, 0xfc, 0xf7, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff,
  0xff, 0xff, 0xff, 0xef, 0xf7, 0xff, 0x1f, 0x1f, 0x5f, 0x1f, 0xf7, 0xff,
  0x3f, 0x3f, 0xbf, 0x3f, 0xf7, 0xff, 0x3f, 0x3f, 0x7f, 0x3f, 0xf7, 0xff,
  0xfe, 0xfe, 0xfe, 0xfe, 0xf6, 0xff, 0xfc, 0xfc, 0xfc, 0xfd, 0xf4, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf3, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xe7, 0xff, 0x1f, 0x1f, 0x1f, 0x5f, 0x17, 0xff,
  0x3f, 0x3f, 0x3f, 0xbf, 0x37, 0xff, 0x3f, 0x3f, 0x3f, 0x7f, 0x37, 0xff,
  0xfe, 0xfe, 0xfe, 0xfe, 0xf6, 0xfe, 0xfe, 0xfe, 0xfc, 0xfc, 0xf5, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xef, 0x7f, 0x3f, 0x1f, 0x1f, 0x57, 0x9f,
  0x7f, 0x3f, 0x3f, 0x3f, 0xb7, 0x3f, 0x7f, 0x3f, 0x3f, 0x3f, 0x77, 0x3f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xf6, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xf4, 0xff,
  0xfe, 0xfe, 0xfe, 0xfe, 0xf6, 0xfb, 0x7e, 0x7e, 0x7e, 0xfe, 0xe3, 0xff,
  0x7f, 0x7f, 0x7f, 0x7f, 0xf7, 0xef, 0x7f, 0x7f, 0x3f, 0x3f, 0x17, 0xff,
  0x7f, 0x7f, 0x3f, 0x3f, 0x37, 0xbf, 0x7f, 0x7f, 0x3f, 0x3f, 0x37, 0x7f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xc2, 0xc0, 0xff, 0xff, 0xff, 0xfe, 0x85, 0x80, 0xff, 0xff, 0xff, 0xfe,
  0x0a, 0x00, 0xff, 0xff, 0xff, 0xfe, 0x15, 0x01, 0xff, 0xff, 0xff, 0xfe,
  0x2b, 0x03, 0xff, 0xff, 0xff, 0xfe, 0x57, 0x07, 0xff, 0xff, 0xff, 0xfe,
  0xaf, 0x0f, 0xff, 0xff, 0xff, 0xfe, 0x5f, 0x1f, 0xff, 0xff, 0xff, 0xfe,
  0xc0, 0xc2, 0xfc, 0xff, 0xff, 0xfe, 0x80, 0x85, 0xc0, 0xff, 0xff, 0xfe,
  0x00, 0x0a, 0x81, 0xff, 0xff, 0xfe, 0x01, 0x15, 0x03, 0xff, 0xff, 0xfe,
  0x03, 0x2b, 0x07, 0xff, 0xff, 0xfe, 0x07, 0x57, 0x0f, 0xff, 0xff, 0xfe,
  0x0f, 0xaf, 0x0f, 0xff, 0xff, 0xfe, 0x1f, 0x5f, 0x1f, 0xff, 0xff, 0xfe,
  0xc0, 0xc0, 0xe2, 0xfe, 0xff, 0xfe, 0x80, 0x80, 0xc5, 0xe1, 0xff, 0xfe,
  0x00, 0x00, 0x8a, 0xc3, 0xff, 0xfe, 0x01, 0x03, 0x17, 0x87, 0xff, 0xfe,
  0x03, 0x07, 0x2f, 0x07, 0xff, 0xfe, 0x07, 0x0f, 0x5f, 0x0f, 0xff, 0xfe,
  0x0f, 0x0f, 0xaf, 0x0f, 0xff, 0xfe, 0x1f, 0x1f, 0x5f, 0x1f, 0xff, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe,
  0x02, 0x02, 0x82, 0xca, 0xe3, 0xfe, 0x07, 0x07, 0x07, 0x97, 0xc7, 0xfe,
  0x07, 0x07, 0x07, 0x2f, 0x87, 0xfe, 0x0f, 0x0f, 0x0f, 0x5f, 0x0f, 0xfe,
  0x0f, 0x0f, 0x0f, 0xaf, 0x0f, 0xfe, 0x1f, 0x1f, 0x1f, 0x5f, 0x1f, 0xfe,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0x02, 0x02, 0x82, 0xc2, 0xea, 0xf2, 0x07, 0x07, 0x07, 0x87, 0xd7, 0xe6,
  0x07, 0x07, 0x07, 0x07, 0xaf, 0xc6, 0x0f, 0x0f, 0x0f, 0x0f, 0x5f, 0x8e,
  0x0f, 0x0f, 0x0f, 0x0f, 0xaf, 0x1e, 0x1f, 0x1f, 0x1f, 0x1f, 0x5f, 0x3e,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0x1e, 0x0e, 0x86, 0xc6, 0xe2, 0xfe, 0x1f, 0x0f, 0x07, 0x87, 0xc7, 0xf6,
  0x1f, 0x0f, 0x07, 0x07, 0x87, 0xee, 0x1f, 0x0f, 0x0f, 0x0f, 0x0f, 0xde,
  0x1f, 0x0f, 0x0f, 0x0f, 0x0f, 0xbe, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x7e,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xfd, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xfa, 0xf8, 0xff, 0xff, 0xff, 0xfd, 0x85, 0x80, 0xff, 0xff, 0xff, 0xfd,
  0x0a, 0x00, 0xff, 0xff, 0xff, 0xfd, 0x15, 0x01, 0xff, 0xff, 0xff, 0xfd,
  0x2b, 0x03, 0xff, 0xff, 0xff, 0xfd, 0x57, 0x07, 0xff, 0xff, 0xff, 0xfd,
  0xaf, 0x0f, 0xff, 0xff, 0xff, 0xfd, 0x5f, 0x1f, 0xff, 0xff, 0xff, 0xfd,
  0xf8, 0xfa, 0xfc, 0xff, 0xff, 0xfd, 0x80, 0x85, 0xf8, 0xff, 0xff, 0xfd,
  0x00, 0x0a, 0x81, 0xff, 0xff, 0xfd, 0x01, 0x15, 0x03, 0xff, 0xff, 0xfd,
  0x03, 0x2b, 0x07, 0xff, 0xff, 0xfd, 0x07, 0x57, 0x0f, 0xff, 0xff, 0xfd,
  0x0f, 0xaf, 0x1f, 0xff, 0xff, 0xfd, 0x1f, 0x5f, 0x1f, 0xff, 0xff, 0xfd,
  0xf8, 0xfc, 0xfe, 0xfe, 0xff, 0xfd, 0x80, 0x80, 0xc5, 0xfd, 0xff, 0xfd,
  0x00, 0x00, 0x8b, 0xc3, 0xff, 0xfd, 0x01, 0x03, 0x17, 0x87, 0xff, 0xfd,
  0x03, 0x07, 0x2f, 0x0f, 0xff, 0xfd, 0x07, 0x0f, 0x5f, 0x0f, 0xff, 0xfd,
  0x0f, 0x1f, 0xbf, 0x1f, 0xff, 0xfd, 0x1f, 0x1f, 0x5f, 0x1f, 0xff, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0x07, 0x07, 0x07, 0x97, 0xc7, 0xfd,
  0x0f, 0x0f, 0x0f, 0x2f, 0x8f, 0xfd, 0x0f, 0x0f, 0x0f, 0x5f, 0x0f, 0xfd,
  0x1f, 0x1f, 0x1f, 0xbf, 0x1f, 0xfd, 0x1f, 0x1f, 0x1f, 0x5f, 0x1f, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf9, 0x07, 0x07, 0x07, 0x87, 0xd7, 0xe5,
  0x0f, 0x0f, 0x0f, 0x0f, 0xaf, 0xcd, 0x0f, 0x0f, 0x0f, 0x0f, 0x5f, 0x8d,
  0x1f, 0x1f, 0x1f, 0x1f, 0xbf, 0x1d, 0x1f, 0x1f, 0x1f, 0x1f, 0x5f, 0x3d,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0x3f, 0x1f, 0x0f, 0x8f, 0xc7, 0xfd,
  0x3f, 0x1f, 0x0f, 0x0f, 0x8f, 0xed, 0x3f, 0x1f, 0x0f, 0x0f, 0x0f, 0xdd,
  0x3f, 0x1f, 0x1f, 0x1f, 0x1f, 0xbd, 0x3f, 0x1f, 0x1f, 0x1f, 0x1f, 0x7d,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xfb, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xfa, 0xf8, 0xff, 0xff, 0xff, 0xfb, 0xf5, 0xf0, 0xff, 0xff, 0xff, 0xfb,
  0x0a, 0x00, 0xff, 0xff, 0xff, 0xfb, 0x15, 0x01, 0xff, 0xff, 0xff, 0xfb,
  0x2b, 0x03, 0xff, 0xff, 0xff, 0xfb, 0x57, 0x07, 0xff, 0xff, 0xff, 0xfb,
  0xaf, 0x0f, 0xff, 0xff, 0xff, 0xfb, 0x5f, 0x1f, 0xff, 0xff, 0xff, 0xfb,
  0xf8, 0xfa, 0xfc, 0xff, 0xff, 0xfb, 0xf0, 0xf5, 0xf8, 0xff, 0xff, 0xfb,
  0x00, 0x0a, 0xf1, 0xff, 0xff, 0xfb, 0x01, 0x15, 0x03, 0xff, 0xff, 0xfb,
  0x03, 0x2b, 0x07, 0xff, 0xff, 0xfb, 0x07, 0x57, 0x0f, 0xff, 0xff, 0xfb,
  0x0f, 0xaf, 0x1f, 0xff, 0xff, 0xfb, 0x1f, 0x5f, 0x3f, 0xff, 0xff, 0xfb,
  0xf8, 0xfc, 0xfe, 0xfe, 0xff, 0xfb, 0xf0, 0xf8, 0xfd, 0xfc, 0xff, 0xfb,
  0x00, 0x00, 0x8a, 0xfb, 0xff, 0xfb, 0x01, 0x03, 0x17, 0x87, 0xff, 0xfb,
  0x03, 0x07, 0x2f, 0x0f, 0xff, 0xfb, 0x07, 0x0f, 0x5f, 0x1f, 0xff, 0xfb,
  0x0f, 0x1f, 0xbf, 0x1f, 0xff, 0xfb, 0x1f, 0x3f, 0x7f, 0x3f, 0xff, 0xfb,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb,
  0x0f, 0x0f, 0x0f, 0x2f, 0x8f, 0xfb, 0x1f, 0x1f, 0x1f, 0x5f, 0x1f, 0xfb,
  0x1f, 0x1f, 0x1f, 0xbf, 0x1f, 0xfb, 0x3f, 0x3f, 0x3f, 0x7f, 0x3f, 0xfb,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf9,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf3,
  0x0f, 0x0f, 0x0f, 0x0f, 0xaf, 0xcb, 0x1f, 0x1f, 0x1f, 0x1f, 0x5f, 0x9b,
  0x1f, 0x1f, 0x1f, 0x1f, 0xbf, 0x1b, 0x3f, 0x3f, 0x3f, 0x3f, 0x7f, 0x3b,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0x7f, 0x3f, 0x1f, 0x1f, 0x8f, 0xfb, 0x7f, 0x3f, 0x1f, 0x1f, 0x1f, 0xdb,
  0x7f, 0x3f, 0x1f, 0x1f, 0x1f, 0xbb, 0x7f, 0x3f, 0x3f, 0x3f, 0x3f, 0x7b,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xf7, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xfa, 0xf8, 0xff, 0xff, 0xff, 0xf7, 0xf5, 0xf0, 0xff, 0xff, 0xff, 0xf7,
  0xea, 0xe0, 0xff, 0xff, 0xff, 0xf7, 0x14, 0x00, 0xff, 0xff, 0xff, 0xf7,
  0x2b, 0x03, 0xff, 0xff, 0xff, 0xf7, 0x57, 0x07, 0xff, 0xff, 0xff, 0xf7,
  0xaf, 0x0f, 0xff, 0xff, 0xff, 0xf7, 0x5f, 0x1f, 0xff, 0xff, 0xff, 0xf7,
  0xf8, 0xfa, 0xfc, 0xff, 0xff, 0xf7, 0xf0, 0xf5, 0xf8, 0xff, 0xff, 0xf7,
  0xe0, 0xea, 0xf0, 0xff, 0xff, 0xf7, 0x00, 0x14, 0xe3, 0xff, 0xff, 0xf7,
  0x03, 0x2b, 0x07, 0xff, 0xff, 0xf7, 0x07, 0x57, 0x0f, 0xff, 0xff, 0xf7,
  0x0f, 0xaf, 0x1f, 0xff, 0xff, 0xf7, 0x1f, 0x5f, 0x3f, 0xff, 0xff, 0xf7,
  0xf8, 0xfc, 0xfe, 0xfe, 0xff, 0xf7, 0xf0, 0xf8, 0xfd, 0xfc, 0xff, 0xf7,
  0xe0, 0xf0, 0xfa, 0xf8, 0xff, 0xf7, 0x00, 0x00, 0x14, 0xf7, 0xff, 0xf7,
  0x03, 0x07, 0x2f, 0x0f, 0xff, 0xf7, 0x07, 0x0f, 0x5f, 0x1f, 0xff, 0xf7,
  0x0f, 0x1f, 0xbf, 0x3f, 0xff, 0xf7, 0x1f, 0x3f, 0x7f, 0x3f, 0xff, 0xf7,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xf7, 0xfc, 0xfc, 0xfc, 0xfd, 0xfc, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xef, 0xf7, 0x1f, 0x1f, 0x1f, 0x5f, 0x1f, 0xf7,
  0x3f, 0x3f, 0x3f, 0xbf, 0x3f, 0xf7, 0x3f, 0x3f, 0x3f, 0x7f, 0x3f, 0xf7,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xf6, 0xfc, 0xfc, 0xfc, 0xfc, 0xfd, 0xf4,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe7, 0x1f, 0x1f, 0x1f, 0x1f, 0x5f, 0x97,
  0x3f, 0x3f, 0x3f, 0x3f, 0xbf, 0x37, 0x3f, 0x3f, 0x3f, 0x3f, 0x7f, 0x37,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xf6, 0xfe, 0xfe, 0xfe, 0xfe, 0xfc, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0x7f, 0x7f, 0x3f, 0x3f, 0x1f, 0xf7,
  0x7f, 0x7f, 0x3f, 0x3f, 0x3f, 0xb7, 0x7f, 0x7f, 0x3f, 0x3f, 0x3f, 0x77,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xc2, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x85, 0x80, 0xff, 0xff, 0xff, 0xff,
  0x0a, 0x00, 0xff, 0xff, 0xff, 0xff, 0x14, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x29, 0x01, 0xff, 0xff, 0xff, 0xff, 0x53, 0x03, 0xff, 0xff, 0xff, 0xff,
  0xa7, 0x07, 0xff, 0xff, 0xff, 0xff, 0x4f, 0x0f, 0xff, 0xff, 0xff, 0xff,
  0xc0, 0xc2, 0xe0, 0xff, 0xff, 0xff, 0x80, 0x85, 0xc0, 0xff, 0xff, 0xff,
  0x00, 0x0a, 0x80, 0xff, 0xff, 0xff, 0x00, 0x14, 0x01, 0xff, 0xff, 0xff,
  0x01, 0x29, 0x03, 0xff, 0xff, 0xff, 0x03, 0x53, 0x07, 0xff, 0xff, 0xff,
  0x07, 0xa7, 0x0f, 0xff, 0xff, 0xff, 0x0f, 0x4f, 0x1f, 0xff, 0xff, 0xff,
  0xc0, 0xc0, 0xe2, 0xfc, 0xff, 0xff, 0x80, 0x80, 0xc5, 0xe0, 0xff, 0xff,
  0x00, 0x00, 0x8a, 0xc1, 0xff, 0xff, 0x00, 0x00, 0x15, 0x83, 0xff, 0xff,
  0x01, 0x01, 0x2b, 0x07, 0xff, 0xff, 0x03, 0x03, 0x57, 0x0f, 0xff, 0xff,
  0x07, 0x07, 0xaf, 0x0f, 0xff, 0xff, 0x0f, 0x0f, 0x5f, 0x1f, 0xff, 0xff,
  0xc0, 0xc0, 0xe0, 0xf2, 0xfe, 0xff, 0x80, 0x80, 0xc0, 0xe5, 0xf0, 0xff,
  0x00, 0x00, 0x80, 0xca, 0xe3, 0xff, 0x00, 0x00, 0x03, 0x97, 0xc7, 0xff,
  0x01, 0x03, 0x07, 0x2f, 0x87, 0xff, 0x03, 0x07, 0x0f, 0x5f, 0x0f, 0xff,
  0x07, 0x0f, 0x0f, 0xaf, 0x0f, 0xff, 0x0f, 0x1f, 0x1f, 0x5f, 0x1f, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0x02, 0x02, 0x82, 0xc2, 0xea, 0xf2, 0x07, 0x07, 0x07, 0x87, 0xd7, 0xe7,
  0x07, 0x07, 0x07, 0x07, 0xaf, 0xc7, 0x0f, 0x0f, 0x0f, 0x0f, 0x5f, 0x8f,
  0x0f, 0x0f, 0x0f, 0x0f, 0xaf, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x5f, 0x3f,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x02, 0x02, 0x82, 0xc2, 0xe2, 0xfa, 0x07, 0x07, 0x07, 0x87, 0xc7, 0xf7,
  0x07, 0x07, 0x07, 0x07, 0x87, 0xef, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0xdf,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0xbf, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x7f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf2, 0xf0, 0xff, 0xff, 0xff, 0xff, 0x85, 0x80, 0xff, 0xff, 0xff, 0xff,
  0x0a, 0x00, 0xff, 0xff, 0xff, 0xff, 0x14, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x29, 0x01, 0xff, 0xff, 0xff, 0xff, 0x53, 0x03, 0xff, 0xff, 0xff, 0xff,
  0xa7, 0x07, 0xff, 0xff, 0xff, 0xff, 0x4f, 0x0f, 0xff, 0xff, 0xff, 0xff,
  0xf0, 0xf2, 0xf8, 0xff, 0xff, 0xff, 0x80, 0x85, 0xc0, 0xff, 0xff, 0xff,
  0x00, 0x0a, 0x80, 0xff, 0xff, 0xff, 0x00, 0x14, 0x01, 0xff, 0xff, 0xff,
  0x01, 0x29, 0x03, 0xff, 0xff, 0xff, 0x03, 0x53, 0x07, 0xff, 0xff, 0xff,
  0x07, 0xa7, 0x0f, 0xff, 0xff, 0xff, 0x0f, 0x4f, 0x1f, 0xff, 0xff, 0xff,
  0xf0, 0xf0, 0xfa, 0xfc, 0xff, 0xff, 0x80, 0x80, 0xc5, 0xf8, 0xff, 0xff,
  0x00, 0x00, 0x8a, 0xc1, 0xff, 0xff, 0x00, 0x00, 0x15, 0x83, 0xff, 0xff,
  0x01, 0x01, 0x2b, 0x07, 0xff, 0xff, 0x03, 0x03, 0x57, 0x0f, 0xff, 0xff,
  0x07, 0x07, 0xaf, 0x1f, 0xff, 0xff, 0x0f, 0x0f, 0x5f, 0x1f, 0xff, 0xff,
  0xf0, 0xf8, 0xfc, 0xfe, 0xfe, 0xff, 0x80, 0x80, 0xc0, 0xe5, 0xfd, 0xff,
  0x00, 0x00, 0x80, 0xcb, 0xe3, 0xff, 0x00, 0x00, 0x03, 0x97, 0xc7, 0xff,
  0x01, 0x03, 0x07, 0x2f, 0x8f, 0xff, 0x03, 0x07, 0x0f, 0x5f, 0x0f, 0xff,
  0x07, 0x0f, 0x1f, 0xbf, 0x1f, 0xff, 0x0f, 0x1f, 0x1f, 0x5f, 0x1f, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0x07, 0x07, 0x07, 0x87, 0xd7, 0xe7,
  0x0f, 0x0f, 0x0f, 0x0f, 0xaf, 0xcf, 0x0f, 0x0f, 0x0f, 0x0f, 0x5f, 0x8f,
  0x1f, 0x1f, 0x1f, 0x1f, 0xbf, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x5f, 0x3f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x07, 0x07, 0x07, 0x87, 0xc7, 0xf7,
  0x0f, 0x0f, 0x0f, 0x0f, 0x8f, 0xef, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0xdf,
  0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0xbf, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x7f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf2, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xe5, 0xe0, 0xff, 0xff, 0xff, 0xff,
  0x0a, 0x00, 0xff, 0xff, 0xff, 0xff, 0x14, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x29, 0x01, 0xff, 0xff, 0xff, 0xff, 0x53, 0x03, 0xff, 0xff, 0xff, 0xff,
  0xa7, 0x07, 0xff, 0xff, 0xff, 0xff, 0x4f, 0x0f, 0xff, 0xff, 0xff, 0xff,
  0xf0, 0xf2, 0xf8, 0xff, 0xff, 0xff, 0xe0, 0xe5, 0xf0, 0xff, 0xff, 0xff,
  0x00, 0x0a, 0x80, 0xff, 0xff, 0xff, 0x00, 0x14, 0x01, 0xff, 0xff, 0xff,
  0x01, 0x29, 0x03, 0xff, 0xff, 0xff, 0x03, 0x53, 0x07, 0xff, 0xff, 0xff,
  0x07, 0xa7, 0x0f, 0xff, 0xff, 0xff, 0x0f, 0x4f, 0x1f, 0xff, 0xff, 0xff,
  0xf0, 0xf0, 0xfa, 0xfc, 0xff, 0xff, 0xe0, 0xe0, 0xf5, 0xf8, 0xff, 0xff,
  0x00, 0x00, 0x8a, 0xf1, 0xff, 0xff, 0x00, 0x00, 0x15, 0x83, 0xff, 0xff,
  0x01, 0x01, 0x2b, 0x07, 0xff, 0xff, 0x03, 0x03, 0x57, 0x0f, 0xff, 0xff,
  0x07, 0x07, 0xaf, 0x1f, 0xff, 0xff, 0x0f, 0x0f, 0x5f, 0x3f, 0xff, 0xff,
  0xf0, 0xf8, 0xfc, 0xfe, 0xfe, 0xff, 0xe0, 0xf0, 0xf8, 0xfd, 0xfc, 0xff,
  0x00, 0x00, 0x80, 0xca, 0xfb, 0xff, 0x00, 0x00, 0x03, 0x97, 0xc7, 0xff,
  0x01, 0x03, 0x07, 0x2f, 0x8f, 0xff, 0x03, 0x07, 0x0f, 0x5f, 0x1f, 0xff,
  0x07, 0x0f, 0x1f, 0xbf, 0x1f, 0xff, 0x0f, 0x1f, 0x3f, 0x7f, 0x3f, 0xff,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0x0f, 0x0f, 0x0f, 0x0f, 0xaf, 0xcf, 0x1f, 0x1f, 0x1f, 0x1f, 0x5f, 0x9f,
  0x1f, 0x1f, 0x1f, 0x1f, 0xbf, 0x1f, 0x3f, 0x3f, 0x3f, 0x3f, 0x7f, 0x3f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x0f, 0x0f, 0x0f, 0x0f, 0x8f, 0xef, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0xdf,
  0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0xbf, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x7f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xbf, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf2, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xe5, 0xe0, 0xff, 0xff, 0xff, 0xff,
  0xca, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x14, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x29, 0x01, 0xff, 0xff, 0xff, 0xff, 0x53, 0x03, 0xff, 0xff, 0xff, 0xff,
  0xa7, 0x07, 0xff, 0xff, 0xff, 0xff, 0x4f, 0x0f, 0xff, 0xff, 0xff, 0xff,
  0xf0, 0xf2, 0xf8, 0xff, 0xff, 0xff, 0xe0, 0xe5, 0xf0, 0xff, 0xff, 0xff,
  0xc0, 0xca, 0xe0, 0xff, 0xff, 0xff, 0x00, 0x14, 0x00, 0xff, 0xff, 0xff,
  0x01, 0x29, 0x03, 0xff, 0xff, 0xff, 0x03, 0x53, 0x07, 0xff, 0xff, 0xff,
  0x07, 0xa7, 0x0f, 0xff, 0xff, 0xff, 0x0f, 0x4f, 0x1f, 0xff, 0xff, 0xff,
  0xf0, 0xf0, 0xfa, 0xfc, 0xff, 0xff, 0xe0, 0xe0, 0xf5, 0xf8, 0xff, 0xff,
  0xc0, 0xc0, 0xea, 0xf0, 0xff, 0xff, 0x00, 0x00, 0x14, 0xe3, 0xff, 0xff,
  0x01, 0x01, 0x2b, 0x07, 0xff, 0xff, 0x03, 0x03, 0x57, 0x0f, 0xff, 0xff,
  0x07, 0x07, 0xaf, 0x1f, 0xff, 0xff, 0x0f, 0x0f, 0x5f, 0x3f, 0xff, 0xff,
  0xf0, 0xf8, 0xfc, 0xfe, 0xfe, 0xff, 0xe0, 0xf0, 0xf8, 0xfd, 0xfc, 0xff,
  0xc0, 0xe0, 0xf0, 0xfa, 0xf8, 0xff, 0x00, 0x00, 0x00, 0x94, 0xf7, 0xff,
  0x01, 0x03, 0x07, 0x2f, 0x8f, 0xff, 0x03, 0x07, 0x0f, 0x5f, 0x1f, 0xff,
  0x07, 0x0f, 0x1f, 0xbf, 0x3f, 0xff, 0x0f, 0x1f, 0x3f, 0x7f, 0x3f, 0xff,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfc, 0xfc, 0xfc, 0xfc, 0xfd, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0x1f, 0x1f, 0x1f, 0x1f, 0x5f, 0x9f,
  0x3f, 0x3f, 0x3f, 0x3f, 0xbf, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x7f, 0x3f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfc, 0xfe, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0xdf,
  0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0xbf, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x7f,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xc7, 0xff, 0xff, 0xff, 0xff, 0xff, 0x8f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x02, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x06, 0x02, 0xff, 0xff, 0xff, 0xff, 0x0e, 0x06, 0xff, 0xff, 0xff, 0xff,
  0x0e, 0x0e, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xff, 0xff, 0xff, 0x00, 0x00, 0x80, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
  0x02, 0x00, 0x00, 0xff, 0xff, 0xff, 0x06, 0x02, 0x00, 0xff, 0xff, 0xff,
  0x0e, 0x06, 0x03, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x07, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xff, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
  0x02, 0x00, 0x00, 0x00, 0xff, 0xff, 0x06, 0x02, 0x00, 0x01, 0xff, 0xff,
  0x0e, 0x06, 0x03, 0x03, 0xff, 0xff, 0x0f, 0x0f, 0x07, 0x07, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x02, 0x00, 0x00, 0x00, 0x01, 0xff, 0x06, 0x02, 0x00, 0x01, 0x03, 0xff,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0xff, 0x0f, 0x0f, 0x07, 0x07, 0x0f, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x02, 0x00, 0x00, 0x00, 0x01, 0x83, 0x06, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x0f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x02, 0x00, 0x00, 0x00, 0x01, 0x83, 0x06, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x0f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x02, 0x00, 0x00, 0x00, 0x01, 0x83, 0x06, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x0f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xc7, 0xff, 0xff, 0xff, 0xff, 0xff, 0x8f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0x02, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x06, 0x02, 0xff, 0xff, 0xff, 0xff, 0x0e, 0x06, 0xff, 0xff, 0xff, 0xff,
  0x1e, 0x0e, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xff, 0xff, 0xff, 0x00, 0x00, 0x80, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
  0x02, 0x00, 0x00, 0xff, 0xff, 0xff, 0x06, 0x02, 0x00, 0xff, 0xff, 0xff,
  0x0e, 0x06, 0x03, 0xff, 0xff, 0xff, 0x1f, 0x0f, 0x07, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xff, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
  0x02, 0x00, 0x00, 0x00, 0xff, 0xff, 0x06, 0x02, 0x00, 0x01, 0xff, 0xff,
  0x0e, 0x06, 0x03, 0x03, 0xff, 0xff, 0x1f, 0x0f, 0x07, 0x07, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x02, 0x00, 0x00, 0x00, 0x01, 0xff, 0x06, 0x02, 0x00, 0x01, 0x03, 0xff,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0xff, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x02, 0x00, 0x00, 0x00, 0x01, 0x83, 0x06, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x02, 0x00, 0x00, 0x00, 0x01, 0x83, 0x06, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x02, 0x00, 0x00, 0x00, 0x01, 0x83, 0x06, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xcf, 0xff, 0xff, 0xff, 0xff, 0xff, 0x8f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf4, 0xe8, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x04, 0x02, 0xff, 0xff, 0xff, 0xff, 0x0e, 0x06, 0xff, 0xff, 0xff, 0xff,
  0x1e, 0x0e, 0xff, 0xff, 0xff, 0xff, 0x3f, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0xf0, 0xe0, 0xc0, 0xff, 0xff, 0xff, 0x00, 0x00, 0x80, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x06, 0x02, 0x00, 0xff, 0xff, 0xff,
  0x0e, 0x06, 0x03, 0xff, 0xff, 0xff, 0x1f, 0x0f, 0x07, 0xff, 0xff, 0xff,
  0xf0, 0xe0, 0xc0, 0xe0, 0xff, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x06, 0x02, 0x00, 0x01, 0xff, 0xff,
  0x0e, 0x06, 0x03, 0x03, 0xff, 0xff, 0x1f, 0x0f, 0x07, 0x07, 0xff, 0xff,
  0xf0, 0xe0, 0xc0, 0xe0, 0xf0, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x01, 0xff, 0x06, 0x02, 0x00, 0x01, 0x03, 0xff,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0xff, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0xff,
  0xf0, 0xe0, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x83, 0x06, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0xf0, 0xe0, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x83, 0x06, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0xf0, 0xe0, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x83, 0x06, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x9f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xf8, 0xff, 0xff, 0xff, 0xff, 0x68, 0x50, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0x0a, 0x04, 0xff, 0xff, 0xff, 0xff,
  0x1e, 0x0e, 0xff, 0xff, 0xff, 0xff, 0x3f, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf0, 0xe0, 0xff, 0xff, 0xff, 0x60, 0x40, 0x80, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x02, 0x00, 0x00, 0xff, 0xff, 0xff,
  0x0e, 0x06, 0x03, 0xff, 0xff, 0xff, 0x1f, 0x0f, 0x07, 0xff, 0xff, 0xff,
  0xf8, 0xf0, 0xe0, 0xe0, 0xff, 0xff, 0x60, 0x40, 0x80, 0xc0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x02, 0x00, 0x00, 0x01, 0xff, 0xff,
  0x0e, 0x06, 0x03, 0x03, 0xff, 0xff, 0x1f, 0x0f, 0x07, 0x07, 0xff, 0xff,
  0xf8, 0xf0, 0xe0, 0xe0, 0xf0, 0xff, 0x60, 0x40, 0x80, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x01, 0xff, 0x02, 0x00, 0x00, 0x01, 0x03, 0xff,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0xff, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0xff,
  0xf8, 0xf0, 0xe0, 0xe0, 0xf0, 0xf8, 0x60, 0x40, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x83, 0x02, 0x00, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0xf8, 0xf0, 0xe0, 0xe0, 0xf0, 0xf8, 0x60, 0x40, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x83, 0x02, 0x00, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0xf8, 0xf0, 0xe0, 0xe0, 0xf0, 0xf8, 0x60, 0x40, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x83, 0x02, 0x00, 0x00, 0x01, 0x03, 0x07,
  0x0e, 0x06, 0x03, 0x03, 0x07, 0x0f, 0x1f, 0x0f, 0x07, 0x07, 0x0f, 0x1f,
  0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf2, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe2, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xc6, 0xff, 0xff, 0xff, 0xff, 0xff, 0x8e, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x1e, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3e, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x02, 0x02, 0xff, 0xff, 0xff, 0xff, 0x02, 0x02, 0xff, 0xff, 0xff, 0xff,
  0x06, 0x07, 0xff, 0xff, 0xff, 0xff, 0x0e, 0x07, 0xff, 0xff, 0xff, 0xff,
  0x0e, 0x0f, 0xff, 0xff, 0xff, 0xff, 0x0e, 0x0f, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x02, 0x02, 0x00, 0xff, 0xff, 0xff,
  0x06, 0x06, 0x02, 0xff, 0xff, 0xff, 0x06, 0x06, 0x06, 0xff, 0xff, 0xff,
  0x0e, 0x0e, 0x0f, 0xff, 0xff, 0xff, 0x0e, 0x0f, 0x0f, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xff, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0x02, 0x00, 0x00, 0x00, 0xff, 0xff,
  0x06, 0x02, 0x00, 0x00, 0xff, 0xff, 0x06, 0x06, 0x02, 0x01, 0xff, 0xff,
  0x0e, 0x0e, 0x07, 0x03, 0xff, 0xff, 0x0e, 0x0f, 0x0f, 0x07, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x02, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x06, 0x02, 0x00, 0x00, 0x01, 0xff, 0x06, 0x06, 0x02, 0x01, 0x03, 0xff,
  0x0e, 0x0e, 0x07, 0x03, 0x07, 0xff, 0x0e, 0x0f, 0x0f, 0x07, 0x0f, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x02, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x06, 0x02, 0x00, 0x00, 0x01, 0x83, 0x06, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x0e, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x0e, 0x0f, 0x0f, 0x07, 0x0f, 0x1f,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x02, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x06, 0x02, 0x00, 0x00, 0x01, 0x83, 0x06, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x0e, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x0e, 0x0f, 0x0f, 0x07, 0x0f, 0x1f,
  0xfe, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x0e, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x0e, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x0e, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x0e, 0x02, 0x00, 0x00, 0x01, 0x83, 0x0e, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x0e, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x0e, 0x0f, 0x0f, 0x07, 0x0f, 0x1f,
  0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe5, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xc5, 0xff, 0xff, 0xff, 0xff, 0xff, 0x8d, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x1d, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3d, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0x05, 0x07, 0xff, 0xff, 0xff, 0xff,
  0x05, 0x07, 0xff, 0xff, 0xff, 0xff, 0x0d, 0x0f, 0xff, 0xff, 0xff, 0xff,
  0x1d, 0x0f, 0xff, 0xff, 0xff, 0xff, 0x1d, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xfb, 0xff, 0xff, 0xff, 0xff, 0x00, 0x02, 0x00, 0xff, 0xff, 0xff,
  0x04, 0x06, 0x02, 0xff, 0xff, 0xff, 0x0c, 0x0e, 0x06, 0xff, 0xff, 0xff,
  0x0c, 0x0e, 0x0f, 0xff, 0xff, 0xff, 0x1d, 0x1f, 0x1f, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xff, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
  0x04, 0x02, 0x00, 0x00, 0xff, 0xff, 0x0c, 0x06, 0x02, 0x01, 0xff, 0xff,
  0x0c, 0x0e, 0x07, 0x03, 0xff, 0xff, 0x1d, 0x1f, 0x0f, 0x07, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x04, 0x02, 0x00, 0x00, 0x01, 0xff, 0x0c, 0x06, 0x02, 0x01, 0x03, 0xff,
  0x0c, 0x0e, 0x07, 0x03, 0x07, 0xff, 0x1d, 0x1f, 0x0f, 0x07, 0x0f, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x04, 0x02, 0x00, 0x00, 0x01, 0x83, 0x0c, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x0c, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x1d, 0x1f, 0x0f, 0x07, 0x0f, 0x1f,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x04, 0x02, 0x00, 0x00, 0x01, 0x83, 0x0c, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x0c, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x1d, 0x1f, 0x0f, 0x07, 0x0f, 0x1f,
  0xfc, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x7c, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x1c, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x1c, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x1c, 0x02, 0x00, 0x00, 0x01, 0x83, 0x1c, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x1c, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x1d, 0x1f, 0x0f, 0x07, 0x0f, 0x1f,
  0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xcb, 0xff, 0xff, 0xff, 0xff, 0xff, 0x8b, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x1b, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3b, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfa, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x0b, 0x0f, 0xff, 0xff, 0xff, 0xff, 0x0b, 0x0f, 0xff, 0xff, 0xff, 0xff,
  0x1b, 0x1f, 0xff, 0xff, 0xff, 0xff, 0x3b, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf4, 0xe8, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0x02, 0x04, 0x02, 0xff, 0xff, 0xff, 0x0a, 0x0e, 0x06, 0xff, 0xff, 0xff,
  0x1a, 0x1e, 0x0f, 0xff, 0xff, 0xff, 0x1b, 0x1f, 0x1f, 0xff, 0xff, 0xff,
  0xf8, 0xf0, 0xe0, 0xe0, 0xff, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
  0x02, 0x00, 0x00, 0x00, 0xff, 0xff, 0x0a, 0x06, 0x02, 0x01, 0xff, 0xff,
  0x1a, 0x0e, 0x07, 0x03, 0xff, 0xff, 0x1b, 0x1f, 0x0f, 0x07, 0xff, 0xff,
  0xf8, 0xf0, 0xe0, 0xe0, 0xf0, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x02, 0x00, 0x00, 0x00, 0x01, 0xff, 0x0a, 0x06, 0x02, 0x01, 0x03, 0xff,
  0x1a, 0x0e, 0x07, 0x03, 0x07, 0xff, 0x1b, 0x1f, 0x0f, 0x07, 0x0f, 0xff,
  0xf8, 0xf0, 0xe0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x02, 0x00, 0x00, 0x00, 0x01, 0x83, 0x0a, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x1a, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x1b, 0x1f, 0x0f, 0x07, 0x0f, 0x1f,
  0xf8, 0xf0, 0xe0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x02, 0x00, 0x00, 0x00, 0x01, 0x83, 0x0a, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x1a, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x1b, 0x1f, 0x0f, 0x07, 0x0f, 0x1f,
  0xfa, 0xf0, 0xe0, 0xe0, 0xf0, 0xf8, 0x7a, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x7a, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x3a, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x3a, 0x00, 0x00, 0x00, 0x01, 0x83, 0x3a, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x3a, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x3b, 0x1f, 0x0f, 0x07, 0x0f, 0x1f,
  0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0x97, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x17, 0xff, 0xff, 0xff, 0xff, 0xff, 0x37, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf4, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xf4, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xf3, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xe7, 0xff, 0xff, 0xff, 0xff, 0xff, 0x17, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0x17, 0x1f, 0xff, 0xff, 0xff, 0xff, 0x37, 0x3f, 0xff, 0xff, 0xff, 0xff,
  0xf4, 0xfc, 0xf8, 0xff, 0xff, 0xff, 0x70, 0x68, 0xd0, 0xff, 0xff, 0xff,
  0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xf7, 0xef, 0xff, 0xff, 0xff, 0xff, 0x06, 0x0a, 0x04, 0xff, 0xff, 0xff,
  0x16, 0x1e, 0x0f, 0xff, 0xff, 0xff, 0x37, 0x3f, 0x1f, 0xff, 0xff, 0xff,
  0xf4, 0xf8, 0xf0, 0xe0, 0xff, 0xff, 0x70, 0x60, 0xc0, 0xc0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x06, 0x02, 0x00, 0x01, 0xff, 0xff,
  0x16, 0x0e, 0x07, 0x03, 0xff, 0xff, 0x37, 0x1f, 0x0f, 0x07, 0xff, 0xff,
  0xf4, 0xf8, 0xf0, 0xe0, 0xf0, 0xff, 0x70, 0x60, 0xc0, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x01, 0xff, 0x06, 0x02, 0x00, 0x01, 0x03, 0xff,
  0x16, 0x0e, 0x07, 0x03, 0x07, 0xff, 0x37, 0x1f, 0x0f, 0x07, 0x0f, 0xff,
  0xf4, 0xf8, 0xf0, 0xe0, 0xf0, 0xf8, 0x70, 0x60, 0xc0, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x83, 0x06, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x16, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x37, 0x1f, 0x0f, 0x07, 0x0f, 0x1f,
  0xf4, 0xf8, 0xf0, 0xe0, 0xf0, 0xf8, 0x70, 0x60, 0xc0, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x83, 0x06, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x16, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x37, 0x1f, 0x0f, 0x07, 0x0f, 0x1f,
  0xf6, 0xf8, 0xf0, 0xe0, 0xf0, 0xf8, 0x76, 0x60, 0xc0, 0xc0, 0xe0, 0xf0,
  0x76, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x76, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x76, 0x00, 0x00, 0x00, 0x01, 0x83, 0x76, 0x02, 0x00, 0x01, 0x03, 0x07,
  0x76, 0x0e, 0x07, 0x03, 0x07, 0x0f, 0x77, 0x1f, 0x0f, 0x07, 0x0f, 0x1f,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xf3, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xc7, 0xfe, 0xff, 0xff, 0xff, 0xff, 0x8f, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xfe, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0x02, 0x02, 0xff, 0xff, 0xff, 0xff, 0x02, 0x02, 0xff, 0xff, 0xff, 0xff,
  0x07, 0x06, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x06, 0xff, 0xff, 0xff, 0xff,
  0x0f, 0x0e, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x0e, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0x02, 0x02, 0x02, 0xff, 0xff, 0xff, 0x02, 0x02, 0x02, 0xff, 0xff, 0xff,
  0x07, 0x06, 0x07, 0xff, 0xff, 0xff, 0x07, 0x06, 0x07, 0xff, 0xff, 0xff,
  0x0f, 0x0e, 0x0f, 0xff, 0xff, 0xff, 0x0f, 0x0e, 0x0f, 0xff, 0xff, 0xff,
  0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff,
  0x02, 0x00, 0x00, 0x80, 0xff, 0xff, 0x02, 0x02, 0x02, 0x00, 0xff, 0xff,
  0x06, 0x06, 0x06, 0x02, 0xff, 0xff, 0x06, 0x06, 0x06, 0x07, 0xff, 0xff,
  0x0e, 0x0e, 0x0f, 0x0f, 0xff, 0xff, 0x0f, 0x0e, 0x0f, 0x0f, 0xff, 0xff,
  0xfe, 0x80, 0xc0, 0xe0, 0xf0, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x02, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x02, 0x02, 0x00, 0x00, 0x80, 0xff,
  0x06, 0x06, 0x02, 0x00, 0x01, 0xff, 0x06, 0x06, 0x06, 0x03, 0x03, 0xff,
  0x0e, 0x0e, 0x0f, 0x07, 0x07, 0xff, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0xff,
  0xc0, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x02, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x02, 0x02, 0x00, 0x00, 0x80, 0xc1,
  0x06, 0x06, 0x02, 0x00, 0x01, 0x83, 0x06, 0x06, 0x06, 0x03, 0x03, 0x07,
  0x0e, 0x0e, 0x0f, 0x07, 0x07, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x1f,
  0xfe, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x06, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x06, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x06, 0x02, 0x00, 0x00, 0x80, 0xc1,
  0x06, 0x06, 0x02, 0x00, 0x01, 0x83, 0x06, 0x06, 0x06, 0x03, 0x03, 0x07,
  0x0e, 0x0e, 0x0f, 0x07, 0x07, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x1f,
  0xfe, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x0e, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x0e, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x0e, 0x02, 0x00, 0x00, 0x80, 0xc1,
  0x0e, 0x06, 0x02, 0x00, 0x01, 0x83, 0x0e, 0x06, 0x06, 0x03, 0x03, 0x07,
  0x0e, 0x0e, 0x0f, 0x07, 0x07, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x1f,
  0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xf7, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xe7, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xc7, 0xfd, 0xff, 0xff, 0xff, 0xff, 0x8f, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xfd, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xfd, 0xff, 0xff, 0xff, 0xff, 0x07, 0x05, 0xff, 0xff, 0xff, 0xff,
  0x07, 0x05, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x0d, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0x0d, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x1d, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xf9, 0xff, 0xff, 0xff, 0xff, 0x07, 0x05, 0x07, 0xff, 0xff, 0xff,
  0x07, 0x05, 0x07, 0xff, 0xff, 0xff, 0x0f, 0x0d, 0x0f, 0xff, 0xff, 0xff,
  0x0f, 0x0d, 0x0f, 0xff, 0xff, 0xff, 0x1f, 0x1d, 0x1f, 0xff, 0xff, 0xff,
  0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xfd, 0xfb, 0xff, 0xff, 0xff, 0x06, 0x00, 0x02, 0x00, 0xff, 0xff,
  0x06, 0x04, 0x06, 0x02, 0xff, 0xff, 0x0e, 0x0c, 0x0e, 0x07, 0xff, 0xff,
  0x0e, 0x0c, 0x0f, 0x0f, 0xff, 0xff, 0x1f, 0x1d, 0x1f, 0x1f, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xff, 0x7e, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x06, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x06, 0x04, 0x02, 0x00, 0x01, 0xff, 0x0e, 0x0c, 0x06, 0x03, 0x03, 0xff,
  0x0e, 0x0c, 0x0f, 0x07, 0x07, 0xff, 0x1f, 0x1d, 0x1f, 0x0f, 0x0f, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x06, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x06, 0x04, 0x02, 0x00, 0x01, 0x83, 0x0e, 0x0c, 0x06, 0x03, 0x03, 0x07,
  0x0e, 0x0c, 0x0f, 0x07, 0x07, 0x0f, 0x1f, 0x1d, 0x1f, 0x0f, 0x0f, 0x1f,
  0xfe, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x7e, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x0e, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x0e, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x0e, 0x04, 0x02, 0x00, 0x01, 0x83, 0x0e, 0x0c, 0x06, 0x03, 0x03, 0x07,
  0x0e, 0x0c, 0x0f, 0x07, 0x07, 0x0f, 0x1f, 0x1d, 0x1f, 0x0f, 0x0f, 0x1f,
  0xfe, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x7e, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x1e, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x1e, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x1e, 0x04, 0x02, 0x00, 0x01, 0x83, 0x1e, 0x0c, 0x06, 0x03, 0x03, 0x07,
  0x1e, 0x0c, 0x0f, 0x07, 0x07, 0x0f, 0x1f, 0x1d, 0x1f, 0x0f, 0x0f, 0x1f,
  0xfe, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xef, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xcf, 0xfb, 0xff, 0xff, 0xff, 0xff, 0x8f, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xfb, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff,
  0x0f, 0x0b, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x0b, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0x1b, 0xff, 0xff, 0xff, 0xff, 0x3f, 0x1b, 0xff, 0xff, 0xff, 0xff,
  0xfe, 0xfa, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xf9, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xf3, 0xff, 0xff, 0xff, 0xff,
  0x0f, 0x0b, 0x0f, 0xff, 0xff, 0xff, 0x0f, 0x0b, 0x0f, 0xff, 0xff, 0xff,
  0x1f, 0x1b, 0x1f, 0xff, 0xff, 0xff, 0x1f, 0x1b, 0x1f, 0xff, 0xff, 0xff,
  0xfe, 0xf8, 0xf4, 0xe8, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff,
  0x0e, 0x02, 0x04, 0x02, 0xff, 0xff, 0x0e, 0x0a, 0x0e, 0x07, 0xff, 0xff,
  0x1e, 0x1a, 0x1f, 0x0f, 0xff, 0xff, 0x1f, 0x1b, 0x1f, 0x1f, 0xff, 0xff,
  0xfe, 0xf8, 0xf0, 0xe0, 0xf0, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x7e, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x0e, 0x02, 0x00, 0x00, 0x01, 0xff, 0x0e, 0x0a, 0x06, 0x03, 0x03, 0xff,
  0x1e, 0x1a, 0x0f, 0x07, 0x07, 0xff, 0x1f, 0x1b, 0x1f, 0x0f, 0x0f, 0xff,
  0xfe, 0xf8, 0xf0, 0xe0, 0xf0, 0xf8, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x0e, 0x02, 0x00, 0x00, 0x01, 0x83, 0x0e, 0x0a, 0x06, 0x03, 0x03, 0x07,
  0x1e, 0x1a, 0x0f, 0x07, 0x07, 0x0f, 0x1f, 0x1b, 0x1f, 0x0f, 0x0f, 0x1f,
  0xfe, 0xf8, 0xf0, 0xe0, 0xf0, 0xf8, 0x7e, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x7e, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x1e, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x1e, 0x02, 0x00, 0x00, 0x01, 0x83, 0x1e, 0x0a, 0x06, 0x03, 0x03, 0x07,
  0x1e, 0x1a, 0x0f, 0x07, 0x07, 0x0f, 0x1f, 0x1b, 0x1f, 0x0f, 0x0f, 0x1f,
  0xfe, 0xf8, 0xf0, 0xe0, 0xf0, 0xf8, 0x7e, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x7e, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x3e, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x3e, 0x02, 0x00, 0x00, 0x01, 0x83, 0x3e, 0x0a, 0x06, 0x03, 0x03, 0x07,
  0x3e, 0x1a, 0x0f, 0x07, 0x07, 0x0f, 0x3f, 0x1b, 0x1f, 0x0f, 0x0f, 0x1f,
  0xfc, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xdf, 0xf7, 0xff, 0xff, 0xff, 0xff, 0x9f, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xf7, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xf4, 0xff, 0xff, 0xff, 0xff,
  0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xf7, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x17, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0x17, 0xff, 0xff, 0xff, 0xff, 0x3f, 0x37, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xf4, 0xfc, 0xff, 0xff, 0xff, 0xfc, 0xf4, 0xfc, 0xff, 0xff, 0xff,
  0xff, 0xf3, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xe7, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x17, 0x1f, 0xff, 0xff, 0xff,
  0x1f, 0x17, 0x1f, 0xff, 0xff, 0xff, 0x3f, 0x37, 0x3f, 0xff, 0xff, 0xff,
  0xfc, 0xf4, 0xfc, 0xf8, 0xff, 0xff, 0x7c, 0x70, 0xe8, 0xd0, 0xff, 0xff,
  0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff, 0xff,
  0xff, 0xf7, 0xef, 0xff, 0xff, 0xff, 0x1e, 0x06, 0x0a, 0x05, 0xff, 0xff,
  0x1e, 0x16, 0x1f, 0x0f, 0xff, 0xff, 0x3f, 0x37, 0x3f, 0x1f, 0xff, 0xff,
  0xfc, 0xf4, 0xf8, 0xf0, 0xf0, 0xff, 0x7c, 0x70, 0xe0, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x7e, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x01, 0xff, 0x1e, 0x06, 0x02, 0x01, 0x03, 0xff,
  0x1e, 0x16, 0x0f, 0x07, 0x07, 0xff, 0x3f, 0x37, 0x1f, 0x0f, 0x0f, 0xff,
  0xfc, 0xf4, 0xf8, 0xf0, 0xf0, 0xf8, 0x7c, 0x70, 0xe0, 0xc0, 0xe0, 0xf0,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x83, 0x1e, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x1e, 0x16, 0x0f, 0x07, 0x07, 0x0f, 0x3f, 0x37, 0x1f, 0x0f, 0x0f, 0x1f,
  0xfe, 0xf4, 0xf8, 0xf0, 0xf0, 0xf8, 0x7e, 0x70, 0xe0, 0xc0, 0xe0, 0xf0,
  0x7e, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x7e, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x3e, 0x00, 0x00, 0x00, 0x01, 0x83, 0x3e, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x3e, 0x16, 0x0f, 0x07, 0x07, 0x0f, 0x3f, 0x37, 0x1f, 0x0f, 0x0f, 0x1f,
  0xfe, 0xf4, 0xf8, 0xf0, 0xf0, 0xf8, 0x7e, 0x70, 0xe0, 0xc0, 0xe0, 0xf0,
  0x7e, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x7e, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x7e, 0x00, 0x00, 0x00, 0x01, 0x83, 0x7e, 0x06, 0x02, 0x01, 0x03, 0x07,
  0x7e, 0x16, 0x0f, 0x07, 0x07, 0x0f, 0x7f, 0x37, 0x1f, 0x0f, 0x0f, 0x1f,
  0xfc, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xf8, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0xf1, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xe3, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0xc7, 0xff, 0xfe, 0xff, 0xff, 0xff, 0x8f, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xfe, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xfe, 0xff, 0xff, 0xff,
  0xfc, 0xff, 0xfe, 0xff, 0xff, 0xff, 0x08, 0x03, 0xfe, 0xff, 0xff, 0xff,
  0x01, 0x03, 0xfe, 0xff, 0xff, 0xff, 0x03, 0x03, 0xfe, 0xff, 0xff, 0xff,
  0x07, 0x07, 0xfe, 0xff, 0xff, 0xff, 0x0f, 0x07, 0xfe, 0xff, 0xff, 0xff,
  0x0f, 0x0f, 0xfe, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0xfe, 0xff, 0xff, 0xff,
  0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff,
  0x00, 0x02, 0x02, 0xff, 0xff, 0xff, 0x02, 0x02, 0x02, 0xff, 0xff, 0xff,
  0x07, 0x07, 0x06, 0xff, 0xff, 0xff, 0x07, 0x07, 0x06, 0xff, 0xff, 0xff,
  0x0f, 0x0f, 0x0e, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0e, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xfc, 0xff, 0xff, 0xff,
  0x02, 0x02, 0x02, 0x82, 0xff, 0xff, 0x02, 0x02, 0x02, 0x02, 0xff, 0xff,
  0x07, 0x07, 0x06, 0x07, 0xff, 0xff, 0x07, 0x07, 0x06, 0x07, 0xff, 0xff,
  0x0f, 0x0f, 0x0e, 0x0f, 0xff, 0xff, 0x0f, 0x0f, 0x0e, 0x0f, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff,
  0x02, 0x02, 0x00, 0x80, 0xc0, 0xff, 0x02, 0x02, 0x02, 0x02, 0x80, 0xff,
  0x06, 0x06, 0x06, 0x06, 0x03, 0xff, 0x06, 0x06, 0x06, 0x07, 0x07, 0xff,
  0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0xff, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0xff,
  0xfe, 0xfe, 0xc0, 0xe0, 0xf0, 0xf8, 0x02, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x02, 0x02, 0x00, 0x80, 0xc0, 0xe0, 0x02, 0x02, 0x02, 0x00, 0x80, 0xc1,
  0x06, 0x06, 0x06, 0x02, 0x03, 0x83, 0x06, 0x06, 0x06, 0x07, 0x07, 0x07,
  0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x1f,
  0xfe, 0xc0, 0xc0, 0xe0, 0xf0, 0xf8, 0x06, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x06, 0x02, 0x00, 0x80, 0xc0, 0xe0, 0x06, 0x02, 0x02, 0x00, 0x80, 0xc1,
  0x06, 0x06, 0x06, 0x02, 0x03, 0x83, 0x06, 0x06, 0x06, 0x07, 0x07, 0x07,
  0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x1f,
  0xfe, 0xfe, 0xc0, 0xe0, 0xf8, 0xf8, 0x0e, 0x06, 0x80, 0xc0, 0xe0, 0xf0,
  0x0e, 0x06, 0x00, 0x80, 0xc0, 0xe0, 0x0e, 0x06, 0x02, 0x00, 0x80, 0xc1,
  0x0e, 0x06, 0x06, 0x02, 0x03, 0x83, 0x0e, 0x06, 0x06, 0x07, 0x07, 0x07,
  0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x1f,
  0xfc, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xf8, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0xf1, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xe3, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0xc7, 0xff, 0xfd, 0xff, 0xff, 0xff, 0x8f, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xfd, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0xfc, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xf8, 0xff, 0xfd, 0xff, 0xff, 0xff,
  0x11, 0x07, 0xfd, 0xff, 0xff, 0xff, 0x03, 0x07, 0xfd, 0xff, 0xff, 0xff,
  0x07, 0x07, 0xfd, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0xfd, 0xff, 0xff, 0xff,
  0x1f, 0x0f, 0xfd, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xfd, 0xff, 0xff, 0xff, 0x03, 0x07, 0x05, 0xff, 0xff, 0xff,
  0x07, 0x07, 0x05, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0d, 0xff, 0xff, 0xff,
  0x0f, 0x0f, 0x0d, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x1d, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xf9, 0xff, 0xff, 0xff, 0x07, 0x07, 0x05, 0x07, 0xff, 0xff,
  0x07, 0x07, 0x05, 0x07, 0xff, 0xff, 0x0f, 0x0f, 0x0d, 0x0f, 0xff, 0xff,
  0x0f, 0x0f, 0x0d, 0x0f, 0xff, 0xff, 0x1f, 0x1f, 0x1d, 0x1f, 0xff, 0xff,
  0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xfd, 0xfb, 0xff, 0xff, 0x06, 0x06, 0x00, 0x02, 0x80, 0xff,
  0x06, 0x06, 0x04, 0x06, 0x03, 0xff, 0x0e, 0x0e, 0x0c, 0x0f, 0x07, 0xff,
  0x0f, 0x0f, 0x0d, 0x0f, 0x0f, 0xff, 0x1f, 0x1f, 0x1d, 0x1f, 0x1f, 0xff,
  0xfe, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x7e, 0x7e, 0x80, 0xc0, 0xe0, 0xf0,
  0x06, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x06, 0x06, 0x00, 0x00, 0x80, 0xc1,
  0x06, 0x06, 0x04, 0x02, 0x03, 0x83, 0x0e, 0x0e, 0x0c, 0x07, 0x07, 0x07,
  0x0f, 0x0f, 0x0d, 0x0f, 0x0f, 0x0f, 0x1f, 0x1f, 0x1d, 0x1f, 0x1f, 0x1f,
  0xfe, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x7e, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x0e, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x0e, 0x06, 0x00, 0x00, 0x80, 0xc1,
  0x0e, 0x06, 0x04, 0x02, 0x03, 0x83, 0x0e, 0x0e, 0x0c, 0x07, 0x07, 0x07,
  0x0f, 0x0f, 0x0d, 0x0f, 0x0f, 0x0f, 0x1f, 0x1f, 0x1d, 0x1f, 0x1f, 0x1f,
  0xfe, 0xfe, 0xc0, 0xf0, 0xf8, 0xf8, 0x7e, 0x7e, 0x80, 0xc0, 0xf0, 0xf0,
  0x1e, 0x0e, 0x00, 0x80, 0xc0, 0xe0, 0x1e, 0x0e, 0x00, 0x00, 0x80, 0xc1,
  0x1e, 0x0e, 0x04, 0x02, 0x03, 0x83, 0x1e, 0x0e, 0x0c, 0x07, 0x07, 0x07,
  0x1f, 0x0f, 0x0d, 0x0f, 0x0f, 0x0f, 0x1f, 0x1f, 0x1d, 0x1f, 0x1f, 0x1f,
  0xfc, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xf8, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xf1, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xe3, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xc7, 0xff, 0xfb, 0xff, 0xff, 0xff, 0x8f, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xfb, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xfc, 0xfe, 0xfb, 0xff, 0xff, 0xff, 0xf8, 0xfe, 0xfb, 0xff, 0xff, 0xff,
  0xf1, 0xff, 0xfb, 0xff, 0xff, 0xff, 0x23, 0x0f, 0xfb, 0xff, 0xff, 0xff,
  0x07, 0x0f, 0xfb, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0xfb, 0xff, 0xff, 0xff,
  0x1f, 0x1f, 0xfb, 0xff, 0xff, 0xff, 0x3f, 0x1f, 0xfb, 0xff, 0xff, 0xff,
  0xfc, 0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff,
  0x07, 0x0f, 0x0b, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0b, 0xff, 0xff, 0xff,
  0x1f, 0x1f, 0x1b, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x1b, 0xff, 0xff, 0xff,
  0xfe, 0xfe, 0xfa, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xf9, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xf3, 0xff, 0xff, 0xff,
  0x0f, 0x0f, 0x0b, 0x0f, 0xff, 0xff, 0x0f, 0x0f, 0x0b, 0x0f, 0xff, 0xff,
  0x1f, 0x1f, 0x1b, 0x1f, 0xff, 0xff, 0x1f, 0x1f, 0x1b, 0x1f, 0xff, 0xff,
  0xfe, 0xfe, 0xf8, 0xf4, 0xf8, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff,
  0x0e, 0x0e, 0x02, 0x04, 0x03, 0xff, 0x0e, 0x0e, 0x0a, 0x0f, 0x07, 0xff,
  0x1f, 0x1f, 0x1b, 0x1f, 0x0f, 0xff, 0x1f, 0x1f, 0x1b, 0x1f, 0x1f, 0xff,
  0xfe, 0xfe, 0xf8, 0xf0, 0xf8, 0xf8, 0x7e, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x7e, 0x7e, 0x00, 0x80, 0xc0, 0xe0, 0x0e, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x0e, 0x0e, 0x02, 0x00, 0x03, 0x83, 0x0e, 0x0e, 0x0a, 0x07, 0x07, 0x07,
  0x1f, 0x1f, 0x1b, 0x0f, 0x0f, 0x0f, 0x1f, 0x1f, 0x1b, 0x1f, 0x1f, 0x1f,
  0xfe, 0xfe, 0xf8, 0xf0, 0xf8, 0xf8, 0x7e, 0x00, 0x80, 0xc0, 0xe0, 0xf0,
  0x7e, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x1e, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x1e, 0x0e, 0x02, 0x00, 0x03, 0x83, 0x1e, 0x0e, 0x0a, 0x07, 0x07, 0x07,
  0x1f, 0x1f, 0x1b, 0x0f, 0x0f, 0x0f, 0x1f, 0x1f, 0x1b, 0x1f, 0x1f, 0x1f,
  0xfe, 0xfe, 0xf8, 0xf0, 0xf8, 0xf8, 0x7e, 0x7e, 0x80, 0xe0, 0xf0, 0xf0,
  0x7e, 0x7e, 0x00, 0x80, 0xe0, 0xe0, 0x3e, 0x1e, 0x00, 0x00, 0x80, 0xc1,
  0x3e, 0x1e, 0x02, 0x00, 0x03, 0x83, 0x3e, 0x1e, 0x0a, 0x07, 0x07, 0x07,
  0x3f, 0x1f, 0x1b, 0x0f, 0x0f, 0x0f, 0x3f, 0x1f, 0x1b, 0x1f, 0x1f, 0x1f,
  0xfc, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xf8, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0xf1, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xe3, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0xc7, 0xff, 0xf7, 0xff, 0xff, 0xff, 0x8f, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xf7, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xf7, 0xff, 0xff, 0xff, 0xf8, 0xfc, 0xf7, 0xff, 0xff, 0xff,
  0xf1, 0xfc, 0xf7, 0xff, 0xff, 0xff, 0xe3, 0xff, 0xf7, 0xff, 0xff, 0xff,
  0x47, 0x1f, 0xf7, 0xff, 0xff, 0xff, 0x0f, 0x1f, 0xf7, 0xff, 0xff, 0xff,
  0x1f, 0x1f, 0xf7, 0xff, 0xff, 0xff, 0x3f, 0x3f, 0xf7, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xf4, 0xff, 0xff, 0xff, 0xf8, 0xfc, 0xf4, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff, 0xff,
  0xff, 0xef, 0xf7, 0xff, 0xff, 0xff, 0x0f, 0x1f, 0x17, 0xff, 0xff, 0xff,
  0x1f, 0x1f, 0x17, 0xff, 0xff, 0xff, 0x3f, 0x3f, 0x37, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xf4, 0xfc, 0xff, 0xff, 0xfc, 0xfc, 0xf4, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xf3, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xe7, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x17, 0x1f, 0xff, 0xff,
  0x1f, 0x1f, 0x17, 0x1f, 0xff, 0xff, 0x3f, 0x3f, 0x37, 0x3f, 0xff, 0xff,
  0xfc, 0xfc, 0xf4, 0xfc, 0xf8, 0xff, 0xfc, 0xfc, 0xf0, 0xe8, 0xf0, 0xff,
  0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xf7, 0xef, 0xff, 0xff, 0x1e, 0x1e, 0x06, 0x0b, 0x07, 0xff,
  0x1f, 0x1f, 0x17, 0x1f, 0x0f, 0xff, 0x3f, 0x3f, 0x37, 0x3f, 0x1f, 0xff,
  0xfc, 0xfc, 0xf4, 0xf8, 0xf8, 0xf8, 0xfc, 0xfc, 0xf0, 0xe0, 0xf0, 0xf0,
  0x7c, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x7e, 0x7e, 0x00, 0x00, 0x80, 0xc1,
  0x1e, 0x00, 0x00, 0x00, 0x01, 0x83, 0x1e, 0x1e, 0x06, 0x03, 0x07, 0x07,
  0x1f, 0x1f, 0x17, 0x0f, 0x0f, 0x0f, 0x3f, 0x3f, 0x37, 0x1f, 0x1f, 0x1f,
  0xfe, 0xfc, 0xf4, 0xf8, 0xf8, 0xf8, 0xfe, 0xfc, 0xf0, 0xe0, 0xf0, 0xf0,
  0x7e, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0x7e, 0x00, 0x00, 0x00, 0x80, 0xc1,
  0x3e, 0x00, 0x00, 0x00, 0x01, 0x83, 0x3e, 0x1e, 0x06, 0x03, 0x07, 0x07,
  0x3f, 0x1f, 0x17, 0x0f, 0x0f, 0x0f, 0x3f, 0x3f, 0x37, 0x1f, 0x1f, 0x1f,
  0xfe, 0xfe, 0xf4, 0xf8, 0xf8, 0xf8, 0xfe, 0xfe, 0xf0, 0xe0, 0xf0, 0xf0,
  0x7e, 0x7e, 0x00, 0xc0, 0xe0, 0xe0, 0x7e, 0x7e, 0x00, 0x00, 0xc0, 0xc1,
  0x7e, 0x3e, 0x00, 0x00, 0x03, 0x83, 0x7e, 0x3e, 0x06, 0x03, 0x07, 0x07,
  0x7f, 0x3f, 0x17, 0x0f, 0x0f, 0x0f, 0x7f, 0x3f, 0x37, 0x1f, 0x1f, 0x1f,
  0xfc, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xf8, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xf1, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xe3, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xc7, 0xff, 0xff, 0xfe, 0xff, 0xff, 0x8f, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xfe, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xfc, 0x80, 0xff, 0xfe, 0xff, 0xff, 0x08, 0x00, 0xff, 0xfe, 0xff, 0xff,
  0x01, 0x01, 0xff, 0xfe, 0xff, 0xff, 0x03, 0x03, 0xff, 0xfe, 0xff, 0xff,
  0x07, 0x07, 0xff, 0xfe, 0xff, 0xff, 0x0f, 0x07, 0xff, 0xfe, 0xff, 0xff,
  0x0f, 0x0f, 0xff, 0xfe, 0xff, 0xff, 0x0f, 0x0f, 0xff, 0xfe, 0xff, 0xff,
  0xfc, 0x80, 0xff, 0xfe, 0xff, 0xff, 0x00, 0x00, 0x83, 0xfe, 0xff, 0xff,
  0x00, 0x00, 0x03, 0xfe, 0xff, 0xff, 0x02, 0x02, 0x03, 0xfe, 0xff, 0xff,
  0x07, 0x07, 0x07, 0xfe, 0xff, 0xff, 0x07, 0x07, 0x07, 0xfe, 0xff, 0xff,
  0x0f, 0x0f, 0x0f, 0xfe, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0xfe, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff,
  0x00, 0x00, 0x02, 0x82, 0xff, 0xff, 0x02, 0x02, 0x02, 0x02, 0xff, 0xff,
  0x07, 0x07, 0x07, 0x06, 0xff, 0xff, 0x07, 0x07, 0x07, 0x06, 0xff, 0xff,
  0x0f, 0x0f, 0x0f, 0x0e, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0e, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xfc, 0xff, 0xff,
  0x02, 0x02, 0x02, 0x82, 0xc2, 0xff, 0x02, 0x02, 0x02, 0x02, 0x82, 0xff,
  0x07, 0x07, 0x07, 0x06, 0x07, 0xff, 0x07, 0x07, 0x07, 0x06, 0x07, 0xff,
  0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0xff, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0xff,
  0xff, 0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff,
  0x02, 0x02, 0x02, 0x80, 0xc0, 0xe0, 0x02, 0x02, 0x02, 0x02, 0x82, 0xc1,
  0x06, 0x06, 0x06, 0x06, 0x07, 0x83, 0x07, 0x07, 0x07, 0x06, 0x07, 0x07,
  0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x1f,
  0xfe, 0xfe, 0xfe, 0xe0, 0xfc, 0xf8, 0x06, 0x02, 0x80, 0xc0, 0xe0, 0xf0,
  0x06, 0x02, 0x02, 0x80, 0xc0, 0xe0, 0x06, 0x02, 0x02, 0x02, 0x82, 0xc1,
  0x06, 0x06, 0x06, 0x06, 0x07, 0x83, 0x07, 0x07, 0x07, 0x06, 0x07, 0x07,
  0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x1f,
  0xfe, 0xfe, 0xe0, 0xe0, 0xfc, 0xf8, 0x0e, 0x06, 0x80, 0xc0, 0xe0, 0xf0,
  0x0e, 0x06, 0x02, 0x80, 0xc0, 0xe0, 0x0e, 0x06, 0x02, 0x02, 0x82, 0xc1,
  0x0e, 0x06, 0x06, 0x06, 0x07, 0x83, 0x0f, 0x07, 0x07, 0x06, 0x07, 0x07,
  0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x1f,
  0xfc, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xf8, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xf1, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xe3, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xc7, 0xff, 0xff, 0xfd, 0xff, 0xff, 0x8f, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xfd, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xfd, 0xff, 0xff, 0xf8, 0x00, 0xff, 0xfd, 0xff, 0xff,
  0x11, 0x01, 0xff, 0xfd, 0xff, 0xff, 0x03, 0x03, 0xff, 0xfd, 0xff, 0xff,
  0x07, 0x07, 0xff, 0xfd, 0xff, 0xff, 0x0f, 0x0f, 0xff, 0xfd, 0xff, 0xff,
  0x1f, 0x0f, 0xff, 0xfd, 0xff, 0xff, 0x1f, 0x1f, 0xff, 0xfd, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xfd, 0xff, 0xff, 0xf8, 0x00, 0xff, 0xfd, 0xff, 0xff,
  0x01, 0x01, 0x07, 0xfd, 0xff, 0xff, 0x03, 0x03, 0x07, 0xfd, 0xff, 0xff,
  0x07, 0x07, 0x07, 0xfd, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0xfd, 0xff, 0xff,
  0x0f, 0x0f, 0x0f, 0xfd, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xfb, 0xfd, 0xff, 0xff, 0x03, 0x03, 0x07, 0x05, 0xff, 0xff,
  0x07, 0x07, 0x07, 0x05, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0d, 0xff, 0xff,
  0x0f, 0x0f, 0x0f, 0x0d, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0x1d, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xf9, 0xff, 0xff, 0x07, 0x07, 0x07, 0x05, 0x87, 0xff,
  0x07, 0x07, 0x07, 0x05, 0x07, 0xff, 0x0f, 0x0f, 0x0f, 0x0d, 0x0f, 0xff,
  0x0f, 0x0f, 0x0f, 0x0d, 0x0f, 0xff, 0x1f, 0x1f, 0x1f, 0x1d, 0x1f, 0xff,
  0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xfd, 0xfb, 0xff, 0x06, 0x06, 0x06, 0x00, 0x82, 0xc1,
  0x06, 0x06, 0x06, 0x04, 0x07, 0x83, 0x0f, 0x0f, 0x0f, 0x0d, 0x0f, 0x07,
  0x0f, 0x0f, 0x0f, 0x0d, 0x0f, 0x0f, 0x1f, 0x1f, 0x1f, 0x1d, 0x1f, 0x1f,
  0xfe, 0xfe, 0xc0, 0xf8, 0xfc, 0xf8, 0x7e, 0x7e, 0xfe, 0xc0, 0xf8, 0xf0,
  0x0e, 0x06, 0x00, 0x80, 0xc0, 0xe0, 0x0e, 0x06, 0x06, 0x00, 0x82, 0xc1,
  0x0e, 0x06, 0x06, 0x04, 0x07, 0x83, 0x0f, 0x0f, 0x0f, 0x0d, 0x0f, 0x07,
  0x0f, 0x0f, 0x0f, 0x0d, 0x0f, 0x0f, 0x1f, 0x1f, 0x1f, 0x1d, 0x1f, 0x1f,
  0xfe, 0xfe, 0xc0, 0xf8, 0xfc, 0xf8, 0x7e, 0x7e, 0xc0, 0xc0, 0xf8, 0xf0,
  0x1e, 0x0e, 0x00, 0x80, 0xc0, 0xe0, 0x1e, 0x0e, 0x06, 0x00, 0x82, 0xc1,
  0x1e, 0x0e, 0x06, 0x04, 0x07, 0x83, 0x1f, 0x0f, 0x0f, 0x0d, 0x0f, 0x07,
  0x1f, 0x0f, 0x0f, 0x0d, 0x0f, 0x0f, 0x1f, 0x1f, 0x1f, 0x1d, 0x1f, 0x1f,
  0xfc, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xf8, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0xf1, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xe3, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0xc7, 0xff, 0xff, 0xfb, 0xff, 0xff, 0x8f, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xfb, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xfb, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xfb, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xfb, 0xff, 0xff,
  0xf1, 0x00, 0xff, 0xfb, 0xff, 0xff, 0x23, 0x03, 0xff, 0xfb, 0xff, 0xff,
  0x07, 0x07, 0xff, 0xfb, 0xff, 0xff, 0x0f, 0x0f, 0xff, 0xfb, 0xff, 0xff,
  0x1f, 0x1f, 0xff, 0xfb, 0xff, 0xff, 0x3f, 0x1f, 0xff, 0xfb, 0xff, 0xff,
  0xfc, 0xfc, 0xfe, 0xfb, 0xff, 0xff, 0xf8, 0xf8, 0xfe, 0xfb, 0xff, 0xff,
  0xf1, 0x00, 0xff, 0xfb, 0xff, 0xff, 0x03, 0x03, 0x0f, 0xfb, 0xff, 0xff,
  0x07, 0x07, 0x0f, 0xfb, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0xfb, 0xff, 0xff,
  0x1f, 0x1f, 0x1f, 0xfb, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0xfb, 0xff, 0xff,
  0xfc, 0xfc, 0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb, 0xff, 0xff,
  0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff,
  0x07, 0x07, 0x0f, 0x0b, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0b, 0xff, 0xff,
  0x1f, 0x1f, 0x1f, 0x1b, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0x1b, 0xff, 0xff,
  0xfe, 0xfe, 0xfe, 0xfa, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xf9, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xf3, 0xff, 0xff,
  0x0f, 0x0f, 0x0f, 0x0b, 0x0f, 0xff, 0x0f, 0x0f, 0x0f, 0x0b, 0x0f, 0xff,
  0x1f, 0x1f, 0x1f, 0x1b, 0x1f, 0xff, 0x1f, 0x1f, 0x1f, 0x1b, 0x1f, 0xff,
  0xfe, 0xfe, 0xfe, 0xf8, 0xfc, 0xf8, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff,
  0x0e, 0x0e, 0x0e, 0x02, 0x07, 0x83, 0x0f, 0x0f, 0x0f, 0x0b, 0x0f, 0x07,
  0x1f, 0x1f, 0x1f, 0x1b, 0x1f, 0x0f, 0x1f, 0x1f, 0x1f, 0x1b, 0x1f, 0x1f,
  0xfe, 0xfe, 0xfe, 0xf8, 0xfc, 0xf8, 0x7e, 0x7e, 0x80, 0xf0, 0xf8, 0xf0,
  0x7e, 0x7e, 0x7e, 0x80, 0xf0, 0xe0, 0x1e, 0x0e, 0x00, 0x00, 0x82, 0xc1,
  0x1e, 0x0e, 0x0e, 0x02, 0x07, 0x83, 0x1f, 0x0f, 0x0f, 0x0b, 0x0f, 0x07,
  0x1f, 0x1f, 0x1f, 0x1b, 0x1f, 0x0f, 0x1f, 0x1f, 0x1f, 0x1b, 0x1f, 0x1f,
  0xfe, 0xfe, 0xfe, 0xf8, 0xfc, 0xf8, 0x7e, 0x7e, 0x80, 0xf0, 0xf8, 0xf0,
  0x7e, 0x7e, 0x00, 0x80, 0xf0, 0xe0, 0x3e, 0x1e, 0x00, 0x00, 0x82, 0xc1,
  0x3e, 0x1e, 0x0e, 0x02, 0x07, 0x83, 0x3f, 0x1f, 0x0f, 0x0b, 0x0f, 0x07,
  0x3f, 0x1f, 0x1f, 0x1b, 0x1f, 0x0f, 0x3f, 0x1f, 0x1f, 0x1b, 0x1f, 0x1f,
  0xfc, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xf8, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xf1, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xe3, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xc7, 0xff, 0xff, 0xf7, 0xff, 0xff, 0x8f, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xf7, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xf7, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xf7, 0xff, 0xff,
  0xf1, 0xf0, 0xff, 0xf7, 0xff, 0xff, 0xe3, 0x00, 0xff, 0xf7, 0xff, 0xff,
  0x47, 0x07, 0xff, 0xf7, 0xff, 0xff, 0x0f, 0x0f, 0xff, 0xf7, 0xff, 0xff,
  0x1f, 0x1f, 0xff, 0xf7, 0xff, 0xff, 0x3f, 0x3f, 0xff, 0xf7, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xf7, 0xff, 0xff, 0xf8, 0xf8, 0xfc, 0xf7, 0xff, 0xff,
  0xf0, 0xf0, 0xfc, 0xf7, 0xff, 0xff, 0xe3, 0x00, 0xff, 0xf7, 0xff, 0xff,
  0x07, 0x07, 0x1f, 0xf7, 0xff, 0xff, 0x0f, 0x0f, 0x1f, 0xf7, 0xff, 0xff,
  0x1f, 0x1f, 0x1f, 0xf7, 0xff, 0xff, 0x3f, 0x3f, 0x3f, 0xf7, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xf4, 0xff, 0xff, 0xf8, 0xf8, 0xfc, 0xf4, 0xff, 0xff,
  0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xef, 0xf7, 0xff, 0xff, 0x0f, 0x0f, 0x1f, 0x17, 0xff, 0xff,
  0x1f, 0x1f, 0x1f, 0x17, 0xff, 0xff, 0x3f, 0x3f, 0x3f, 0x37, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xf4, 0xfc, 0xff, 0xfc, 0xfc, 0xfc, 0xf4, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xf3, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xe7, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0x17, 0x1f, 0xff,
  0x1f, 0x1f, 0x1f, 0x17, 0x1f, 0xff, 0x3f, 0x3f, 0x3f, 0x37, 0x3f, 0xff,
  0xfc, 0xfc, 0xfc, 0xf4, 0xfc, 0xf8, 0xfc, 0xfc, 0xfc, 0xf0, 0xf8, 0xf0,
  0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff,
  0xff, 0xff, 0xff, 0xf7, 0xef, 0xff, 0x1f, 0x1f, 0x1f, 0x07, 0x0f, 0x07,
  0x1f, 0x1f, 0x1f, 0x17, 0x1f, 0x0f, 0x3f, 0x3f, 0x3f, 0x37, 0x3f, 0x1f,
  0xfe, 0xfc, 0xfc, 0xf4, 0xfc, 0xf8, 0xfe, 0xfc, 0xfc, 0xf0, 0xf8, 0xf0,
  0x7e, 0x7c, 0x00, 0xe0, 0xf0, 0xe0, 0x7e, 0x7e, 0x7e, 0x00, 0xe2, 0xc1,
  0x3e, 0x1e, 0x00, 0x02, 0x07, 0x83, 0x3f, 0x1f, 0x1f, 0x07, 0x0f, 0x07,
  0x3f, 0x1f, 0x1f, 0x17, 0x1f, 0x0f, 0x3f, 0x3f, 0x3f, 0x37, 0x3f, 0x1f,
  0xfe, 0xfe, 0xfc, 0xf4, 0xfc, 0xf8, 0xfe, 0xfe, 0xfc, 0xf0, 0xf8, 0xf0,
  0x7e, 0x7e, 0x00, 0xe0, 0xf0, 0xe0, 0x7e, 0x7e, 0x00, 0x00, 0xe2, 0xc1,
  0x7e, 0x3e, 0x00, 0x02, 0x07, 0x83, 0x7f, 0x3f, 0x1f, 0x07, 0x0f, 0x07,
  0x7f, 0x3f, 0x1f, 0x17, 0x1f, 0x0f, 0x7f, 0x3f, 0x3f, 0x37, 0x3f, 0x1f,
  0xfc, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xf8, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0xf1, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xe3, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0xc7, 0xff, 0xff, 0xff, 0xfe, 0xff, 0x8f, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xfe, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xfe, 0xff,
  0xfc, 0x80, 0xff, 0xff, 0xfe, 0xff, 0x08, 0x00, 0xff, 0xff, 0xfe, 0xff,
  0x01, 0x00, 0xff, 0xff, 0xfe, 0xff, 0x03, 0x01, 0xff, 0xff, 0xfe, 0xff,
  0x07, 0x03, 0xff, 0xff, 0xfe, 0xff, 0x0f, 0x07, 0xff, 0xff, 0xfe, 0xff,
  0x0f, 0x0f, 0xff, 0xff, 0xfe, 0xff, 0x0f, 0x0f, 0xff, 0xff, 0xfe, 0xff,
  0x80, 0x80, 0xc0, 0xff, 0xfe, 0xff, 0x00, 0x00, 0x80, 0xff, 0xfe, 0xff,
  0x00, 0x00, 0x01, 0xff, 0xfe, 0xff, 0x00, 0x00, 0x03, 0xff, 0xfe, 0xff,
  0x03, 0x03, 0x07, 0xff, 0xfe, 0xff, 0x07, 0x07, 0x07, 0xff, 0xfe, 0xff,
  0x0f, 0x0f, 0x0f, 0xff, 0xfe, 0xff, 0x0f, 0x0f, 0x0f, 0xff, 0xfe, 0xff,
  0x80, 0x80, 0xc0, 0xff, 0xfe, 0xff, 0x00, 0x00, 0x80, 0xc3, 0xfe, 0xff,
  0x00, 0x00, 0x00, 0x83, 0xfe, 0xff, 0x00, 0x00, 0x02, 0x03, 0xfe, 0xff,
  0x03, 0x03, 0x07, 0x07, 0xfe, 0xff, 0x07, 0x07, 0x07, 0x07, 0xfe, 0xff,
  0x0f, 0x0f, 0x0f, 0x0f, 0xfe, 0xff, 0x0f, 0x0f, 0x0f, 0x0f, 0xfe, 0xff,
  0xff, 0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff,
  0x00, 0x00, 0x00, 0x82, 0xc2, 0xff, 0x00, 0x02, 0x02, 0x02, 0x82, 0xff,
  0x03, 0x07, 0x07, 0x07, 0x06, 0xff, 0x07, 0x07, 0x07, 0x07, 0x06, 0xff,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0xff, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xff,
  0x02, 0x02, 0x02, 0x82, 0xc2, 0xe3, 0x02, 0x02, 0x02, 0x02, 0x82, 0xc3,
  0x07, 0x07, 0x07, 0x07, 0x06, 0x87, 0x07, 0x07, 0x07, 0x07, 0x06, 0x07,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x1f,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd,
  0x02, 0x02, 0x02, 0x82, 0xc2, 0xe1, 0x02, 0x02, 0x02, 0x02, 0x82, 0xc3,
  0x07, 0x07, 0x07, 0x07, 0x06, 0x87, 0x07, 0x07, 0x07, 0x07, 0x06, 0x07,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x1f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xf8, 0x0e, 0x06, 0x82, 0xc2, 0xe0, 0xf2,
  0x0e, 0x06, 0x02, 0x82, 0xc2, 0xe1, 0x0e, 0x06, 0x02, 0x02, 0x82, 0xc3,
  0x0f, 0x07, 0x07, 0x07, 0x06, 0x87, 0x0f, 0x07, 0x07, 0x07, 0x06, 0x07,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x1f,
  0xfc, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xf8, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xf1, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xe3, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xc7, 0xff, 0xff, 0xff, 0xfd, 0xff, 0x8f, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xfd, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xfc, 0xf8, 0xff, 0xff, 0xfd, 0xff, 0xf8, 0x00, 0xff, 0xff, 0xfd, 0xff,
  0x11, 0x00, 0xff, 0xff, 0xfd, 0xff, 0x03, 0x01, 0xff, 0xff, 0xfd, 0xff,
  0x07, 0x03, 0xff, 0xff, 0xfd, 0xff, 0x0f, 0x07, 0xff, 0xff, 0xfd, 0xff,
  0x1f, 0x0f, 0xff, 0xff, 0xfd, 0xff, 0x1f, 0x1f, 0xff, 0xff, 0xfd, 0xff,
  0x80, 0x80, 0xfc, 0xff, 0xfd, 0xff, 0x00, 0x00, 0x80, 0xff, 0xfd, 0xff,
  0x00, 0x00, 0x01, 0xff, 0xfd, 0xff, 0x01, 0x01, 0x03, 0xff, 0xfd, 0xff,
  0x03, 0x03, 0x07, 0xff, 0xfd, 0xff, 0x07, 0x07, 0x0f, 0xff, 0xfd, 0xff,
  0x0f, 0x0f, 0x0f, 0xff, 0xfd, 0xff, 0x1f, 0x1f, 0x1f, 0xff, 0xfd, 0xff,
  0x80, 0x80, 0xfc, 0xff, 0xfd, 0xff, 0x00, 0x00, 0x80, 0xff, 0xfd, 0xff,
  0x00, 0x00, 0x01, 0x87, 0xfd, 0xff, 0x01, 0x01, 0x03, 0x07, 0xfd, 0xff,
  0x03, 0x03, 0x07, 0x07, 0xfd, 0xff, 0x07, 0x07, 0x0f, 0x0f, 0xfd, 0xff,
  0x0f, 0x0f, 0x0f, 0x0f, 0xfd, 0xff, 0x1f, 0x1f, 0x1f, 0x1f, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff, 0x01, 0x03, 0x03, 0x07, 0x85, 0xff,
  0x03, 0x07, 0x07, 0x07, 0x05, 0xff, 0x07, 0x0f, 0x0f, 0x0f, 0x0d, 0xff,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0d, 0xff, 0x1f, 0x1f, 0x1f, 0x1f, 0x1d, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xf9, 0xff, 0x07, 0x07, 0x07, 0x07, 0x85, 0xc7,
  0x07, 0x07, 0x07, 0x07, 0x05, 0x87, 0x0f, 0x0f, 0x0f, 0x0f, 0x0d, 0x0f,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0d, 0x0f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1d, 0x1f,
  0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb, 0x06, 0x06, 0x06, 0x06, 0x85, 0xc3,
  0x07, 0x07, 0x07, 0x07, 0x05, 0x87, 0x0f, 0x0f, 0x0f, 0x0f, 0x0d, 0x0f,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0d, 0x0f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1d, 0x1f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xf8, 0xfa, 0x7e, 0x7e, 0xfe, 0xfe, 0xfc, 0xf8,
  0x1e, 0x0e, 0x06, 0x86, 0xc0, 0xe7, 0x1e, 0x0e, 0x06, 0x06, 0x85, 0xc3,
  0x1f, 0x0f, 0x07, 0x07, 0x05, 0x87, 0x1f, 0x0f, 0x0f, 0x0f, 0x0d, 0x0f,
  0x1f, 0x0f, 0x0f, 0x0f, 0x0d, 0x0f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1d, 0x1f,
  0xfc, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xf8, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xf1, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xe3, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xc7, 0xff, 0xff, 0xff, 0xfb, 0xff, 0x8f, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xfb, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xfc, 0xf8, 0xff, 0xff, 0xfb, 0xff, 0xf8, 0xf0, 0xff, 0xff, 0xfb, 0xff,
  0xf1, 0x00, 0xff, 0xff, 0xfb, 0xff, 0x23, 0x01, 0xff, 0xff, 0xfb, 0xff,
  0x07, 0x03, 0xff, 0xff, 0xfb, 0xff, 0x0f, 0x07, 0xff, 0xff, 0xfb, 0xff,
  0x1f, 0x0f, 0xff, 0xff, 0xfb, 0xff, 0x3f, 0x1f, 0xff, 0xff, 0xfb, 0xff,
  0xf8, 0xf8, 0xfc, 0xff, 0xfb, 0xff, 0x00, 0x00, 0xf8, 0xff, 0xfb, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xfb, 0xff, 0x00, 0x00, 0x03, 0xff, 0xfb, 0xff,
  0x03, 0x03, 0x07, 0xff, 0xfb, 0xff, 0x07, 0x07, 0x0f, 0xff, 0xfb, 0xff,
  0x0f, 0x0f, 0x1f, 0xff, 0xfb, 0xff, 0x1f, 0x1f, 0x1f, 0xff, 0xfb, 0xff,
  0xf8, 0xf8, 0xfc, 0xfe, 0xfb, 0xff, 0x00, 0x00, 0xf8, 0xfe, 0xfb, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xfb, 0xff, 0x00, 0x00, 0x03, 0x0f, 0xfb, 0xff,
  0x03, 0x03, 0x07, 0x0f, 0xfb, 0xff, 0x07, 0x07, 0x0f, 0x0f, 0xfb, 0xff,
  0x0f, 0x0f, 0x1f, 0x1f, 0xfb, 0xff, 0x1f, 0x1f, 0x1f, 0x1f, 0xfb, 0xff,
  0xf8, 0xfc, 0xfc, 0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff,
  0x03, 0x07, 0x07, 0x0f, 0x0b, 0xff, 0x07, 0x0f, 0x0f, 0x0f, 0x0b, 0xff,
  0x0f, 0x1f, 0x1f, 0x1f, 0x1b, 0xff, 0x1f, 0x1f, 0x1f, 0x1f, 0x1b, 0xff,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfa, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xf9, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xf3, 0xff,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0b, 0x8f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0b, 0x0f,
  0x1f, 0x1f, 0x1f, 0x1f, 0x1b, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1b, 0x1f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xf8, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0b, 0x87, 0x0f, 0x0f, 0x0f, 0x0f, 0x0b, 0x0f,
  0x1f, 0x1f, 0x1f, 0x1f, 0x1b, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1b, 0x1f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfa, 0xfc, 0x7e, 0x7e, 0xfe, 0xfe, 0xf0, 0xfe,
  0x7e, 0x7e, 0x7e, 0xfe, 0xfa, 0xf1, 0x3e, 0x1e, 0x0e, 0x0e, 0x81, 0xcf,
  0x3f, 0x1f, 0x0f, 0x0f, 0x0b, 0x87, 0x3f, 0x1f, 0x0f, 0x0f, 0x0b, 0x0f,
  0x3f, 0x1f, 0x1f, 0x1f, 0x1b, 0x1f, 0x3f, 0x1f, 0x1f, 0x1f, 0x1b, 0x1f,
  0xfc, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xf8, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0xf1, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xe3, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0xc7, 0xff, 0xff, 0xff, 0xf7, 0xff, 0x8f, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xf7, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xf7, 0xff,
  0xfc, 0xf8, 0xff, 0xff, 0xf7, 0xff, 0xf8, 0xf0, 0xff, 0xff, 0xf7, 0xff,
  0xf1, 0xe0, 0xff, 0xff, 0xf7, 0xff, 0xe3, 0x00, 0xff, 0xff, 0xf7, 0xff,
  0x47, 0x03, 0xff, 0xff, 0xf7, 0xff, 0x0f, 0x07, 0xff, 0xff, 0xf7, 0xff,
  0x1f, 0x0f, 0xff, 0xff, 0xf7, 0xff, 0x3f, 0x1f, 0xff, 0xff, 0xf7, 0xff,
  0xf8, 0xf8, 0xfc, 0xff, 0xf7, 0xff, 0xf0, 0xf0, 0xf8, 0xff, 0xf7, 0xff,
  0x00, 0x00, 0xf0, 0xff, 0xf7, 0xff, 0x00, 0x00, 0x00, 0xff, 0xf7, 0xff,
  0x00, 0x00, 0x07, 0xff, 0xf7, 0xff, 0x07, 0x07, 0x0f, 0xff, 0xf7, 0xff,
  0x0f, 0x0f, 0x1f, 0xff, 0xf7, 0xff, 0x1f, 0x1f, 0x3f, 0xff, 0xf7, 0xff,
  0xf8, 0xf8, 0xfc, 0xfc, 0xf7, 0xff, 0xf0, 0xf0, 0xf8, 0xfc, 0xf7, 0xff,
  0x00, 0x00, 0xf0, 0xfc, 0xf7, 0xff, 0x00, 0x00, 0x00, 0xff, 0xf7, 0xff,
  0x00, 0x00, 0x07, 0x1f, 0xf7, 0xff, 0x07, 0x07, 0x0f, 0x1f, 0xf7, 0xff,
  0x0f, 0x0f, 0x1f, 0x1f, 0xf7, 0xff, 0x1f, 0x1f, 0x3f, 0x3f, 0xf7, 0xff,
  0xf8, 0xfc, 0xfc, 0xfc, 0xf4, 0xff, 0xf0, 0xf8, 0xf8, 0xfc, 0xf4, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7, 0xff,
  0xff, 0xff, 0xff, 0xef, 0xf7, 0xff, 0x07, 0x0f, 0x0f, 0x1f, 0x17, 0xff,
  0x0f, 0x1f, 0x1f, 0x1f, 0x17, 0xff, 0x1f, 0x3f, 0x3f, 0x3f, 0x37, 0xff,
  0xfc, 0xfc, 0xfc, 0xfc, 0xf4, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xf4, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xf3, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xe7, 0xff, 0x1f, 0x1f, 0x1f, 0x1f, 0x17, 0x1f,
  0x1f, 0x1f, 0x1f, 0x1f, 0x17, 0x1f, 0x3f, 0x3f, 0x3f, 0x3f, 0x37, 0x3f,
  0xfc, 0xfc, 0xfc, 0xfc, 0xf4, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xf4, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xef, 0x1f, 0x1f, 0x1f, 0x1f, 0x17, 0x0f,
  0x1f, 0x1f, 0x1f, 0x1f, 0x17, 0x1f, 0x3f, 0x3f, 0x3f, 0x3f, 0x37, 0x3f,
  0xfe, 0xfe, 0xfc, 0xfc, 0xf4, 0xfc, 0xfe, 0xfe, 0xfc, 0xfc, 0xf4, 0xf8,
  0x7e, 0x7e, 0x7c, 0xfc, 0xe0, 0xfc, 0x7e, 0x7e, 0x7e, 0x7e, 0xf6, 0xe3,
  0x7e, 0x3e, 0x1e, 0x1e, 0x03, 0x9f, 0x7f, 0x3f, 0x1f, 0x1f, 0x17, 0x0f,
  0x7f, 0x3f, 0x1f, 0x1f, 0x17, 0x1f, 0x7f, 0x3f, 0x3f, 0x3f, 0x37, 0x3f,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xf1, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xc7, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x8f, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0x80, 0x80, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0xff, 0xff, 0xff, 0xfe,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0xff, 0xff, 0xff, 0xfe,
  0x01, 0x01, 0xff, 0xff, 0xff, 0xfe, 0x03, 0x03, 0xff, 0xff, 0xff, 0xfe,
  0x07, 0x07, 0xff, 0xff, 0xff, 0xfe, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xfe,
  0x80, 0x80, 0xc0, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x80, 0xff, 0xff, 0xfe,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x01, 0xff, 0xff, 0xfe,
  0x01, 0x01, 0x03, 0xff, 0xff, 0xfe, 0x03, 0x03, 0x07, 0xff, 0xff, 0xfe,
  0x07, 0x07, 0x0f, 0xff, 0xff, 0xfe, 0x0f, 0x0f, 0x0f, 0xff, 0xff, 0xfe,
  0x80, 0x80, 0xc0, 0xe0, 0xff, 0xfe, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xfe,
  0x00, 0x00, 0x00, 0x81, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x03, 0xff, 0xfe,
  0x01, 0x01, 0x03, 0x07, 0xff, 0xfe, 0x03, 0x03, 0x07, 0x07, 0xff, 0xfe,
  0x07, 0x07, 0x0f, 0x0f, 0xff, 0xfe, 0x0f, 0x0f, 0x0f, 0x0f, 0xff, 0xfe,
  0x80, 0x80, 0xc0, 0xe0, 0xff, 0xfe, 0x00, 0x00, 0x80, 0xc0, 0xe3, 0xfe,
  0x00, 0x00, 0x00, 0x80, 0xc3, 0xfe, 0x00, 0x00, 0x00, 0x02, 0x83, 0xfe,
  0x01, 0x01, 0x03, 0x07, 0x07, 0xfe, 0x03, 0x03, 0x07, 0x07, 0x07, 0xfe,
  0x07, 0x07, 0x0f, 0x0f, 0x0f, 0xfe, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe,
  0x00, 0x00, 0x00, 0x80, 0xc2, 0xe2, 0x00, 0x00, 0x02, 0x02, 0x82, 0xc2,
  0x01, 0x03, 0x07, 0x07, 0x07, 0x86, 0x03, 0x07, 0x07, 0x07, 0x07, 0x06,
  0x07, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1e,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0x02, 0x02, 0x02, 0x82, 0xc2, 0xe2, 0x02, 0x02, 0x02, 0x02, 0x82, 0xc2,
  0x07, 0x07, 0x07, 0x07, 0x07, 0x86, 0x07, 0x07, 0x07, 0x07, 0x07, 0x06,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1e,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0x02, 0x02, 0x02, 0x82, 0xc2, 0xe2, 0x02, 0x02, 0x02, 0x02, 0x82, 0xc2,
  0x07, 0x07, 0x07, 0x07, 0x07, 0x86, 0x07, 0x07, 0x07, 0x07, 0x07, 0x06,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1e,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xf1, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xc7, 0xff, 0xff, 0xff, 0xff, 0xfd, 0x8f, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xfd, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0x80, 0x80, 0xff, 0xff, 0xff, 0xfd, 0x00, 0x00, 0xff, 0xff, 0xff, 0xfd,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xfd, 0x00, 0x00, 0xff, 0xff, 0xff, 0xfd,
  0x01, 0x01, 0xff, 0xff, 0xff, 0xfd, 0x03, 0x03, 0xff, 0xff, 0xff, 0xfd,
  0x07, 0x07, 0xff, 0xff, 0xff, 0xfd, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xfd,
  0x80, 0x80, 0xf8, 0xff, 0xff, 0xfd, 0x00, 0x00, 0x80, 0xff, 0xff, 0xfd,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xfd, 0x00, 0x00, 0x01, 0xff, 0xff, 0xfd,
  0x01, 0x01, 0x03, 0xff, 0xff, 0xfd, 0x03, 0x03, 0x07, 0xff, 0xff, 0xfd,
  0x07, 0x07, 0x0f, 0xff, 0xff, 0xfd, 0x0f, 0x0f, 0x1f, 0xff, 0xff, 0xfd,
  0x80, 0x80, 0xc0, 0xfc, 0xff, 0xfd, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xfd,
  0x00, 0x00, 0x00, 0x81, 0xff, 0xfd, 0x00, 0x00, 0x01, 0x03, 0xff, 0xfd,
  0x01, 0x01, 0x03, 0x07, 0xff, 0xfd, 0x03, 0x03, 0x07, 0x0f, 0xff, 0xfd,
  0x07, 0x07, 0x0f, 0x0f, 0xff, 0xfd, 0x0f, 0x0f, 0x1f, 0x1f, 0xff, 0xfd,
  0x80, 0x80, 0xc0, 0xfc, 0xff, 0xfd, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xfd,
  0x00, 0x00, 0x00, 0x81, 0xc7, 0xfd, 0x00, 0x00, 0x01, 0x03, 0x87, 0xfd,
  0x01, 0x01, 0x03, 0x07, 0x07, 0xfd, 0x03, 0x03, 0x07, 0x0f, 0x0f, 0xfd,
  0x07, 0x07, 0x0f, 0x0f, 0x0f, 0xfd, 0x0f, 0x0f, 0x1f, 0x1f, 0x1f, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0x00, 0x00, 0x03, 0x03, 0x87, 0xc5,
  0x01, 0x03, 0x07, 0x07, 0x07, 0x85, 0x03, 0x07, 0x0f, 0x0f, 0x0f, 0x0d,
  0x07, 0x0f, 0x0f, 0x0f, 0x0f, 0x0d, 0x0f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1d,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf9, 0x07, 0x07, 0x07, 0x07, 0x87, 0xc5,
  0x07, 0x07, 0x07, 0x07, 0x07, 0x85, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0d,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0d, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1d,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0x07, 0x07, 0x07, 0x07, 0x87, 0xc5,
  0x07, 0x07, 0x07, 0x07, 0x07, 0x85, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0d,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0d, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1d,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xf1, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xc7, 0xff, 0xff, 0xff, 0xff, 0xfb, 0x8f, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xfb, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xf0, 0xf0, 0xff, 0xff, 0xff, 0xfb, 0x00, 0x00, 0xff, 0xff, 0xff, 0xfb,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xfb, 0x00, 0x00, 0xff, 0xff, 0xff, 0xfb,
  0x01, 0x01, 0xff, 0xff, 0xff, 0xfb, 0x03, 0x03, 0xff, 0xff, 0xff, 0xfb,
  0x07, 0x07, 0xff, 0xff, 0xff, 0xfb, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xfb,
  0xf0, 0xf0, 0xf8, 0xff, 0xff, 0xfb, 0x00, 0x00, 0xf0, 0xff, 0xff, 0xfb,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xfb, 0x00, 0x00, 0x01, 0xff, 0xff, 0xfb,
  0x01, 0x01, 0x03, 0xff, 0xff, 0xfb, 0x03, 0x03, 0x07, 0xff, 0xff, 0xfb,
  0x07, 0x07, 0x0f, 0xff, 0xff, 0xfb, 0x0f, 0x0f, 0x1f, 0xff, 0xff, 0xfb,
  0xf0, 0xf0, 0xf8, 0xfc, 0xff, 0xfb, 0x00, 0x00, 0x80, 0xf8, 0xff, 0xfb,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xfb, 0x00, 0x00, 0x00, 0x03, 0xff, 0xfb,
  0x01, 0x01, 0x03, 0x07, 0xff, 0xfb, 0x03, 0x03, 0x07, 0x0f, 0xff, 0xfb,
  0x07, 0x07, 0x0f, 0x1f, 0xff, 0xfb, 0x0f, 0x0f, 0x1f, 0x1f, 0xff, 0xfb,
  0xf0, 0xf0, 0xf8, 0xfc, 0xfe, 0xfb, 0x00, 0x00, 0x80, 0xf8, 0xfe, 0xfb,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xfb, 0x00, 0x00, 0x00, 0x03, 0x8f, 0xfb,
  0x01, 0x01, 0x03, 0x07, 0x0f, 0xfb, 0x03, 0x03, 0x07, 0x0f, 0x0f, 0xfb,
  0x07, 0x07, 0x0f, 0x1f, 0x1f, 0xfb, 0x0f, 0x0f, 0x1f, 0x1f, 0x1f, 0xfb,
  0xf0, 0xf8, 0xfc, 0xfc, 0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfb,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb,
  0x01, 0x03, 0x07, 0x07, 0x0f, 0x8b, 0x03, 0x07, 0x0f, 0x0f, 0x0f, 0x0b,
  0x07, 0x0f, 0x1f, 0x1f, 0x1f, 0x1b, 0x0f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1b,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf9,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf3,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x8b, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0b,
  0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1b, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1b,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfc, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x8b, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0b,
  0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1b, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1b,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xf1, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xc7, 0xff, 0xff, 0xff, 0xff, 0xf7, 0x8f, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xf7, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xf0, 0xf0, 0xff, 0xff, 0xff, 0xf7, 0xe0, 0xe0, 0xff, 0xff, 0xff, 0xf7,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xf7, 0x00, 0x00, 0xff, 0xff, 0xff, 0xf7,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xf7, 0x03, 0x03, 0xff, 0xff, 0xff, 0xf7,
  0x07, 0x07, 0xff, 0xff, 0xff, 0xf7, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xf7,
  0xf0, 0xf0, 0xf8, 0xff, 0xff, 0xf7, 0xe0, 0xe0, 0xf0, 0xff, 0xff, 0xf7,
  0x00, 0x00, 0xe0, 0xff, 0xff, 0xf7, 0x00, 0x00, 0x00, 0xff, 0xff, 0xf7,
  0x00, 0x00, 0x03, 0xff, 0xff, 0xf7, 0x03, 0x03, 0x07, 0xff, 0xff, 0xf7,
  0x07, 0x07, 0x0f, 0xff, 0xff, 0xf7, 0x0f, 0x0f, 0x1f, 0xff, 0xff, 0xf7,
  0xf0, 0xf0, 0xf8, 0xfc, 0xff, 0xf7, 0xe0, 0xe0, 0xf0, 0xf8, 0xff, 0xf7,
  0x00, 0x00, 0x00, 0xf0, 0xff, 0xf7, 0x00, 0x00, 0x00, 0x00, 0xff, 0xf7,
  0x00, 0x00, 0x00, 0x07, 0xff, 0xf7, 0x03, 0x03, 0x07, 0x0f, 0xff, 0xf7,
  0x07, 0x07, 0x0f, 0x1f, 0xff, 0xf7, 0x0f, 0x0f, 0x1f, 0x3f, 0xff, 0xf7,
  0xf0, 0xf0, 0xf8, 0xfc, 0xfc, 0xf7, 0xe0, 0xe0, 0xf0, 0xf8, 0xfc, 0xf7,
  0x00, 0x00, 0x00, 0xf0, 0xfc, 0xf7, 0x00, 0x00, 0x00, 0x00, 0xff, 0xf7,
  0x00, 0x00, 0x00, 0x07, 0x1f, 0xf7, 0x03, 0x03, 0x07, 0x0f, 0x1f, 0xf7,
  0x07, 0x07, 0x0f, 0x1f, 0x1f, 0xf7, 0x0f, 0x0f, 0x1f, 0x3f, 0x3f, 0xf7,
  0xf0, 0xf8, 0xfc, 0xfc, 0xfc, 0xf4, 0xe0, 0xf0, 0xf8, 0xf8, 0xfc, 0xf4,
  0xff, 0xff, 0xff, 0xff, 0xfb, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xef, 0xf7, 0x03, 0x07, 0x0f, 0x0f, 0x1f, 0x17,
  0x07, 0x0f, 0x1f, 0x1f, 0x1f, 0x17, 0x0f, 0x1f, 0x3f, 0x3f, 0x3f, 0x37,
  0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xf4, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xf4,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe7, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x17,
  0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x17, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x37,
  0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xf4, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xf4,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x17,
  0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x17, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x37,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xc7, 0xff, 0xff, 0xff, 0xff, 0xff, 0x8f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff,
  0x03, 0x03, 0xff, 0xff, 0xff, 0xff, 0x07, 0x07, 0xff, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xff, 0xff, 0xff, 0x00, 0x00, 0x80, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x01, 0xff, 0xff, 0xff, 0x01, 0x01, 0x03, 0xff, 0xff, 0xff,
  0x03, 0x03, 0x07, 0xff, 0xff, 0xff, 0x07, 0x07, 0x0f, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xff, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0x00, 0x00, 0x00, 0x01, 0xff, 0xff,
  0x00, 0x00, 0x01, 0x03, 0xff, 0xff, 0x01, 0x01, 0x03, 0x07, 0xff, 0xff,
  0x03, 0x03, 0x07, 0x0f, 0xff, 0xff, 0x07, 0x07, 0x0f, 0x0f, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x83, 0xff,
  0x00, 0x00, 0x01, 0x03, 0x07, 0xff, 0x01, 0x01, 0x03, 0x07, 0x07, 0xff,
  0x03, 0x03, 0x07, 0x0f, 0x0f, 0xff, 0x07, 0x07, 0x0f, 0x0f, 0x0f, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xf0, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xf2,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe2, 0x00, 0x00, 0x00, 0x00, 0x82, 0xc2,
  0x00, 0x00, 0x01, 0x03, 0x07, 0x87, 0x01, 0x01, 0x03, 0x07, 0x07, 0x07,
  0x03, 0x03, 0x07, 0x0f, 0x0f, 0x0f, 0x07, 0x07, 0x0f, 0x0f, 0x0f, 0x1f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xe2, 0x00, 0x00, 0x00, 0x02, 0x82, 0xc2,
  0x00, 0x00, 0x03, 0x07, 0x07, 0x87, 0x01, 0x03, 0x07, 0x07, 0x07, 0x07,
  0x03, 0x07, 0x0f, 0x0f, 0x0f, 0x0f, 0x07, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x02, 0x02, 0x02, 0x82, 0xc2, 0xe2, 0x02, 0x02, 0x02, 0x02, 0x82, 0xc2,
  0x07, 0x07, 0x07, 0x07, 0x07, 0x87, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xc7, 0xff, 0xff, 0xff, 0xff, 0xff, 0x8f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff,
  0x03, 0x03, 0xff, 0xff, 0xff, 0xff, 0x07, 0x07, 0xff, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xff, 0xff, 0xff, 0x00, 0x00, 0x80, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x01, 0xff, 0xff, 0xff, 0x01, 0x01, 0x03, 0xff, 0xff, 0xff,
  0x03, 0x03, 0x07, 0xff, 0xff, 0xff, 0x07, 0x07, 0x0f, 0xff, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xf8, 0xff, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0x00, 0x00, 0x00, 0x01, 0xff, 0xff,
  0x00, 0x00, 0x01, 0x03, 0xff, 0xff, 0x01, 0x01, 0x03, 0x07, 0xff, 0xff,
  0x03, 0x03, 0x07, 0x0f, 0xff, 0xff, 0x07, 0x07, 0x0f, 0x1f, 0xff, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xfc, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc1, 0xff, 0x00, 0x00, 0x00, 0x01, 0x83, 0xff,
  0x00, 0x00, 0x01, 0x03, 0x07, 0xff, 0x01, 0x01, 0x03, 0x07, 0x0f, 0xff,
  0x03, 0x03, 0x07, 0x0f, 0x0f, 0xff, 0x07, 0x07, 0x0f, 0x1f, 0x1f, 0xff,
  0x80, 0x80, 0xc0, 0xe0, 0xfc, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc1, 0xe7, 0x00, 0x00, 0x00, 0x01, 0x83, 0xc7,
  0x00, 0x00, 0x01, 0x03, 0x07, 0x87, 0x01, 0x01, 0x03, 0x07, 0x0f, 0x0f,
  0x03, 0x03, 0x07, 0x0f, 0x0f, 0x0f, 0x07, 0x07, 0x0f, 0x1f, 0x1f, 0x1f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0x00, 0x00, 0x00, 0x03, 0x83, 0xc7,
  0x00, 0x00, 0x03, 0x07, 0x07, 0x87, 0x01, 0x03, 0x07, 0x0f, 0x0f, 0x0f,
  0x03, 0x07, 0x0f, 0x0f, 0x0f, 0x0f, 0x07, 0x0f, 0x1f, 0x1f, 0x1f, 0x1f,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x07, 0x07, 0x07, 0x07, 0x87, 0xc7,
  0x07, 0x07, 0x07, 0x07, 0x07, 0x87, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xc7, 0xff, 0xff, 0xff, 0xff, 0xff, 0x8f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe0, 0xe0, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff,
  0x03, 0x03, 0xff, 0xff, 0xff, 0xff, 0x07, 0x07, 0xff, 0xff, 0xff, 0xff,
  0xe0, 0xe0, 0xf0, 0xff, 0xff, 0xff, 0x00, 0x00, 0x80, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x01, 0xff, 0xff, 0xff, 0x01, 0x01, 0x03, 0xff, 0xff, 0xff,
  0x03, 0x03, 0x07, 0xff, 0xff, 0xff, 0x07, 0x07, 0x0f, 0xff, 0xff, 0xff,
  0xe0, 0xe0, 0xf0, 0xf8, 0xff, 0xff, 0x00, 0x00, 0x80, 0xf0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0x00, 0x00, 0x00, 0x01, 0xff, 0xff,
  0x00, 0x00, 0x01, 0x03, 0xff, 0xff, 0x01, 0x01, 0x03, 0x07, 0xff, 0xff,
  0x03, 0x03, 0x07, 0x0f, 0xff, 0xff, 0x07, 0x07, 0x0f, 0x1f, 0xff, 0xff,
  0xe0, 0xe0, 0xf0, 0xf8, 0xfc, 0xff, 0x00, 0x00, 0x80, 0xc0, 0xf8, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x83, 0xff,
  0x00, 0x00, 0x01, 0x03, 0x07, 0xff, 0x01, 0x01, 0x03, 0x07, 0x0f, 0xff,
  0x03, 0x03, 0x07, 0x0f, 0x1f, 0xff, 0x07, 0x07, 0x0f, 0x1f, 0x1f, 0xff,
  0xe0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe, 0x00, 0x00, 0x80, 0xc0, 0xf8, 0xfe,
  0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x83, 0xcf,
  0x00, 0x00, 0x01, 0x03, 0x07, 0x8f, 0x01, 0x01, 0x03, 0x07, 0x0f, 0x0f,
  0x03, 0x03, 0x07, 0x0f, 0x1f, 0x1f, 0x07, 0x07, 0x0f, 0x1f, 0x1f, 0x1f,
  0xe0, 0xf0, 0xf8, 0xfc, 0xfc, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0x00, 0x00, 0x03, 0x07, 0x07, 0x8f, 0x01, 0x03, 0x07, 0x0f, 0x0f, 0x0f,
  0x03, 0x07, 0x0f, 0x1f, 0x1f, 0x1f, 0x07, 0x0f, 0x1f, 0x1f, 0x1f, 0x1f,
  0xfe, 0xfe, 0xfe, 0xfe, 0xfc, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x8f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
  0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xc7, 0xff, 0xff, 0xff, 0xff, 0xff, 0x8f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe0, 0xe0, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
  0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff,
  0x03, 0x03, 0xff, 0xff, 0xff, 0xff, 0x07, 0x07, 0xff, 0xff, 0xff, 0xff,
  0xe0, 0xe0, 0xf0, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xe0, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x01, 0x01, 0x03, 0xff, 0xff, 0xff,
  0x03, 0x03, 0x07, 0xff, 0xff, 0xff, 0x07, 0x07, 0x0f, 0xff, 0xff, 0xff,
  0xe0, 0xe0, 0xf0, 0xf8, 0xff, 0xff, 0xc0, 0xc0, 0xe0, 0xf0, 0xff, 0xff,
  0x00, 0x00, 0x00, 0xe0, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x03, 0xff, 0xff, 0x01, 0x01, 0x03, 0x07, 0xff, 0xff,
  0x03, 0x03, 0x07, 0x0f, 0xff, 0xff, 0x07, 0x07, 0x0f, 0x1f, 0xff, 0xff,
  0xe0, 0xe0, 0xf0, 0xf8, 0xfc, 0xff, 0xc0, 0xc0, 0xe0, 0xf0, 0xf8, 0xff,
  0x00, 0x00, 0x00, 0x80, 0xf0, 0xff, 0x00, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x07, 0xff, 0x01, 0x01, 0x03, 0x07, 0x0f, 0xff,
  0x03, 0x03, 0x07, 0x0f, 0x1f, 0xff, 0x07, 0x07, 0x0f, 0x1f, 0x3f, 0xff,
  0xe0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfc, 0xc0, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc,
  0x00, 0x00, 0x00, 0x80, 0xf0, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x80, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x07, 0x9f, 0x01, 0x01, 0x03, 0x07, 0x0f, 0x1f,
  0x03, 0x03, 0x07, 0x0f, 0x1f, 0x1f, 0x07, 0x07, 0x0f, 0x1f, 0x3f, 0x3f,
  0xe0, 0xf0, 0xf8, 0xfc, 0xfc, 0xfc, 0xc0, 0xe0, 0xf0, 0xf8, 0xf8, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0x01, 0x03, 0x07, 0x0f, 0x0f, 0x1f,
  0x03, 0x07, 0x0f, 0x1f, 0x1f, 0x1f, 0x07, 0x0f, 0x1f, 0x3f, 0x3f, 0x3f,
  0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f,
  0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f
};
